@@ -40,6 +40,7 @@
 #include "address-book.h"
 #include "address-book-ldap.h"
 #include "information.h"
+#include "libbalsa-conf.h"
 
 static const int DEBUG_LDAP = 0;
 /* don't search when prefix has length shorter than LDAP_MIN_LEN */
@@ -747,11 +748,11 @@ libbalsa_address_book_ldap_save_config(LibBalsaAddressBook * ab,
 
     ldap = LIBBALSA_ADDRESS_BOOK_LDAP(ab);
 
-    gnome_config_set_string("Host", ldap->host);
-    if(ldap->base_dn) gnome_config_set_string("BaseDN", ldap->base_dn);
-    if(ldap->bind_dn) gnome_config_private_set_string("BindDN", ldap->bind_dn);
-    if(ldap->passwd)  gnome_config_private_set_string("Passwd", ldap->passwd);
-    gnome_config_set_bool("EnableTLS", ldap->enable_tls);
+    libbalsa_conf_set_string("Host", ldap->host);
+    if(ldap->base_dn) libbalsa_conf_set_string("BaseDN", ldap->base_dn);
+    if(ldap->bind_dn) libbalsa_conf_private_set_string("BindDN", ldap->bind_dn);
+    if(ldap->passwd)  libbalsa_conf_private_set_string("Passwd", ldap->passwd);
+    libbalsa_conf_set_bool("EnableTLS", ldap->enable_tls);
     if (LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->save_config)
 	LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->save_config(ab, prefix);
 }
@@ -766,21 +767,21 @@ libbalsa_address_book_ldap_load_config(LibBalsaAddressBook * ab,
 
     ldap = LIBBALSA_ADDRESS_BOOK_LDAP(ab);
 
-    ldap->host = gnome_config_get_string("Host");
-    ldap->base_dn = gnome_config_get_string("BaseDN");
+    ldap->host = libbalsa_conf_get_string("Host");
+    ldap->base_dn = libbalsa_conf_get_string("BaseDN");
     if(ldap->base_dn && *ldap->base_dn == 0) { 
 	g_free(ldap->base_dn); ldap->base_dn = NULL; 
     }
 
-    ldap->bind_dn = gnome_config_private_get_string("BindDN");
+    ldap->bind_dn = libbalsa_conf_private_get_string("BindDN");
     if(ldap->bind_dn && *ldap->bind_dn == 0) { 
 	g_free(ldap->bind_dn); ldap->bind_dn = NULL; 
     }
-    ldap->passwd = gnome_config_private_get_string("Passwd");
+    ldap->passwd = libbalsa_conf_private_get_string("Passwd");
     if(ldap->passwd && *ldap->passwd == 0) { 
 	g_free(ldap->passwd); ldap->passwd = NULL; 
     }
-    ldap->enable_tls = gnome_config_get_bool("EnableTLS");
+    ldap->enable_tls = libbalsa_conf_get_bool("EnableTLS");
 
     if (LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->load_config)
 	LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->load_config(ab, prefix);
