@@ -31,6 +31,7 @@
 
 #include "libbalsa.h"
 #include "misc.h"
+#include "mime-stream-shared.h"
 #include "i18n.h"
 
 LibBalsaMessageBody *
@@ -108,7 +109,8 @@ libbalsa_message_body_extract_embedded_headers(GMimeMessage* msg)
 
     subj = g_mime_message_get_subject(msg);
     if (subj) {
-	ehdr->subject = g_mime_utils_header_decode_text(subj);
+	ehdr->subject =
+	    g_mime_utils_header_decode_text((const unsigned char *) subj);
 	libbalsa_utf8_sanitize(&ehdr->subject, TRUE, NULL);
     } else 
 	ehdr->subject = g_strdup(_("(No subject)"));
@@ -382,7 +384,7 @@ libbalsa_message_body_get_stream(LibBalsaMessageBody * body)
     if (body->message->mailbox
         && libbalsa_mailbox_get_message_part(body->message, body)) {
         GMimeDataWrapper *wrapper;
-	GMimePartEncodingType encoding;
+        GMimePartEncodingType encoding;
 
         wrapper =
             g_mime_part_get_content_object(GMIME_PART(body->mime_part));
@@ -393,7 +395,8 @@ libbalsa_message_body_get_stream(LibBalsaMessageBody * body)
         switch (encoding) {
         case GMIME_PART_ENCODING_BASE64:
             filter =
-                g_mime_filter_basic_new_type(GMIME_FILTER_BASIC_BASE64_DEC);
+                g_mime_filter_basic_new_type
+                (GMIME_FILTER_BASIC_BASE64_DEC);
             stream =
                 libbalsa_message_body_stream_add_filter(stream, filter);
             break;
@@ -414,31 +417,60 @@ libbalsa_message_body_get_stream(LibBalsaMessageBody * body)
         }
     } else if (body->mime_part) {
         /* Not a GMimePart... */
-	GMimeObject *object = body->mime_part;
+        GMimeObject *object = body->mime_part;
+        GMimeStream *tmp = NULL;
         if (GMIME_IS_MESSAGE_PART(object))
-            object =
-                GMIME_OBJECT(g_mime_message_part_get_message
-                             ((GMimeMessagePart *) object));
-	else
-	    g_object_ref(object);
+            object = GMIME_OBJECT(g_mime_message_part_get_message
+                                  ((GMimeMessagePart *) object));
+        else
+            g_object_ref(object);
         stream = g_mime_stream_mem_new();
+        tmp = libbalsa_message_stream(body->message);
+        libbalsa_mime_stream_shared_lock(tmp);
         g_mime_object_write_to_stream(object, stream);
-	g_object_unref(object);
+        libbalsa_mime_stream_shared_unlock(tmp);
+        g_object_unref(tmp);
+        g_object_unref(object);
     } else
         return NULL;
 
     /* convert text bodies but HTML - gtkhtml does conversion on its own. */
     if (libbalsa_message_body_type(body) == LIBBALSA_MESSAGE_BODY_TYPE_TEXT
-	&& strcmp(mime_type = libbalsa_message_body_get_mime_type(body),
-			                  "text/html") != 0
-	&& (charset = libbalsa_message_body_charset(body)) != NULL
-	&& g_ascii_strcasecmp(charset, "unknown-8bit") != 0
-	&& (filter = g_mime_filter_charset_new(charset, "UTF-8")) != NULL)
-	stream = libbalsa_message_body_stream_add_filter(stream, filter);
+        && strcmp(mime_type = libbalsa_message_body_get_mime_type(body),
+                  "text/html") != 0
+        && (charset = libbalsa_message_body_charset(body)) != NULL
+        && g_ascii_strcasecmp(charset, "unknown-8bit") != 0) {
+        GMimeStream *stream_null;
+        GMimeStream *stream_filter;
+        GMimeFilter *filter_windows;
+
+        stream_null = g_mime_stream_null_new();
+        stream_filter = g_mime_stream_filter_new_with_stream(stream_null);
+        g_object_unref(stream_null);
+
+        filter_windows = g_mime_filter_windows_new(charset);
+        g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream_filter),
+                                 filter_windows);
+
+        libbalsa_mime_stream_shared_lock(stream);
+        g_mime_stream_reset(stream);
+        g_mime_stream_write_to_stream(stream, stream_filter);
+        libbalsa_mime_stream_shared_unlock(stream);
+        g_object_unref(stream_filter);
+
+        charset = g_mime_filter_windows_real_charset(GMIME_FILTER_WINDOWS
+                                                     (filter_windows));
+        if ((filter = g_mime_filter_charset_new(charset, "UTF-8"))) {
+            stream =
+                libbalsa_message_body_stream_add_filter(stream, filter);
+            g_free(body->charset);
+            body->charset = g_strdup(charset);
+        }
+        g_object_unref(filter_windows);
+    }
 
     g_free(mime_type);
 
-    g_mime_stream_reset(stream);
     return stream;
 }
 
@@ -461,16 +493,20 @@ libbalsa_message_body_get_content(LibBalsaMessageBody * body, gchar ** buf)
     array = g_byte_array_new();
     stream_mem = g_mime_stream_mem_new_with_byte_array(array);
     g_mime_stream_mem_set_owner(GMIME_STREAM_MEM(stream_mem), FALSE);
+
+    libbalsa_mime_stream_shared_lock(stream);
+    g_mime_stream_reset(stream);
     len = g_mime_stream_write_to_stream(stream, stream_mem);
-    g_object_unref(stream_mem);
+    libbalsa_mime_stream_shared_unlock(stream);
     g_object_unref(stream);
+    g_object_unref(stream_mem);
 
     if (len >= 0) {
+	guint8 zero = 0;
         len = array->len;
 	/* NULL-terminate, in case it is used as a string. */
-	g_byte_array_append(array, "", 1);
-        *buf = array->data;
-        g_byte_array_free(array, FALSE);
+	g_byte_array_append(array, &zero, 1);
+        *buf = (gchar *) g_byte_array_free(array, FALSE);
     } else
         g_byte_array_free(array, TRUE);
 
@@ -490,14 +526,17 @@ libbalsa_message_body_get_pixbuf(LibBalsaMessageBody * body, GError ** err)
     stream = libbalsa_message_body_get_stream(body);
     if (!stream)
         return pixbuf;
+    libbalsa_mime_stream_shared_lock(stream);
+    g_mime_stream_reset(stream);
 
     loader = gdk_pixbuf_loader_new();
     while ((count = g_mime_stream_read(stream, buf, sizeof(buf))) > 0) {
-        if (!gdk_pixbuf_loader_write(loader, buf, count, err)) {
+        if (!gdk_pixbuf_loader_write(loader, (guchar *) buf, count, err)) {
             ok = FALSE;
             break;
         }
     }
+    libbalsa_mime_stream_shared_unlock(stream);
     g_object_unref(stream);
 
     if (!gdk_pixbuf_loader_close(loader, ok ? err : NULL))
@@ -520,17 +559,28 @@ libbalsa_message_body_save_fd(LibBalsaMessageBody * body, int fd,
     gboolean retval = TRUE;
 
     stream = libbalsa_message_body_get_stream(body);
+    libbalsa_mime_stream_shared_lock(stream);
+    g_mime_stream_reset(stream);
     stream_fs = g_mime_stream_fs_new(fd);
 
-    if(filter_crlf) {
+    if (filter_crlf) {
         GMimeFilter *filter;
+
+        if (!GMIME_IS_STREAM_FILTER(stream)) {
+            GMimeStream *stream_filter =
+                g_mime_stream_filter_new_with_stream(stream);
+            g_object_unref(stream);
+            stream = stream_filter;
+        }
         filter = g_mime_filter_crlf_new(GMIME_FILTER_CRLF_DECODE,
                                         GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
         g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream), filter);
         g_object_unref(filter);
     }
+
     if (g_mime_stream_write_to_stream(stream, stream_fs) < 0)
 	retval = FALSE;
+    libbalsa_mime_stream_shared_unlock(stream);
     g_object_unref(stream);
     g_object_unref(stream_fs);
 
