@@ -48,6 +48,7 @@
  * directly gets into some kind of circular #include problem, whereas
  * including it indirectly through "libbalsa.h" doesn't! */
 #include "libbalsa.h"
+#include "mime-stream-shared.h"
 
 /* Forward reference. */
 static gboolean libbalsa_html_url_requested(GtkWidget * html,
@@ -87,6 +88,7 @@ libbalsa_html_write_mime_stream(GtkHTMLStream * stream,
 /* Helper for creating a new GtkHTML widget:
  * text			the HTML source;
  * len			length of text;
+ * charset		source charset, or NULL;
  * export_string 	if we want the text exported as text/plain, a
  * 			GString to receive it; otherwise NULL;
  * message 		the LibBalsaMessage from which to extract any
@@ -95,7 +97,9 @@ libbalsa_html_write_mime_stream(GtkHTMLStream * stream,
  *			if NULL.
  */
 static GtkWidget *
-lbh_new(const gchar * text, size_t len, GString * export_string,
+lbh_new(const gchar * text, size_t len,
+	const gchar * charset,
+        GString * export_string,
 	gpointer message, GCallback link_clicked_cb)
 {
     GtkWidget *html;
@@ -110,8 +114,28 @@ lbh_new(const gchar * text, size_t len, GString * export_string,
 			 link_clicked_cb, NULL);
 
     stream = gtk_html_begin(GTK_HTML(html));
-    if (len > 0)
-	gtk_html_write(GTK_HTML(html), stream, text, len);
+    if (len > 0) {
+        if (charset && g_ascii_strcasecmp(charset, "us-ascii") != 0
+            && g_ascii_strcasecmp(charset, "utf-8") != 0) {
+            const gchar *charset_iconv;
+            gchar *s;
+            gsize bytes_written;
+            GError *error = NULL;
+
+            charset_iconv = g_mime_charset_iconv_name(charset);
+            s = g_convert(text, len, "utf-8", charset_iconv, NULL,
+                          &bytes_written, &error);
+            if (error) {
+                g_message("(%s) error converting from %s to utf-8:\n%s\n",
+                          __func__, charset_iconv, error->message);
+                g_error_free(error);
+                gtk_html_write(GTK_HTML(html), stream, text, len);
+            } else
+                gtk_html_write(GTK_HTML(html), stream, s, bytes_written);
+            g_free(s);
+        } else
+            gtk_html_write(GTK_HTML(html), stream, text, len);
+    }
     if (export_string)
 	gtk_html_export(GTK_HTML(html), "text/plain",
 			(GtkHTMLSaveReceiverFn) libbalsa_html_receiver_fn,
@@ -131,6 +155,7 @@ lbh_new(const gchar * text, size_t len, GString * export_string,
 /* Create a new HtmlView widget:
  * text			the HTML source;
  * len			length of text;
+ * charset		source charset, or NULL;
  * message 		the LibBalsaMessage from which to extract any
  *			HTML objects (by url); ignored if NULL;
  * link_clicked_cb	callback for the "link-clicked" signal; ignored
@@ -138,9 +163,10 @@ lbh_new(const gchar * text, size_t len, GString * export_string,
  */
 GtkWidget *
 libbalsa_html_new(const gchar * text, size_t len,
+		  const gchar * charset,
 		  gpointer message, GCallback link_clicked_cb)
 {
-    return lbh_new(text, len, NULL, message, link_clicked_cb);
+    return lbh_new(text, len, charset, NULL, message, link_clicked_cb);
 }
 
 /* Use an HtmlView widget to convert html text to a (null-terminated) string:
@@ -155,7 +181,7 @@ libbalsa_html_to_string(gchar ** text, size_t len)
     GString *str;
 
     str = g_string_new(NULL);	/* We want only the text, in str. */
-    html = lbh_new(*text, len, str, NULL, NULL);
+    html = lbh_new(*text, len, NULL, str, NULL, NULL);
     gtk_widget_destroy(html);
 
     g_free(*text);
@@ -289,6 +315,7 @@ libbalsa_html_write_mime_stream(HtmlStream * stream,
 /* Create a new HtmlView widget:
  * text			the HTML source;
  * len			length of text;
+ * charset		ignored;
  * message 		the LibBalsaMessage from which to extract any
  *			HTML objects (by url); ignored if NULL;
  * link_clicked_cb	callback for the "link-clicked" signal; ignored
@@ -297,6 +324,7 @@ libbalsa_html_write_mime_stream(HtmlStream * stream,
 
 GtkWidget *
 libbalsa_html_new(const gchar * text, size_t len,
+		  const gchar * charset,
 		  gpointer message, GCallback link_clicked_cb)
 {
     GtkWidget *html;
@@ -448,7 +476,10 @@ libbalsa_html_url_requested(GtkWidget * html, const gchar * url,
 	    return FALSE;
     }
 
+    libbalsa_mime_stream_shared_lock(mime_stream);
+    g_mime_stream_reset(mime_stream);
     libbalsa_html_write_mime_stream(stream, mime_stream);
+    libbalsa_mime_stream_shared_unlock(mime_stream);
 
     g_object_unref(mime_stream);
 
@@ -466,6 +497,7 @@ libbalsa_html_filter(LibBalsaHTMLType html_type, gchar ** text, guint len)
     GMimeStream *filter_stream;
     GMimeFilter *filter;
     guint32 flags;
+    guint8 zero = 0;
 
     switch (html_type) {
     case LIBBALSA_HTML_TYPE_ENRICHED:
@@ -493,11 +525,11 @@ libbalsa_html_filter(LibBalsaHTMLType html_type, gchar ** text, guint len)
     g_mime_stream_write(filter_stream, *text, len);
     g_object_unref(filter_stream);
 
-    g_byte_array_append(array, "", 1);
+    g_byte_array_append(array, &zero, 1);
 
     retval = array->len;
     g_free(*text);
-    *text = g_byte_array_free(array, FALSE);
+    *text = (gchar *) g_byte_array_free(array, FALSE);
     return retval;
 }
 
