@@ -33,6 +33,11 @@
 #include "libbalsa-conf.h"
 #include "i18n.h"
 
+#if ENABLE_ESMTP
+#include <string.h>
+#include "smtp-server.h"
+#endif                          /* ENABLE_ESMTP */
+
 /*
  * The class.
  */
@@ -105,6 +110,7 @@ libbalsa_identity_init(LibBalsaIdentity* ident)
 #ifdef HAVE_GPGME
     ident->gpg_sign = FALSE;
     ident->gpg_encrypt = FALSE;
+    ident->always_trust = FALSE;
     ident->crypt_protocol = LIBBALSA_PROTECT_OPENPGP;
 #endif
 }
@@ -125,6 +131,10 @@ libbalsa_identity_finalize(GObject * object)
     g_free(ident->reply_string);
     g_free(ident->forward_string);
     g_free(ident->signature_path);
+#if ENABLE_ESMTP
+    if (ident->smtp_server)
+        g_object_unref(ident->smtp_server);
+#endif                          /* ENABLE_ESMTP */
 
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -289,6 +299,22 @@ libbalsa_identity_set_sig_prepend(LibBalsaIdentity* ident, gboolean prepend)
 }
 
 
+#if ENABLE_ESMTP
+void
+libbalsa_identity_set_smtp_server(LibBalsaIdentity * ident,
+                                  LibBalsaSmtpServer * smtp_server)
+{
+    g_return_if_fail(ident != NULL);
+
+    if (ident->smtp_server)
+	g_object_unref(ident->smtp_server);
+    ident->smtp_server = smtp_server;
+    if (smtp_server)
+	g_object_ref(smtp_server);
+}
+#endif                          /* ENABLE_ESMTP */
+
+
 /* Used by both dialogs: */
 
 /* Widget padding: */
@@ -476,9 +502,12 @@ static void config_frame_button_select_cb(GtkTreeSelection * selection,
                                           GtkDialog * dialog);
 
 static void ident_dialog_add_checkbutton(GtkWidget *, gint, GtkDialog *,
-                                         const gchar *, const gchar *);
+                                         const gchar *, const gchar *,
+					 gboolean sensitive);
 static void ident_dialog_add_entry(GtkWidget *, gint, GtkDialog *,
                                    const gchar *, const gchar *);
+#if ENABLE_ESMTP
+#endif /* ENABLE_ESMTP */
 static gchar *ident_dialog_get_text(GtkDialog *, const gchar *);
 static gboolean ident_dialog_get_bool(GtkDialog *, const gchar *);
 static gboolean ident_dialog_update(GtkDialog *);
@@ -505,16 +534,32 @@ static void md_name_changed(GtkEntry * name, GtkTreeView * tree);
 static void md_sig_path_changed(GtkEntry * sig_path, GObject * dialog);
 
 #ifdef HAVE_GPGME
-static void add_show_menu(const char* label, gint value, GtkWidget* menu);
-static void ident_dialog_add_option_menu(GtkWidget * table, gint row,
-                                         GtkDialog * dialog,
-                                         const gchar * label_name,
-                                         const gchar * menu_key);
-static gint ident_dialog_get_menu(GtkDialog * dialog, const gchar * key);
-static void display_frame_set_menu(GtkDialog * dialog, const gchar* key,
-                                   gint * value);
+static void ident_dialog_add_gpg_menu(GtkWidget * table, gint row,
+                                      GtkDialog * dialog,
+                                      const gchar * label_name,
+                                      const gchar * menu_key);
+static void display_frame_set_gpg_mode(GtkDialog * dialog,
+                                       const gchar * key, gint * value);
 #endif /* HAVE_GPGME */
 
+#if ENABLE_ESMTP
+static void ident_dialog_add_smtp_menu(GtkWidget * table, gint row,
+                                       GtkDialog * dialog,
+                                       const gchar * label_name,
+                                       const gchar * menu_key,
+				       GSList * smtp_servers);
+static void display_frame_set_server(GtkDialog * dialog,
+                                     const gchar * key,
+                                     LibBalsaSmtpServer * smtp_server);
+#endif /* ENABLE_ESMTP */
+
+#if defined(HAVE_GPGME) || ENABLE_ESMTP
+static void add_show_menu(const char *label, gpointer data,
+                          GtkWidget * menu);
+static gpointer ident_dialog_get_value(GtkDialog * dialog,
+                                       const gchar * key);
+static void ident_dialog_free_values(GPtrArray * values);
+#endif                          /* defined(HAVE_GPGME) || ENABLE_ESMTP */
 
 /* Callback for the "toggled" signal of the "Default" column. */
 static void
@@ -761,31 +806,51 @@ new_ident_cb(GtkTreeView * tree, GtkWidget * dialog)
 }
 
 
+/*
+ * Helper: append a notebook page containing a table with the requested
+ * number of rows
+ */
+static GtkWidget*
+append_ident_notebook_page(GtkNotebook *notebook, guint rows,
+			   const gchar * tab_label)
+{
+    GtkWidget *vbox;
+    GtkWidget *table;
+
+    vbox = gtk_vbox_new(FALSE, 0);
+    table = gtk_table_new(rows, 2, FALSE);
+    gtk_table_set_row_spacings(GTK_TABLE(table), padding);
+    gtk_table_set_col_spacings(GTK_TABLE(table), padding);
+    gtk_container_set_border_width(GTK_CONTAINER(table), padding);
+    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
+    gtk_notebook_append_page(notebook, vbox, gtk_label_new(tab_label));
+    
+    return table;
+}
+
+
 /*
  * Put the required GtkEntries, Labels, and Checkbuttons in the dialog
  * for creating/editing identities.
  */
+#if ENABLE_ESMTP
+static GtkWidget*
+setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree,
+                  GSList * smtp_servers)
+#else                           /* ENABLE_ESMTP */
 static GtkWidget*
 setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
+#endif                          /* ENABLE_ESMTP */
 {
-     
-    GtkWidget* frame = gtk_frame_new(NULL);
-#ifdef HAVE_GPGME
-    GtkWidget *table = gtk_table_new(18, 2, FALSE);
-#else
-    GtkWidget *table = gtk_table_new(15, 2, FALSE);
-#endif
-    gint row = 0;
+    GtkNotebook *notebook = GTK_NOTEBOOK(gtk_notebook_new());
+    GtkWidget *table;
+    gint row;
     GObject *name;
     GObject *sig_path;
 
-    gtk_container_set_border_width(GTK_CONTAINER(frame), padding);
-    gtk_container_set_border_width(GTK_CONTAINER(table), padding);
-
-    gtk_container_add(GTK_CONTAINER(frame), table);
-    gtk_table_set_row_spacings(GTK_TABLE(table), padding);
-    gtk_table_set_col_spacings(GTK_TABLE(table), padding);
-
+    /* create the "General" tab */
+    table = append_ident_notebook_page(notebook, 8, _("General"));
+    row = 0;
     ident_dialog_add_entry(table, row++, dialog, _("_Identity Name:"), 
 		           "identity-name");
     ident_dialog_add_entry(table, row++, dialog, _("_Full Name:"), 
@@ -802,47 +867,51 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
                            "identity-replystring");
     ident_dialog_add_entry(table, row++, dialog, _("F_orward String:"), 
                            "identity-forwardstring");
+#if ENABLE_ESMTP
+    ident_dialog_add_smtp_menu(table, row++, dialog, _("SMT_P Server"),
+                               "identity-smtp-server", smtp_servers);
+#endif /* ENABLE_ESMTP */
+
+    /* create the "Signature" tab */
+    table = append_ident_notebook_page(notebook, 7, _("Signature"));
+    row = 0;
     ident_dialog_add_entry(table, row++, dialog, _("Signature _Path:"), 
-                           "identity-sigpath");
-    
+			   "identity-sigpath");
     ident_dialog_add_checkbutton(table, row++, dialog,
                                 _("_Execute Signature"),
-				"identity-sigexecutable");
+				 "identity-sigexecutable", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog,
                                  _("Incl_ude Signature"), 
-                                 "identity-sigappend");
+                                 "identity-sigappend", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog, 
                                  _("Include Signature When For_warding"),
-                                 "identity-whenforward");
+                                 "identity-whenforward", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog,
                                  _("Include Signature When Rep_lying"),
-                                 "identity-whenreply");
+                                 "identity-whenreply", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog, 
                                  _("_Add Signature Separator"),
-                                 "identity-sigseparator");
+                                 "identity-sigseparator", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog,
                                  _("Prepend Si_gnature"),
-                                 "identity-sigprepend");
+                                 "identity-sigprepend", FALSE);
 
 #ifdef HAVE_GPGME
+    /* create the "Security" tab */
+    table = append_ident_notebook_page(notebook, 4, _("Security"));
+    row = 0;
     ident_dialog_add_checkbutton(table, row++, dialog, 
                                  _("sign messages by default"),
-                                 "identity-gpgsign");
-    gtk_widget_set_sensitive(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),
-							  "identity-gpgsign")),
-			     TRUE);
+                                 "identity-gpgsign", TRUE);
     ident_dialog_add_checkbutton(table, row++, dialog,
                                  _("encrypt messages by default"),
-                                 "identity-gpgencrypt");
-    gtk_widget_set_sensitive(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),
-							  "identity-gpgencrypt")),
-			     TRUE);
-    ident_dialog_add_option_menu(table, row++, dialog,
-				 _("default crypto protocol"),
+                                 "identity-gpgencrypt", TRUE);
+    ident_dialog_add_gpg_menu(table, row++, dialog,
+				 _("default protocol"),
 				 "identity-crypt-protocol");
-    gtk_widget_set_sensitive(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),
-							  "identity-crypt-protocol")),
-			     TRUE);
+    ident_dialog_add_checkbutton(table, row++, dialog,
+                                 _("always trust GnuPG keys when encrypting"),
+                                 "identity-trust-always", TRUE);
 #endif
 
     name = g_object_get_data(G_OBJECT(dialog), "identity-name");
@@ -852,7 +921,9 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     g_signal_connect(sig_path, "changed",
                      G_CALLBACK(md_sig_path_changed), dialog);
 
-    return GTK_WIDGET(frame);
+    gtk_notebook_set_current_page(notebook, 0);
+
+    return GTK_WIDGET(notebook);
 }
 
 /* Callback for the "changed" signal of the name entry; updates the name
@@ -895,14 +966,14 @@ md_sig_path_changed(GtkEntry * sig_path, GObject * dialog)
 static void
 ident_dialog_add_checkbutton(GtkWidget * table, gint row,
                              GtkDialog * dialog, const gchar * check_label,
-                             const gchar * check_key)
+                             const gchar * check_key, gboolean sensitive)
 {
     GtkWidget *check;
 
     check = gtk_check_button_new_with_mnemonic(check_label);
     gtk_table_attach_defaults(GTK_TABLE(table), check, 0, 2, row, row + 1);
     g_object_set_data(G_OBJECT(dialog), check_key, check);
-    gtk_widget_set_sensitive(check, FALSE);
+    gtk_widget_set_sensitive(check, sensitive);
 }
 
 
@@ -1025,6 +1096,12 @@ ident_dialog_update(GtkDialog* dlg)
     id->reply_string    = ident_dialog_get_text(dlg, "identity-replystring");
     g_free(id->forward_string);
     id->forward_string  = ident_dialog_get_text(dlg, "identity-forwardstring");
+#if ENABLE_ESMTP
+    g_object_unref(id->smtp_server);
+    id->smtp_server = ident_dialog_get_value(dlg, "identity-smtp-server");
+    g_object_ref(id->smtp_server);
+#endif /* ENABLE_ESMTP */
+
     g_free(id->signature_path);
     id->signature_path  = ident_dialog_get_text(dlg, "identity-sigpath");
     
@@ -1038,7 +1115,9 @@ ident_dialog_update(GtkDialog* dlg)
 #ifdef HAVE_GPGME
     id->gpg_sign        = ident_dialog_get_bool(dlg, "identity-gpgsign");
     id->gpg_encrypt     = ident_dialog_get_bool(dlg, "identity-gpgencrypt");
-    id->crypt_protocol  = ident_dialog_get_menu(dlg, "identity-crypt-protocol");
+    id->always_trust    = ident_dialog_get_bool(dlg, "identity-trust-always");
+    id->crypt_protocol  = GPOINTER_TO_INT(ident_dialog_get_value
+                                          (dlg, "identity-crypt-protocol"));
 #endif
    
     return TRUE;
@@ -1204,9 +1283,22 @@ help_ident_cb(void)
    identites, the default one. Additionally, a callback is passed that
    will be executed when the identity list is modified: new entries
    are added or other entries are removed. */
+#if ENABLE_ESMTP
+static void
+lbi_free_smtp_server_list(GSList ** smtp_server_list)
+{
+    g_slist_foreach(*smtp_server_list, (GFunc) g_object_unref, NULL);
+    g_slist_free(*smtp_server_list);
+    g_free(smtp_server_list);
+}
+#endif /* ENABLE_ESMTP */
+
 void
 libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
 				LibBalsaIdentity **default_id,
+#if ENABLE_ESMTP
+				GSList * smtp_servers,
+#endif /* ENABLE_ESMTP */
                                 void (*changed_cb)(gpointer))
 {
     static GtkWidget *dialog = NULL;
@@ -1215,6 +1307,9 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
     GtkWidget* hbox;
     GtkTreeView* tree;
     GtkTreeSelection *select;
+#if ENABLE_ESMTP
+    GSList **smtp_server_list;
+#endif /* ENABLE_ESMTP */
 
     /* Show only one dialog at a time. */
     if (dialog) {
@@ -1249,8 +1344,19 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
 
     gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
 
+#if ENABLE_ESMTP
+    smtp_server_list = g_new(GSList *, 1);
+    *smtp_server_list = g_slist_copy(smtp_servers);
+    g_slist_foreach(smtp_servers, (GFunc) g_object_ref, NULL);
     display_frame = setup_ident_frame(GTK_DIALOG(dialog),
-                                      FALSE, tree);
+                                      FALSE, tree, smtp_servers);
+    g_object_weak_ref(G_OBJECT(display_frame),
+	              (GWeakNotify) lbi_free_smtp_server_list,
+		      smtp_server_list);
+#else  /* ENABLE_ESMTP */
+    display_frame = setup_ident_frame(GTK_DIALOG(dialog), FALSE, tree);
+#endif /* ENABLE_ESMTP */
+
     gtk_box_pack_start(GTK_BOX(hbox), display_frame, TRUE, TRUE, 0);
 
     select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
@@ -1328,8 +1434,12 @@ display_frame_update(GtkDialog * dialog, LibBalsaIdentity* ident)
                             ident->reply_string);
     display_frame_set_field(dialog, "identity-forwardstring", 
                             ident->forward_string);
-    display_frame_set_field(dialog, "identity-sigpath", ident->signature_path);
+#if ENABLE_ESMTP
+    display_frame_set_server(dialog, "identity-smtp-server",
+                             ident->smtp_server);
+#endif /* ENABLE_ESMTP */
 
+    display_frame_set_field(dialog, "identity-sigpath", ident->signature_path);
     display_frame_set_boolean(dialog, "identity-sigexecutable", ident->sig_executable);
 
     display_frame_set_boolean(dialog, "identity-sigappend", ident->sig_sending);
@@ -1347,7 +1457,9 @@ display_frame_update(GtkDialog * dialog, LibBalsaIdentity* ident)
                               ident->gpg_sign);    
     display_frame_set_boolean(dialog, "identity-gpgencrypt", 
                               ident->gpg_encrypt);    
-    display_frame_set_menu(dialog, "identity-crypt-protocol",
+    display_frame_set_boolean(dialog, "identity-trust-always", 
+                              ident->always_trust);    
+    display_frame_set_gpg_mode(dialog, "identity-crypt-protocol",
 			   &ident->crypt_protocol);
 #endif
 }
@@ -1380,15 +1492,11 @@ display_frame_set_boolean(GtkDialog * dialog,
    configuration data.
 */
 LibBalsaIdentity*
-libbalsa_identity_new_config(const gchar* prefix, const gchar* name)
+libbalsa_identity_new_config(const gchar* name)
 {
     LibBalsaIdentity* ident;
     gchar* tmpstr;
     
-    g_return_val_if_fail(prefix != NULL, NULL);
-
-    libbalsa_conf_push_prefix(prefix);
-
     ident = LIBBALSA_IDENTITY(libbalsa_identity_new_with_name(name));
 
     tmpstr = libbalsa_conf_get_string("FullName");
@@ -1428,20 +1536,19 @@ libbalsa_identity_new_config(const gchar* prefix, const gchar* name)
 #ifdef HAVE_GPGME
     ident->gpg_sign = libbalsa_conf_get_bool("GpgSign");
     ident->gpg_encrypt = libbalsa_conf_get_bool("GpgEncrypt");
+    ident->always_trust = libbalsa_conf_get_bool("GpgTrustAlways");
     ident->crypt_protocol = libbalsa_conf_get_int("CryptProtocol=16");
 #endif
 
-    libbalsa_conf_pop_prefix();
-
     return ident;
 }
 
 void 
-libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* prefix)
+libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
 {
     g_return_if_fail(ident);
 
-    libbalsa_conf_push_prefix(prefix);
+    libbalsa_conf_push_group(group);
     libbalsa_conf_set_string("FullName", ident->ia->name);
     
     if (ident->ia->type == INTERNET_ADDRESS_NAME)
@@ -1452,8 +1559,13 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* prefix)
     libbalsa_conf_set_string("Bcc", ident->bcc);
     libbalsa_conf_set_string("ReplyString", ident->reply_string);
     libbalsa_conf_set_string("ForwardString", ident->forward_string);
-    libbalsa_conf_set_string("SignaturePath", ident->signature_path);
+#if ENABLE_ESMTP
+    libbalsa_conf_set_string("SmtpServer",
+                             libbalsa_smtp_server_get_name(ident->
+                                                           smtp_server));
+#endif                          /* ENABLE_ESMTP */
 
+    libbalsa_conf_set_string("SignaturePath", ident->signature_path);
     libbalsa_conf_set_bool("SigExecutable", ident->sig_executable);
     libbalsa_conf_set_bool("SigSending", ident->sig_sending);
     libbalsa_conf_set_bool("SigForward", ident->sig_whenforward);
@@ -1464,10 +1576,11 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* prefix)
 #ifdef HAVE_GPGME
     libbalsa_conf_set_bool("GpgSign", ident->gpg_sign);
     libbalsa_conf_set_bool("GpgEncrypt", ident->gpg_encrypt);
+    libbalsa_conf_set_bool("GpgTrustAlways", ident->always_trust);
     libbalsa_conf_set_int("CryptProtocol", ident->crypt_protocol);
 #endif
 
-    libbalsa_conf_pop_prefix();
+    libbalsa_conf_pop_group();
 }
 
 
@@ -1498,75 +1611,44 @@ libbalsa_identity_set_crypt_protocol(LibBalsaIdentity* ident, gint protocol)
 }
 
 
-static void
-add_show_menu(const char* label, gint value, GtkWidget* menu)
-{
-    GArray *values = g_object_get_data(G_OBJECT(menu), "identity-value");
-    gtk_combo_box_append_text(GTK_COMBO_BOX(menu), label);
-    g_array_append_val(values, value);
-}
-
-
 /*
  * Add an option menu to the given dialog with a label next to it
  * explaining the contents.  A reference to the entry is stored as
  * object data attached to the dialog with the given key.
  */
-static void
-ident_dialog_free_values(GArray * values)
-{
-    g_array_free(values, TRUE);
-}
 
 static void
-ident_dialog_add_option_menu(GtkWidget * table, gint row, GtkDialog * dialog,
-                             const gchar * label_name, const gchar * menu_key)
+ident_dialog_add_gpg_menu(GtkWidget * table, gint row, GtkDialog * dialog,
+                          const gchar * label_name, const gchar * menu_key)
 {
     GtkWidget *label;
     GtkWidget *opt_menu;
-    GArray *values;
+    GPtrArray *values;
 
     label = gtk_label_new_with_mnemonic(label_name);
     gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
     gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row + 1);
 
     opt_menu = gtk_combo_box_new_text();
-    values = g_array_new(FALSE, FALSE, sizeof(gint));
+    values = g_ptr_array_sized_new(3);
     g_object_set_data_full(G_OBJECT(opt_menu), "identity-value", values,
                            (GDestroyNotify) ident_dialog_free_values);
     gtk_table_attach_defaults(GTK_TABLE(table), opt_menu, 1, 2, row, row + 1);
     g_object_set_data(G_OBJECT(dialog), menu_key, opt_menu);
 
-    add_show_menu(_("GnuPG using MIME mode"), LIBBALSA_PROTECT_RFC3156,
-                  opt_menu);
-    add_show_menu(_("GnuPG using OpenPGP mode"), LIBBALSA_PROTECT_OPENPGP,
-                  opt_menu);
+    add_show_menu(_("GnuPG MIME mode"),
+                  GINT_TO_POINTER(LIBBALSA_PROTECT_RFC3156), opt_menu);
+    add_show_menu(_("GnuPG OpenPGP mode"),
+                  GINT_TO_POINTER(LIBBALSA_PROTECT_OPENPGP), opt_menu);
 #ifdef HAVE_SMIME
-    add_show_menu(_("S/MIME mode"), LIBBALSA_PROTECT_SMIMEV3, opt_menu);
+    add_show_menu(_("GpgSM S/MIME mode"),
+                  GINT_TO_POINTER(LIBBALSA_PROTECT_SMIMEV3), opt_menu);
 #endif
 }
 
 
-/*
- * Get the value of the active option menu item
- */
-static gint
-ident_dialog_get_menu(GtkDialog * dialog, const gchar * key)
-{
-    GtkWidget * menu;
-    gint value;
-    GArray *values;
-
-    menu = g_object_get_data(G_OBJECT(dialog), key);
-    value = gtk_combo_box_get_active(GTK_COMBO_BOX(menu));
-    values = g_object_get_data(G_OBJECT(menu), "identity-value");
-    
-    return g_array_index(values, gint, value);
-}
-
-
 static void
-display_frame_set_menu(GtkDialog * dialog, const gchar* key, gint * value)
+display_frame_set_gpg_mode(GtkDialog * dialog, const gchar* key, gint * value)
 {
     GtkComboBox *opt_menu = g_object_get_data(G_OBJECT(dialog), key);
  
@@ -1586,5 +1668,90 @@ display_frame_set_menu(GtkDialog * dialog, const gchar* key, gint * value)
             *value = LIBBALSA_PROTECT_RFC3156;
         }
 }
-
 #endif  /* HAVE_GPGME */
+
+#if ENABLE_ESMTP
+static void
+ident_dialog_add_smtp_menu(GtkWidget * table, gint row, GtkDialog * dialog,
+                           const gchar * label_name,
+                           const gchar * menu_key, GSList * smtp_servers)
+{
+    GtkWidget *label;
+    GtkWidget *combo_box;
+    GSList *list;
+    GPtrArray *values;
+
+    label = gtk_label_new_with_mnemonic(label_name);
+    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row + 1);
+
+    combo_box = gtk_combo_box_new_text();
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), combo_box);
+    values = g_ptr_array_sized_new(g_slist_length(smtp_servers));
+    g_object_set_data_full(G_OBJECT(combo_box), "identity-value", values,
+                           (GDestroyNotify) ident_dialog_free_values);
+    gtk_table_attach_defaults(GTK_TABLE(table), combo_box, 1, 2, row,
+                              row + 1);
+    g_object_set_data(G_OBJECT(dialog), menu_key, combo_box);
+
+    for (list = smtp_servers; list; list = list->next) {
+        LibBalsaSmtpServer *smtp_server = LIBBALSA_SMTP_SERVER(list->data);
+        add_show_menu(libbalsa_smtp_server_get_name(smtp_server),
+                      smtp_server, combo_box);
+    }
+}
+
+static void
+display_frame_set_server(GtkDialog * dialog, const gchar * key,
+                         LibBalsaSmtpServer * smtp_server)
+{
+    GtkComboBox *combo_box = g_object_get_data(G_OBJECT(dialog), key);
+    GPtrArray *values;
+    guint i;
+
+    values = g_object_get_data(G_OBJECT(combo_box), "identity-value");
+
+    for (i = 0; i < values->len; i++) {
+        if (g_ptr_array_index(values, i) == smtp_server)
+            gtk_combo_box_set_active(combo_box, i);
+    }
+}
+
+#endif                          /* ENABLE_ESMTP */
+
+#if defined(HAVE_GPGME) || ENABLE_ESMTP
+/* add_show_menu: helper function */
+static void
+add_show_menu(const char *label, gpointer data, GtkWidget * menu)
+{
+    GPtrArray *values =
+        g_object_get_data(G_OBJECT(menu), "identity-value");
+
+    gtk_combo_box_append_text(GTK_COMBO_BOX(menu), label);
+    g_ptr_array_add(values, data);
+}
+
+/*
+ * Get the value of the active option menu item
+ */
+static gpointer
+ident_dialog_get_value(GtkDialog * dialog, const gchar * key)
+{
+    GtkWidget * menu;
+    gint value;
+    GPtrArray *values;
+
+    menu = g_object_get_data(G_OBJECT(dialog), key);
+    value = gtk_combo_box_get_active(GTK_COMBO_BOX(menu));
+    values = g_object_get_data(G_OBJECT(menu), "identity-value");
+    
+    return g_ptr_array_index(values, value);
+}
+
+/* ident_dialog_free_values: helper function */
+static void
+ident_dialog_free_values(GPtrArray * values)
+{
+    g_ptr_array_free(values, TRUE);
+}
+#endif /* defined(HAVE_GPGME) || ENABLE_ESMTP */
