@@ -18,8 +18,9 @@
  */
 #include "config.h"
 
-#define _POSIX_SOURCE 1
-#define _BSD_SOURCE   1
+#define _POSIX_C_SOURCE 199506L
+#define _XOPEN_SOURCE 500
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netdb.h>
@@ -142,6 +143,8 @@ imap_mbox_handle_init(ImapMboxHandle *handle)
 #endif
   handle->tls_mode = IMAP_TLS_ENABLED;
   handle->cmd_queue = g_hash_table_new(NULL, NULL);
+  handle->status_resps = g_hash_table_new_full(g_str_hash, g_str_equal,
+                                               NULL, NULL);
 
   handle->info_cb  = NULL;
   handle->info_arg = NULL;
@@ -168,8 +171,8 @@ imap_mbox_handle_class_init(ImapMboxHandleClass * klass)
                  G_SIGNAL_RUN_FIRST,
                  G_STRUCT_OFFSET(ImapMboxHandleClass, list_response),
                  NULL, NULL,
-                 libimap_VOID__INT_STRING_POINTER, G_TYPE_NONE, 3,
-                 G_TYPE_INT, G_TYPE_STRING, G_TYPE_POINTER);
+                 libimap_VOID__INT_INT_POINTER, G_TYPE_NONE, 3,
+                 G_TYPE_INT, G_TYPE_INT, G_TYPE_POINTER);
 
   imap_mbox_handle_signals[LSUB_RESPONSE] = 
     g_signal_new("lsub-response",
@@ -177,8 +180,8 @@ imap_mbox_handle_class_init(ImapMboxHandleClass * klass)
                  G_SIGNAL_RUN_FIRST,
                  G_STRUCT_OFFSET(ImapMboxHandleClass, lsub_response),
                  NULL, NULL,
-                 libimap_VOID__INT_STRING_POINTER, G_TYPE_NONE, 3,
-                 G_TYPE_INT, G_TYPE_STRING, G_TYPE_POINTER);
+                 libimap_VOID__INT_INT_POINTER, G_TYPE_NONE, 3,
+                 G_TYPE_INT, G_TYPE_INT, G_TYPE_POINTER);
 
   imap_mbox_handle_signals[EXPUNGE_NOTIFY] = 
     g_signal_new("expunge-notify",
@@ -260,16 +263,16 @@ idle_process(GIOChannel *source, GIOCondition condition, gpointer data)
   ImapMboxHandle *h = (ImapMboxHandle*)data;
   ImapResponse rc;
   int retval;
-
+  g_return_val_if_fail(h, FALSE);
   while( (retval = sio_poll(h->sio, TRUE, FALSE, TRUE)) != -1 &&
          (retval & SIO_READ) != 0) {
     if ( (rc=imap_cmd_step(h, 0)) == IMR_UNKNOWN ||
          rc == IMR_SEVERED || rc == IMR_BYE || rc == IMR_PROTOCOL ||
          rc  == IMR_BAD) {
       printf("idle_process() got unexpected response %i!\n"
-             "Last message was: \"%s\" - shutting down idle listener\n",
+             "Last message was: \"%s\" - shutting down connection.\n",
              rc, h->last_msg);
-      /* FIXME: consider aborting connection here. */
+      imap_handle_disconnect(h);
       return FALSE;
     }
   }
@@ -294,7 +297,7 @@ idle_start(gpointer data)
     rc = imap_cmd_step (h, 0);
   } while (rc == IMR_UNTAGGED);
   if(rc != IMR_RESPOND) {
-    g_warning("Expected IMR_RESPOND but got %d\n", rc);
+    g_message("Expected IMR_RESPOND but got %d\n", rc);
     return FALSE;
   }
   while( (c=sio_getc(h->sio)) != -1 && c != '\n');
@@ -331,8 +334,9 @@ imap_handle_idle_disable(ImapMboxHandle *h)
   if(h->idle_watch_id) {
     g_source_remove(h->idle_watch_id);
     h->idle_watch_id = 0;
-    sio_write(h->sio,"DONE\r\n",6); sio_flush(h->sio);
-    /* tagged OK will be ignored */
+    if(h->sio) { /* we might have been disconnected before */
+      sio_write(h->sio,"DONE\r\n",6); sio_flush(h->sio);
+    }
   }
   return TRUE;
 }
@@ -344,6 +348,9 @@ imap_handle_disconnect(ImapMboxHandle *h)
   if(h->sio) {
     sio_detach(h->sio); h->sio = NULL;
   }
+  if(h->iochannel) {
+    g_io_channel_unref(h->iochannel); h->iochannel = NULL;
+  }
   close(h->sd);
   h->state = IMHS_DISCONNECTED;
 }
@@ -364,8 +371,10 @@ imap_mbox_handle_connect(ImapMboxHandle* ret, const char *host, int over_ssl)
 
   g_return_val_if_fail(imap_mbox_is_disconnected(ret), IMAP_CONNECT_FAILED);
 #if !defined(USE_TLS)
-  if(over_ssl)
+  if(over_ssl) {
+    imap_mbox_handle_set_msg(ret,"SSL requested but SSL support not compiled");
     return IMAP_UNSECURE;
+  }
 #else
   ret->over_ssl = over_ssl;
 #endif
@@ -525,11 +534,15 @@ imap_mbox_connect(ImapMboxHandle* handle)
 #ifdef USE_TLS
   if(handle->over_ssl) {
     SSL *ssl = imap_create_ssl();
-    if(!ssl) return IMAP_UNSECURE;
+    if(!ssl) {
+      imap_mbox_handle_set_msg(handle,"SSL context could not be created");
+      return IMAP_UNSECURE;
+    }
     if(imap_setup_ssl(handle->sio, handle->host, ssl,
                       handle->user_cb, handle->user_arg)) 
       handle->using_tls = 1;
     else {
+      imap_mbox_handle_set_msg(handle,"SSL negotiation failed");
       imap_handle_disconnect(handle);
       return IMAP_UNSECURE;
     }
@@ -540,7 +553,7 @@ imap_mbox_connect(ImapMboxHandle* handle)
 
   handle->state = IMHS_CONNECTED;
   if (imap_cmd_step(handle, 0) != IMR_UNTAGGED) {
-    g_warning("imap_mbox_connect:unexpected initial response\n");
+    g_message("imap_mbox_connect:unexpected initial response\n");
     imap_handle_disconnect(handle);
     return IMAP_PROTOCOL_ERROR;
   }
@@ -551,17 +564,20 @@ imap_mbox_connect(ImapMboxHandle* handle)
     resp = IMR_OK; /* secured already with SSL */
   else if(handle->tls_mode != IMAP_TLS_DISABLED &&
           imap_mbox_handle_can_do(handle, IMCAP_STARTTLS)) {
-    if( imap_handle_starttls(handle) != IMR_OK)
+    if( imap_handle_starttls(handle) != IMR_OK) {
+      imap_mbox_handle_set_msg(handle,"TLS negotiation failed");
       return IMAP_UNSECURE; /* TLS negotiation error */
+    }
     resp = IMR_OK; /* secured with TLS */
   } else
     resp = IMR_NO; /* not over SSL and TLS unavailable */
 #else
   resp = IMR_NO;
 #endif
-  if(handle->tls_mode == IMAP_TLS_REQUIRED && resp != IMR_OK)
+  if(handle->tls_mode == IMAP_TLS_REQUIRED && resp != IMR_OK) {
+    imap_mbox_handle_set_msg(handle,"TLS required but not available");
     return IMAP_UNSECURE;
-    
+  }
   return IMAP_SUCCESS;
 }
 
@@ -712,6 +728,7 @@ imap_mbox_handle_finalize(GObject* gobject)
   g_free(handle->last_msg);handle->last_msg = NULL;
 
   g_hash_table_destroy(handle->cmd_queue); handle->cmd_queue = NULL;
+  g_hash_table_destroy(handle->status_resps); handle->status_resps = NULL;
 
   mbox_view_dispose(&handle->mbox_view);
   imap_mbox_resize_cache(handle, 0);
@@ -1382,9 +1399,6 @@ imap_cmd_exec(ImapMboxHandle* handle, const char* cmd)
     rc = imap_cmd_step (handle, cmdno);
   } while (rc == IMR_UNTAGGED);
 
-  if (rc != IMR_OK) /* we should be silent here */
-    printf("cmd '%6s' failed, rc=%d.\n", cmd, rc);
-
   imap_handle_idle_enable(handle, 30);
 
   return rc;
@@ -1723,7 +1737,7 @@ ir_list_lsub(ImapMboxHandle *h, ImapHandleSignal signal)
     "Marked", "Unmarked", "Noselect", "Noinferiors",
     "HasChildren", "HasNoChildren"
   };
-  int flags = 0;
+  ImapMboxFlags flags = 0;
   char buf[LONG_STRING], *s, *mbx;
   int c, delim;
   ImapResponse rc;
@@ -1760,7 +1774,7 @@ ir_list_lsub(ImapMboxHandle *h, ImapHandleSignal signal)
   mbx = imap_mailbox_to_utf8(s);
   rc = ir_check_crlf(h, c);
   g_signal_emit(G_OBJECT(h), imap_mbox_handle_signals[signal],
-                0, delim, &flags, mbx);
+                0, delim, flags, mbx);
   g_free(s);
   g_free(mbx);
   return rc;
@@ -1778,12 +1792,43 @@ ir_lsub(ImapMboxHandle *h)
   return ir_list_lsub(h, LSUB_RESPONSE);
 }
 
+/* 7.2.4 STATUS Response */
+const char* imap_status_item_names[5] = {
+  "MESSAGES", "RECENT", "UIDNEXT", "UIDVALIDITY", "UNSEEN" };
 static ImapResponse
 ir_status(ImapMboxHandle *h)
 {
-  /* FIXME: implement! */
-  int c; while( (c=sio_getc(h->sio))!=EOF && c != 0x0a);
-  return IMR_OK;
+  int c;
+  char *name;
+  struct ImapStatusResult *resp;
+
+  name = imap_get_astring(h->sio, &c);
+  resp = g_hash_table_lookup(h->status_resps, name);
+  if(c                != ' ') return IMR_PROTOCOL;
+  if(sio_getc(h->sio) != '(') return IMR_PROTOCOL;
+  do {
+    char item[13], count[13]; /* longest than UIDVALIDITY */
+    c = imap_get_atom(h->sio, item, sizeof(item));
+    if(c == ')') break;
+    if(c != ' ') return IMR_PROTOCOL;
+    c = imap_get_atom(h->sio, count, sizeof(count));
+    /* FIXME: process the response */
+    if(resp) {
+      unsigned idx, i;
+      for(idx=0; idx<ELEMENTS(imap_status_item_names); idx++)
+        if(g_ascii_strcasecmp(item, imap_status_item_names[idx]) == 0)
+          break;
+      for(i= 0; resp[i].item != IMSTAT_NONE; i++) {
+        if(resp[i].item == idx) {
+          sscanf(count, "%u", &resp[i].result);
+          break;
+        }
+      }
+    }
+  } while(c == ' ');
+  g_free(name);
+  /* g_return_val_if-fail(c == ')', IMR_BAD) */
+  return ir_check_crlf(h, sio_getc(h->sio));
 }
 
 static ImapResponse
@@ -1873,14 +1918,20 @@ flags_tasklet(ImapMboxHandle *h, void *data)
     h->flags_cb(1, &seqno, h->flags_arg);
 }
 
+#define CREATE_IMSG_IF_NEEDED(h,seqno) \
+  if((h)->msg_cache[seqno-1] == NULL) \
+     (h)->msg_cache[(seqno)-1] = imap_message_new();
+
 static ImapResponse
 ir_msg_att_flags(ImapMboxHandle *h, int c, unsigned seqno)
 {
   unsigned i;
-  ImapMessage *msg = h->msg_cache[seqno-1];
+  ImapMessage *msg;
   ImapFlagCache *flags;
 
   if(sio_getc(h->sio) != '(') return IMR_PROTOCOL;
+  CREATE_IMSG_IF_NEEDED(h, seqno);
+  msg = h->msg_cache[seqno-1];
   msg->flags = 0;
 
   do {
@@ -2113,9 +2164,11 @@ ir_envelope(struct siobuf *sio, ImapEnvelope *env)
 static ImapResponse
 ir_msg_att_envelope(ImapMboxHandle *h, int c, unsigned seqno)
 {
-  ImapMessage *msg = h->msg_cache[seqno-1];
+  ImapMessage *msg;
   ImapEnvelope *env;
 
+  CREATE_IMSG_IF_NEEDED(h, seqno);
+  msg = h->msg_cache[seqno-1];
   if(msg->envelope) env = NULL;
   else {
     msg->envelope = env = imap_envelope_new();
@@ -2133,7 +2186,7 @@ ir_msg_att_rfc822(ImapMboxHandle *h, int c, unsigned seqno)
 {
   gchar *str = imap_get_nstring(h->sio);
   if(h->body_cb)
-    h->body_cb(str, strlen(str), h->body_arg);
+    h->body_cb(seqno, str, strlen(str), h->body_arg);
   g_free(str);
   return IMR_OK;
 }
@@ -2152,11 +2205,15 @@ static ImapResponse
 ir_msg_att_rfc822_size(ImapMboxHandle *h, int c, unsigned seqno)
 {
   char buf[12];
-  ImapMessage *msg = h->msg_cache[seqno-1];
-  
+  ImapMessage *msg;
+
   c = imap_get_atom(h->sio, buf, sizeof(buf));
 
   if(c!= -1) sio_ungetc(h->sio);
+
+  CREATE_IMSG_IF_NEEDED(h, seqno);
+  msg = h->msg_cache[seqno-1];
+  
   msg->rfc822size = atoi(buf);  
   return IMR_OK;
 }
@@ -2770,7 +2827,8 @@ ir_body (struct siobuf *sio, int c, ImapBody * body,
 
 /* read [section] and following string. FIXME: other kinds of body. */ 
 static ImapResponse
-ir_body_section(struct siobuf *sio, ImapFetchBodyCb body_cb, void *arg)
+ir_body_section(struct siobuf *sio, unsigned seqno,
+		ImapFetchBodyCb body_cb, void *arg)
 {
   char buf[80], *str;
   int c = imap_get_atom(sio, buf, sizeof(buf));
@@ -2778,37 +2836,47 @@ ir_body_section(struct siobuf *sio, ImapFetchBodyCb body_cb, void *arg)
   if(sio_getc(sio) != ' ') { puts("space expected"); return IMR_PROTOCOL;}
   str = imap_get_nstring(sio);
   if(body_cb)
-    body_cb(str, strlen(str), arg);
+    body_cb(seqno, str, strlen(str), arg);
   g_free(str);
   return IMR_OK;
 }
 
 static ImapResponse
-ir_body_header_fields(struct siobuf* sio, ImapMessage *msg)
+ir_body_header_fields(ImapMboxHandle *h, unsigned seqno)
 {
+  ImapMessage *msg;
   char *tmp;
   int c;
 
-  if(sio_getc(sio) != '(') return IMR_PROTOCOL;
+  if(sio_getc(h->sio) != '(') return IMR_PROTOCOL;
 
-  while ((tmp = imap_get_astring(sio, &c))) {
+  while ((tmp = imap_get_astring(h->sio, &c))) {
       /* nothing (yet?) */;
     g_free(tmp);
     if (c == ')')
       break;
   }
   if(c != ')') return IMR_PROTOCOL;
-  if(sio_getc(sio) != ']') return IMR_PROTOCOL;
-  if(sio_getc(sio) != ' ') return IMR_PROTOCOL;
-  g_free(msg->fetched_header_fields);
-  msg->fetched_header_fields = imap_get_nstring(sio);
+  if(sio_getc(h->sio) != ']') return IMR_PROTOCOL;
+  if(sio_getc(h->sio) != ' ') return IMR_PROTOCOL;
+
+  tmp = imap_get_nstring(h->sio);
+  if(h->body_cb) {
+    h->body_cb(seqno, tmp, strlen(tmp), h->body_arg);
+    g_free(tmp);
+  } else {
+    CREATE_IMSG_IF_NEEDED(h, seqno);
+    msg = h->msg_cache[seqno-1];
+    g_free(msg->fetched_header_fields);
+    msg->fetched_header_fields = tmp;
+  }
   return IMR_OK;
 }
 
 static ImapResponse
 ir_msg_att_body(ImapMboxHandle *h, int c, unsigned seqno)
 {
-  ImapMessage *msg = h->msg_cache[seqno-1];
+  ImapMessage *msg;
   ImapResponse rc;
   char buf[19];	/* Just large enough to hold "HEADER.FIELDS.NOT". */
 
@@ -2817,24 +2885,26 @@ ir_msg_att_body(ImapMboxHandle *h, int c, unsigned seqno)
     c = sio_getc (h->sio);
     sio_ungetc (h->sio);
     if(isdigit (c)) {
-      rc = ir_body_section(h->sio, h->body_cb, h->body_arg);
+      rc = ir_body_section(h->sio, seqno, h->body_cb, h->body_arg);
       break;
     }
     c = imap_get_atom(h->sio, buf, sizeof buf);
     if (c == ']' &&
         g_ascii_strcasecmp(buf, "HEADER") == 0) {
       sio_ungetc (h->sio); /* put the ']' back */
-      rc = ir_body_section(h->sio, h->body_cb, h->body_arg);
+      rc = ir_body_section(h->sio, seqno, h->body_cb, h->body_arg);
     } else {
       if (c == ' ' && 
           (g_ascii_strcasecmp(buf, "HEADER.FIELDS") == 0 ||
            g_ascii_strcasecmp(buf, "HEADER.FIELDS.NOT") == 0))
-        rc = ir_body_header_fields(h->sio, msg);
+        rc = ir_body_header_fields(h, seqno);
       else
         rc = IMR_PROTOCOL;
     }
     break;
   case ' ':
+    CREATE_IMSG_IF_NEEDED(h, seqno);
+    msg = h->msg_cache[seqno-1];
     rc = ir_body(h->sio, sio_getc(h->sio),
                  msg->body ? NULL : (msg->body = imap_body_new()), 
 		 IMB_NON_EXTENSIBLE);
@@ -2847,11 +2917,13 @@ ir_msg_att_body(ImapMboxHandle *h, int c, unsigned seqno)
 static ImapResponse
 ir_msg_att_bodystructure(ImapMboxHandle *h, int c, unsigned seqno)
 {
-  ImapMessage *msg = h->msg_cache[seqno-1];
+  ImapMessage *msg;
   ImapResponse rc;
 
   switch(c) {
   case ' ':
+    CREATE_IMSG_IF_NEEDED(h, seqno);
+    msg = h->msg_cache[seqno-1];
     rc = ir_body(h->sio, sio_getc(h->sio),
                  msg->body ? NULL : (msg->body = imap_body_new()), 
 		 IMB_EXTENSIBLE);
@@ -2865,11 +2937,11 @@ static ImapResponse
 ir_msg_att_uid(ImapMboxHandle *h, int c, unsigned seqno)
 {
   char buf[12];
-  ImapMessage *msg = h->msg_cache[seqno-1];
   c = imap_get_atom(h->sio, buf, sizeof(buf));
 
   if(c!= -1) sio_ungetc(h->sio);
-  msg->uid = atoi(buf);
+  CREATE_IMSG_IF_NEEDED(h, seqno);
+  h->msg_cache[seqno-1]->uid = atoi(buf);
   return IMR_OK;
 }
 
@@ -2899,8 +2971,6 @@ ir_fetch_seq(ImapMboxHandle *h, unsigned seqno)
 
   if(seqno<1 || seqno > h->exists) return IMR_PROTOCOL;
   if(sio_getc(h->sio) != '(') return IMR_PROTOCOL;
-  if(h->msg_cache[seqno-1] == NULL)
-    h->msg_cache[seqno-1] = imap_message_new();
   do {
     for(i=0; (c = sio_getc(h->sio)) != -1; i++) {
       c = toupper(c);
