@@ -44,6 +44,7 @@
 
 #ifdef BALSA_USE_THREADS
 static pthread_t main_thread_id;
+static pthread_t libbalsa_threads_id;
 #endif
 
 #define POP_SERVER "pop"
@@ -148,14 +149,14 @@ libbalsa_guess_email_address(void)
     /* Q: Find this location with configure? or at run-time? */
     static const gchar* MAILNAME_FILE = "/etc/mailname";
     char hostbuf[512];
+    FILE *mailname_in;
 
     gchar* preset, *domain;
     if(g_getenv("EMAIL") != NULL){                  /* 1. */
         preset = g_strdup(g_getenv("EMAIL"));
-    } else if(access(MAILNAME_FILE, F_OK) == 0){    /* 2. */
-        FILE *mailname_in = fopen(MAILNAME_FILE, "r");
-        fgets(hostbuf, 511, mailname_in);
-        hostbuf[strlen(hostbuf)-1] = '\0';
+    } else if( (mailname_in = fopen(MAILNAME_FILE, "r")) != NULL){ /* 2. */
+        fgets(hostbuf, sizeof(hostbuf)-1, mailname_in);
+        hostbuf[sizeof(hostbuf)-1] = '\0';
         fclose(mailname_in);
         preset = g_strconcat(g_get_user_name(), "@", hostbuf, NULL);
         
@@ -181,8 +182,8 @@ libbalsa_guess_mail_spool(void)
     gchar *env;
     gchar *spool;
     static const gchar *guesses[] = {
-	"/var/spool/mail/",
 	"/var/mail/",
+	"/var/spool/mail/",
 	"/usr/spool/mail/",
 	"/usr/mail/",
 	NULL
@@ -422,7 +423,7 @@ static void
 x509_fingerprint (char *s, unsigned len, X509 * cert)
 {
     unsigned j, i, n, c;
-    char md[EVP_MAX_MD_SIZE];
+    unsigned char md[EVP_MAX_MD_SIZE];
 
 
     X509_digest(cert, EVP_md5(), md, &n);
@@ -680,37 +681,114 @@ static pthread_cond_t mailbox_cond = PTHREAD_COND_INITIALIZER;
 
 /* Lock/unlock a mailbox; no argument checking--we'll assume the caller
  * took care of that. 
- * We assume that this function is called for helper threads without gdk lock
- * since they usually do not have it but WITH for the main lock - which 
- * usually has it.
  */
+#define LIBBALSA_DEBUG_THREADS FALSE
 void
 libbalsa_lock_mailbox(LibBalsaMailbox * mailbox)
 {
     pthread_t thread_id = pthread_self();
-    gboolean is_main_thread =
-	(thread_id == main_thread_id);
 
     pthread_mutex_lock(&mailbox_lock);
+
     while (mailbox->lock && mailbox->thread_id != thread_id) {
-        if(is_main_thread) gdk_threads_leave();
+        gint count = 0;
+
+        while (thread_id == libbalsa_threads_id) {
+	    ++count;
+#if LIBBALSA_DEBUG_THREADS
+            g_message("Temporary gdk_threads_leave!!!");
+#endif /* LIBBALSA_DEBUG_THREADS */
+            gdk_threads_leave();
+	}
 	pthread_cond_wait(&mailbox_cond, &mailbox_lock);
-        if(is_main_thread) gdk_threads_enter();
+        while (--count >= 0) {
+            gdk_threads_enter();
+#if LIBBALSA_DEBUG_THREADS
+            g_message("...and gdk_threads_enter!!!");
+#endif /* LIBBALSA_DEBUG_THREADS */
+        }
     }
+
     /* We'll assume that no-one would destroy a mailbox while we've been
      * trying to lock it. If they have, we have larger problems than
      * this reference! */
     mailbox->lock++;
     mailbox->thread_id = thread_id;
+
     pthread_mutex_unlock(&mailbox_lock);
 }
 
 void
 libbalsa_unlock_mailbox(LibBalsaMailbox * mailbox)
 {
+#if LIBBALSA_DEBUG_THREADS
+    pthread_t self;
+
+    self = pthread_self();
+
+    if (self != mailbox->thread_id) {
+	g_warning("Not holding mailbox lock!!!");
+	return;
+    }
+
+#endif /* LIBBALSA_DEBUG_THREADS */
     pthread_mutex_lock(&mailbox_lock);
     if(!--mailbox->lock)
         pthread_cond_broadcast(&mailbox_cond);
     pthread_mutex_unlock(&mailbox_lock);
 }
+
+/* Recursive mutex for gdk_threads_{enter,leave}. */
+static pthread_mutex_t libbalsa_threads_mutex;
+static guint libbalsa_threads_lock;
+
+static void
+libbalsa_threads_enter(void)
+{
+    pthread_t self;
+
+    self = pthread_self();
+
+    if (self != libbalsa_threads_id) {
+        pthread_mutex_lock(&libbalsa_threads_mutex);
+        libbalsa_threads_id = self;
+    }
+    ++libbalsa_threads_lock;
+}
+
+static void
+libbalsa_threads_leave(void)
+{
+    pthread_t self;
+
+    self = pthread_self();
+
+#if LIBBALSA_DEBUG_THREADS
+    if (self != libbalsa_threads_id) {
+	g_warning("Not holding gdk lock!!!");
+	return;
+    }
+#endif /* LIBBALSA_DEBUG_THREADS */
+    if (--libbalsa_threads_lock == 0) {
+	if (self != main_thread_id)
+	    gdk_display_flush(gdk_display_get_default());
+        libbalsa_threads_id = 0;
+        pthread_mutex_unlock(&libbalsa_threads_mutex);
+    }
+}
+
+void
+libbalsa_threads_init(void)
+{
+    pthread_mutex_init(&libbalsa_threads_mutex, NULL);
+    gdk_threads_set_lock_functions(G_CALLBACK(libbalsa_threads_enter),
+                                   G_CALLBACK(libbalsa_threads_leave));
+}
+
+void
+libbalsa_threads_destroy(void)
+{
+    pthread_mutex_destroy(&libbalsa_threads_mutex);
+}
+
 #endif				/* BALSA_USE_THREADS */
