@@ -30,6 +30,7 @@ typedef struct _LibBalsaMessage LibBalsaMessage;
 typedef struct _LibBalsaMessageHeaders LibBalsaMessageHeaders;
 typedef struct _LibBalsaMessageBody LibBalsaMessageBody;
 typedef struct _LibBalsaServer LibBalsaServer;
+typedef struct _LibBalsaSmtpServer LibBalsaSmtpServer;
 typedef struct _LibBalsaCondition LibBalsaCondition;
 
 
@@ -130,6 +131,8 @@ void libbalsa_certs_destroy(void);
 #include <pthread.h>
 pthread_t libbalsa_get_main_thread(void);
 gboolean libbalsa_am_i_subthread(void);
+void libbalsa_threads_init(void);
+void libbalsa_threads_destroy(void);
 #else
 #define libbalsa_am_i_subthread() FALSE
 #endif /* BALSA_USE_THREADS */
