@@ -171,10 +171,22 @@ struct _LibBalsaMailboxView {
     unsigned open:1;
     unsigned in_sync:1;		/* view is in sync with config */
     unsigned frozen:1;		/* don't update view if set    */
+    unsigned used:1;		/* keep track of usage         */
 
 #ifdef HAVE_GPGME
     LibBalsaChkCryptoMode gpg_chk_mode;
 #endif
+
+    /* Display statistics:
+     * - total >= 0                both counts are valid;
+     * - total <  0 && unread == 0 unread is known to be zero;
+     * - total <  0 && unread >  0 unread is known to be > 0,
+     *                             but the count is not valid;
+     * - total <  0 && unread <  0 both are unknown.
+     */
+    int unread;
+    int total;
+    time_t mtime;       /* Mailbox mtime when counts were cached. */
 };
 
 struct _LibBalsaMailbox {
@@ -214,6 +226,7 @@ struct _LibBalsaMailbox {
                             */
     /* Associated filters (struct mailbox_filter) */
     GSList * filters;
+    gboolean filters_loaded;
 
     LibBalsaMailboxView *view;
     LibBalsaMailboxState state;
@@ -251,7 +264,7 @@ struct _LibBalsaMailboxClass {
     gboolean (*get_message_part) (LibBalsaMessage *message,
 				  LibBalsaMessageBody *part);
     GMimeStream *(*get_message_stream) (LibBalsaMailbox * mailbox,
-					LibBalsaMessage * message);
+                                        guint msgno);
 
     void (*check) (LibBalsaMailbox * mailbox);
 
@@ -286,6 +299,7 @@ struct _LibBalsaMailboxClass {
     void (*sort) (LibBalsaMailbox * mailbox, GArray *sort_array);
     gboolean (*close_backend)(LibBalsaMailbox * mailbox);
     guint (*total_messages)(LibBalsaMailbox * mailbox);
+    GArray *(*duplicate_msgnos) (LibBalsaMailbox * mailbox);
 };
 
 GType libbalsa_mailbox_get_type(void);
@@ -302,6 +316,7 @@ gboolean libbalsa_mailbox_is_open(LibBalsaMailbox *mailbox);
 void libbalsa_mailbox_close(LibBalsaMailbox * mailbox, gboolean expunge);
 
 void libbalsa_mailbox_check(LibBalsaMailbox * mailbox);
+void libbalsa_mailbox_changed(LibBalsaMailbox * mailbox);
 void libbalsa_mailbox_set_unread_messages_flag(LibBalsaMailbox * mailbox,
 					       gboolean has_unread);
 void libbalsa_mailbox_progress_notify(LibBalsaMailbox * mailbox,
@@ -348,8 +363,8 @@ void libbalsa_mailbox_release_message(LibBalsaMailbox * mailbox,
 void libbalsa_mailbox_set_msg_headers(LibBalsaMailbox * mailbox,
 				      LibBalsaMessage * message);
 
-/** libbalsa_mailbox_get_message_stream() returns an allocated block containing
-    selected, single part of the message.
+/** libbalsa_mailbox_get_message_part() ensures that a selected, single
+    part of the message is loaded.
 */
 gboolean libbalsa_mailbox_get_message_part(LibBalsaMessage    *message,
 					   LibBalsaMessageBody *part);
@@ -358,7 +373,7 @@ gboolean libbalsa_mailbox_get_message_part(LibBalsaMessage    *message,
     with full RFC822 text of the message.
 */
 GMimeStream *libbalsa_mailbox_get_message_stream(LibBalsaMailbox * mailbox,
-						 LibBalsaMessage * message);
+						 guint msgno);
 
 
 /** libbalsa_mailbox_sync_storage() asks the mailbox to synchronise
@@ -402,8 +417,6 @@ void libbalsa_mailbox_run_filters_on_reception(LibBalsaMailbox * mailbox);
 
 void libbalsa_mailbox_save_config(LibBalsaMailbox * mailbox,
 				  const gchar * prefix);
-void libbalsa_mailbox_load_config(LibBalsaMailbox * mailbox,
-				  const gchar * prefix);
 
 int libbalsa_mailbox_copy_message(LibBalsaMessage *message,
 				  LibBalsaMailbox *dest, GError **err);
@@ -428,6 +441,10 @@ gboolean libbalsa_mailbox_messages_move(LibBalsaMailbox * mailbox,
 GType libbalsa_mailbox_type_from_path(const gchar * filename);
 
 guint libbalsa_mailbox_total_messages(LibBalsaMailbox * mailbox);
+gboolean libbalsa_mailbox_can_move_duplicates(LibBalsaMailbox * mailbox);
+void libbalsa_mailbox_move_duplicates(LibBalsaMailbox * mailbox,
+                                      LibBalsaMailbox * dest,
+                                      GError ** err);
 
 /*
  * Mailbox views-related functions.
@@ -486,6 +503,9 @@ void libbalsa_mailbox_set_frozen(LibBalsaMailbox * mailbox, gboolean frozen);
 gboolean libbalsa_mailbox_set_crypto_mode(LibBalsaMailbox * mailbox,
 					  LibBalsaChkCryptoMode gpg_chk_mode);
 #endif
+void libbalsa_mailbox_set_unread(LibBalsaMailbox * mailbox, gint unread);
+void libbalsa_mailbox_set_total (LibBalsaMailbox * mailbox, gint total);
+void libbalsa_mailbox_set_mtime (LibBalsaMailbox * mailbox, time_t mtime);
 
 InternetAddressList
     *libbalsa_mailbox_get_mailing_list_address(LibBalsaMailbox * mailbox);
@@ -504,6 +524,9 @@ gboolean libbalsa_mailbox_get_frozen(LibBalsaMailbox * mailbox);
 #ifdef HAVE_GPGME
 LibBalsaChkCryptoMode libbalsa_mailbox_get_crypto_mode(LibBalsaMailbox * mailbox);
 #endif
+gint libbalsa_mailbox_get_unread(LibBalsaMailbox * mailbox);
+gint libbalsa_mailbox_get_total (LibBalsaMailbox * mailbox);
+time_t libbalsa_mailbox_get_mtime(LibBalsaMailbox * mailbox);
 
 /** force update of given msgno */
 void libbalsa_mailbox_msgno_changed(LibBalsaMailbox  *mailbox, guint seqno);
@@ -514,7 +537,8 @@ void libbalsa_mailbox_msgno_filt_out(LibBalsaMailbox * mailbox, guint seqno);
 void libbalsa_mailbox_msgno_filt_check(LibBalsaMailbox * mailbox,
 				       guint seqno,
 				       LibBalsaMailboxSearchIter
-				       * search_iter);
+				       * search_iter,
+				       gboolean hold_selected);
 
 /* Search */
 gboolean libbalsa_mailbox_msgno_find(LibBalsaMailbox * mailbox,
