@@ -58,6 +58,7 @@
 #include "imap-commands.h"
 #include "imap-server.h"
 #include "libbalsa-conf.h"
+#include "mime-stream-shared.h"
 #include "i18n.h"
 
 struct _LibBalsaMailboxImap {
@@ -70,10 +71,11 @@ struct _LibBalsaMailboxImap {
     ImapUID      uid_validity;
 
     GArray* messages_info;
-    gboolean opened;
+    GPtrArray *msgids; /* message-ids */
 
     GArray *sort_ranks;
     LibBalsaMailboxSortFields sort_field;
+    unsigned opened:1;
 };
 
 struct _LibBalsaMailboxImapClass {
@@ -101,8 +103,7 @@ static void libbalsa_mailbox_imap_close(LibBalsaMailbox * mailbox,
                                         gboolean expunge);
 static GMimeStream *libbalsa_mailbox_imap_get_message_stream(LibBalsaMailbox *
 							     mailbox,
-							     LibBalsaMessage *
-							     message);
+							     guint msgno);
 static void libbalsa_mailbox_imap_check(LibBalsaMailbox * mailbox);
 
 static void
@@ -135,6 +136,8 @@ static void libbalsa_mailbox_imap_fetch_headers(LibBalsaMailbox *mailbox,
                                                 LibBalsaMessage *message);
 static gboolean libbalsa_mailbox_imap_get_msg_part(LibBalsaMessage *msg,
 						   LibBalsaMessageBody *);
+static GArray *libbalsa_mailbox_imap_duplicate_msgnos(LibBalsaMailbox *
+						      mailbox);
 
 static int libbalsa_mailbox_imap_add_message(LibBalsaMailbox * mailbox,
 					     LibBalsaMessage * message,
@@ -185,9 +188,10 @@ static struct message_info *message_info_from_msgno(
     return msg_info;
 }
 
+#define IMAP_MSGNO_UID(mailbox, msgno) \
+        (mi_get_imsg(LIBBALSA_MAILBOX_IMAP(mailbox), (msgno))->uid)
 #define IMAP_MESSAGE_UID(msg) \
-        (mi_get_imsg(LIBBALSA_MAILBOX_IMAP((msg)->mailbox),\
-                 (msg)->msgno)->uid)
+        IMAP_MSGNO_UID((msg)->mailbox, (msg)->msgno)
 
 #define IMAP_MAILBOX_UID_VALIDITY(mailbox) (LIBBALSA_MAILBOX_IMAP(mailbox)->uid_validity)
 
@@ -259,6 +263,8 @@ libbalsa_mailbox_imap_class_init(LibBalsaMailboxImapClass * klass)
         libbalsa_mailbox_imap_get_msg_part;
     libbalsa_mailbox_class->get_message_stream =
 	libbalsa_mailbox_imap_get_message_stream;
+    libbalsa_mailbox_class->duplicate_msgnos =
+        libbalsa_mailbox_imap_duplicate_msgnos;
     libbalsa_mailbox_class->add_message = libbalsa_mailbox_imap_add_message;
     libbalsa_mailbox_class->messages_change_flags =
 	lbm_imap_messages_change_flags;
@@ -580,7 +586,7 @@ struct collect_seq_data {
     unsigned has_it;
 };
 
-static const unsigned MAX_CHUNK_LENGTH = 40; 
+static const unsigned MAX_CHUNK_LENGTH = 35; 
 static gboolean
 collect_seq_cb(GNode *node, gpointer data)
 {
@@ -747,11 +753,15 @@ imap_exists_cb(ImapMboxHandle *handle, LibBalsaMailboxImap *mimap)
                __func__, mimap->messages_info->len, cnt);
         while(cnt<mimap->messages_info->len) {
             unsigned seqno = mimap->messages_info->len;
+	    gchar *msgid;
             struct message_info *msg_info =
                 message_info_from_msgno(mimap, seqno);
-                if(msg_info->message)
+	    if(msg_info->message)
                 g_object_unref(msg_info->message);
             g_array_remove_index(mimap->messages_info, seqno-1);
+	    msgid = g_ptr_array_index(mimap->msgids, seqno-1);
+	    if(msgid) g_free(msgid);
+            g_ptr_array_remove_index(mimap->msgids, seqno-1);
         }
         ++mimap->search_stamp;
     } else if (cnt > mimap->messages_info->len) { /* new messages arrived */
@@ -764,6 +774,7 @@ imap_exists_cb(ImapMboxHandle *handle, LibBalsaMailboxImap *mimap)
 	i=mimap->messages_info->len+1;
         do {
             g_array_append_val(mimap->messages_info, a);
+            g_ptr_array_add(mimap->msgids, NULL);
 	    /* dummy entry in mindex for now */
 	    g_ptr_array_add(mailbox->mindex, NULL);
             libbalsa_mailbox_msgno_inserted(mailbox, i);
@@ -787,6 +798,7 @@ imap_expunge_cb(ImapMboxHandle *handle, unsigned seqno,
 {
     ImapMessage *imsg;
     guint i;
+    gchar *msgid;
 
     LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(mimap);
     struct message_info *msg_info = message_info_from_msgno(mimap, seqno);
@@ -812,6 +824,9 @@ imap_expunge_cb(ImapMboxHandle *handle, unsigned seqno,
     if(msg_info->message)
         g_object_unref(msg_info->message);
     g_array_remove_index(mimap->messages_info, seqno-1);
+    msgid = g_ptr_array_index(mimap->msgids, seqno-1);
+    if(msgid) g_free(msgid);
+    g_ptr_array_remove_index(mimap->msgids, seqno-1);
 
     for (i = seqno - 1; i < mimap->messages_info->len; i++) {
 	struct message_info *msg_info =
@@ -951,9 +966,11 @@ libbalsa_mailbox_imap_open(LibBalsaMailbox * mailbox, GError **err)
     mimap->messages_info = g_array_sized_new(FALSE, TRUE, 
 					     sizeof(struct message_info),
 					     total_messages);
+    mimap->msgids = g_ptr_array_sized_new(total_messages);
     for(i=0; i < total_messages; i++) {
 	struct message_info a = {0};
 	g_array_append_val(mimap->messages_info, a);
+	g_ptr_array_add(mimap->msgids, NULL);
         /* dummy entry in mindex for now */
         g_ptr_array_add(mailbox->mindex, NULL);
     }
@@ -982,16 +999,23 @@ free_messages_info(LibBalsaMailboxImap * mbox)
     guint i;
     GArray *messages_info = mbox->messages_info;
 
+    if(messages_info->len != mbox->msgids->len)
+	g_warning("free_messages_info: array sizes do not match.");
     for (i = 0; i < messages_info->len; i++) {
+	gchar *msgid;
 	struct message_info *msg_info =
 	    &g_array_index(messages_info, struct message_info, i);
 	if (msg_info->message) {
 	    msg_info->message->mailbox = NULL;
 	    g_object_unref(msg_info->message);
 	}
+	msgid = g_ptr_array_index(mbox->msgids, i);
+	if(msgid) g_free(msgid);
     }
     g_array_free(mbox->messages_info, TRUE);
     mbox->messages_info = NULL;
+    g_ptr_array_free(mbox->msgids, TRUE);
+    mbox->msgids = NULL;
 }
 
 static void
@@ -1017,9 +1041,9 @@ libbalsa_mailbox_imap_close(LibBalsaMailbox * mailbox, gboolean expunge)
 }
 
 static FILE*
-get_cache_stream(LibBalsaMailbox *mailbox, LibBalsaMessage *msg)
+get_cache_stream(LibBalsaMailbox *mailbox, guint msgno)
 {
-    unsigned uid = IMAP_MESSAGE_UID(msg);
+    unsigned uid = IMAP_MSGNO_UID(mailbox, msgno);
     LibBalsaMailboxImap *mimap = LIBBALSA_MAILBOX_IMAP(mailbox);
     FILE *stream;
     gchar **pair, *path;
@@ -1034,10 +1058,12 @@ get_cache_stream(LibBalsaMailbox *mailbox, LibBalsaMessage *msg)
 
         libbalsa_assure_balsa_dir();
         mkdir(pair[0], S_IRUSR|S_IWUSR|S_IXUSR); /* ignore errors */
+#if 0
         if(msg->length>(signed)SizeMsgThreshold)
             libbalsa_information(LIBBALSA_INFORMATION_MESSAGE, 
                                  _("Downloading %ld kB"),
                                  msg->length/1024);
+#endif
         cache = fopen(path, "wb");
         II(rc,mimap->handle,
            imap_mbox_handle_fetch_rfc822_uid(mimap->handle, uid, cache));
@@ -1057,12 +1083,11 @@ get_cache_stream(LibBalsaMailbox *mailbox, LibBalsaMessage *msg)
 */
 static GMimeStream *
 libbalsa_mailbox_imap_get_message_stream(LibBalsaMailbox * mailbox,
-					 LibBalsaMessage * message)
+					 guint msgno)
 {
     FILE *stream;
 
     g_return_val_if_fail(LIBBALSA_IS_MAILBOX_IMAP(mailbox), NULL);
-    g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), NULL);
 
     libbalsa_lock_mailbox(mailbox);
     /* this may get called when the mailbox is being opened ie,
@@ -1072,7 +1097,7 @@ libbalsa_mailbox_imap_get_message_stream(LibBalsaMailbox * mailbox,
         return NULL;
     }
 
-    stream = get_cache_stream(mailbox, message);
+    stream = get_cache_stream(mailbox, msgno);
     libbalsa_unlock_mailbox(mailbox);
 
     return g_mime_stream_file_new(stream);
@@ -1088,13 +1113,13 @@ struct mark_info {
 };
 
 static void
-lbm_imap_list_cb(ImapMboxHandle * handle, int delim, ImapMboxFlags * flags,
+lbm_imap_list_cb(ImapMboxHandle * handle, int delim, ImapMboxFlags flags,
                  char *folder, gpointer data)
 {
     struct mark_info *info = data;
 
     if (strcmp(folder, info->path) == 0
-        && IMAP_MBOX_HAS_FLAG(*flags, IMLIST_MARKED))
+        && IMAP_MBOX_HAS_FLAG(flags, IMLIST_MARKED))
         info->marked = TRUE;
 }
 
@@ -1102,27 +1127,39 @@ static gboolean
 lbm_imap_check(LibBalsaMailbox * mailbox)
 {
     LibBalsaMailboxImap *mimap = LIBBALSA_MAILBOX_IMAP(mailbox);
+    LibBalsaServer *server = LIBBALSA_MAILBOX_REMOTE_SERVER(mailbox);
     ImapMboxHandle *handle;
     gulong id;
-    struct mark_info info;
 
     handle = libbalsa_mailbox_imap_get_handle(mimap, NULL);
     if (!handle)
 	return FALSE;
 
-    info.path = mimap->path;
-    info.marked = FALSE;
-
-    id = g_signal_connect(G_OBJECT(handle), "list-response",
-                          G_CALLBACK(lbm_imap_list_cb), &info);
-
-    if (imap_mbox_list(handle, mimap->path) != IMR_OK)
+    if(libbalsa_imap_server_get_use_status(LIBBALSA_IMAP_SERVER(server))) {
+        static struct ImapStatusResult info[] = {
+            { IMSTAT_RECENT, 0 }, { IMSTAT_NONE, 0 } };
+        /* cannot do status on an open mailbox */
+        g_return_val_if_fail(!mimap->opened, FALSE);
+        if(imap_mbox_status(handle, mimap->path, info) != IMR_OK)
+            return FALSE;
+        libbalsa_mailbox_imap_release_handle(mimap);
+        return info[0].result > 0;
+    } else {
+        struct mark_info info;
+        info.path = mimap->path;
         info.marked = FALSE;
 
-    g_signal_handler_disconnect(G_OBJECT(handle), id);
-    libbalsa_mailbox_imap_release_handle(mimap);
+        id = g_signal_connect(G_OBJECT(handle), "list-response",
+                              G_CALLBACK(lbm_imap_list_cb), &info);
 
-    return info.marked;
+        if (imap_mbox_list(handle, mimap->path) != IMR_OK)
+            info.marked = FALSE;
+
+        g_signal_handler_disconnect(G_OBJECT(handle), id);
+        libbalsa_mailbox_imap_release_handle(mimap);
+
+        return info.marked;
+    }
 }
 
 static void
@@ -1546,6 +1583,7 @@ libbalsa_imap_rename_subfolder(LibBalsaMailboxImap* imap,
     ImapResult rc;
     ImapMboxHandle* handle;
     gchar *new_path;
+    char delim[2];
 
     handle = libbalsa_mailbox_imap_get_handle(imap, NULL);
     if (!handle) {
@@ -1557,8 +1595,9 @@ libbalsa_imap_rename_subfolder(LibBalsaMailboxImap* imap,
 
     II(rc,handle,
        imap_mbox_subscribe(handle, imap->path, FALSE));
-    /* FIXME: should use imap server folder separator */ 
-    new_path = g_strjoin("/", new_parent, folder, NULL);
+    delim[0] = imap_mbox_handle_get_delim(handle, new_parent);
+    delim[1] = '\0';
+    new_path = g_build_path(delim, new_parent, folder, NULL);
     rc = imap_mbox_rename(handle, imap->path, new_path);
     if (subscribe && rc == IMR_OK)
 	rc = imap_mbox_subscribe(handle, new_path, TRUE);
@@ -1594,14 +1633,11 @@ libbalsa_imap_new_subfolder(const gchar *parent, const gchar *folder,
                     _("Cannot get IMAP handle"));
 	return FALSE;
     }
-    if(parent && *parent) {
+    if (parent) {
         char delim[2];
         delim[0] = imap_mbox_handle_get_delim(handle, parent);
         delim[1] = '\0';
-        if(parent[strlen(parent)-1] == delim[0])
-            new_path = g_strconcat(parent, folder, NULL);
-        else
-            new_path = g_strjoin(delim, parent, folder, NULL);
+        new_path = g_build_path(delim, parent, folder, NULL);
     } else
         new_path = g_strdup(folder);
     II(rc,handle,
@@ -1697,12 +1733,14 @@ internet_address_new_from_imap_address(ImapAddress *addr)
     /* it will be owned by the caller */
 
     if (addr->name) {
-	gchar *tmp = g_mime_utils_header_decode_text(addr->name);
+	gchar *tmp =
+	    g_mime_utils_header_decode_text((unsigned char *) addr->name);
 	internet_address_set_name(address, tmp);
 	g_free(tmp);
     }
     if (addr->addr_spec) {
-	gchar *tmp = g_mime_utils_header_decode_text(addr->addr_spec);
+	gchar *tmp =
+	    g_mime_utils_header_decode_text((unsigned char *) addr->addr_spec);
 	internet_address_set_addr(address, tmp);
 	g_free(tmp);
     } else { /* FIXME: is that a right thing? */
@@ -1745,8 +1783,9 @@ lb_set_headers(LibBalsaMessageHeaders *headers, ImapEnvelope *  envelope,
 	internet_address_new_list_from_imap_address_list(envelope->bcc);
 
     if(is_embedded) {
-        headers->subject = 
-            g_mime_utils_header_decode_text(envelope->subject);
+        headers->subject =
+            g_mime_utils_header_decode_text((unsigned char *) envelope->
+                                            subject);
         libbalsa_utf8_sanitize(&headers->subject, TRUE, NULL);
     }
 }
@@ -1773,7 +1812,8 @@ libbalsa_mailbox_imap_load_envelope(LibBalsaMailboxImap *mimap,
 	libbalsa_message_set_headers_from_string(message, hdr);
     envelope        = imsg->envelope;
     message->length = imsg->rfc822size;
-    message->subj   = g_mime_utils_header_decode_text(envelope->subject);
+    message->subj   =
+        g_mime_utils_header_decode_text((unsigned char *) envelope->subject);
     libbalsa_utf8_sanitize(&message->subj, TRUE, NULL);
     message->sender =
 	internet_address_new_list_from_imap_address_list(envelope->sender);
@@ -1862,7 +1902,8 @@ lbm_imap_construct_body(LibBalsaMessageBody *lbbody, ImapBody *imap_body)
     str = imap_body_get_dsp_param(imap_body, "filename");
     if(!str) str = imap_body_get_param(imap_body, "name");
     if(str) {
-        lbbody->filename  = g_mime_utils_header_decode_text(str);
+        lbbody->filename  =
+	    g_mime_utils_header_decode_text((unsigned char *) str);
         libbalsa_utf8_sanitize(&lbbody->filename, TRUE, NULL);
     }
     lbbody->charset   = g_strdup(imap_body_get_param(imap_body, "charset"));
@@ -1947,7 +1988,7 @@ libbalsa_mailbox_imap_fetch_structure(LibBalsaMailbox *mailbox,
         /* we could optimize this part a little bit: we do not need to
          * keep reopening the stream. */
         GMimeStream *stream = 
-            libbalsa_mailbox_imap_get_message_stream(mailbox, message);
+            libbalsa_mailbox_imap_get_message_stream(mailbox, message->msgno);
         if(stream)
             g_object_unref(stream);
     }
@@ -2082,7 +2123,7 @@ get_section_for(LibBalsaMessage *msg, LibBalsaMessageBody *part)
 }
 struct part_data { char *block; unsigned pos; ImapBody *body; };
 static void
-append_str(const char *buf, int buflen, void *arg)
+append_str(unsigned seqno, const char *buf, int buflen, void *arg)
 {
     struct part_data *dt = (struct part_data*)arg;
 
@@ -2147,6 +2188,7 @@ lbm_imap_get_msg_part_from_cache(LibBalsaMessage * msg,
         ImapFetchBodyOptions ifbo;
         ImapMessage *im;
         ImapResponse rc;
+        LibBalsaMessageBody *parent;
 
         libbalsa_lock_mailbox(msg->mailbox);
         mimap = LIBBALSA_MAILBOX_IMAP(msg->mailbox);
@@ -2165,11 +2207,10 @@ lbm_imap_get_msg_part_from_cache(LibBalsaMessage * msg,
          * which has no headers. In this case, we have to fake them.
          * We could and probably should dump there first the headers 
          * that we have already fetched... */
-        if(strcmp(section, "1") == 0)
+        parent = get_parent(msg->body_list, part, NULL);
+        if(parent == NULL)
             ifbo = IMFB_NONE;
         else {
-            LibBalsaMessageBody *parent =
-                get_parent(msg->body_list, part, NULL);
             if(parent->body_type == LIBBALSA_MESSAGE_BODY_TYPE_MESSAGE)
                 ifbo = IMFB_HEADER;
             else
@@ -2281,6 +2322,65 @@ libbalsa_mailbox_imap_get_msg_part(LibBalsaMessage *msg,
     return lbm_imap_get_msg_part(msg, part, FALSE, NULL);
 }
 
+/* libbalsa_mailbox_imap_duplicate_msgnos: identify messages with same
+   non-empty message-ids. An efficient implementation requires that a
+   list of msgids is maintained client side. The algorithm consists of
+   four phases:
+   a). identify x=largest UID in the message-id hash.
+   b). complete the message-id hash from (possibly new) envelopes.
+   c). check for any missing message-ids by fetching the header for 
+   UID>x.
+   d). identify duplicates in the message hash. When a duplicate 
+   is encountered, keep the one with lower MSGNO/UID.
+*/
+static GArray*
+libbalsa_mailbox_imap_duplicate_msgnos(LibBalsaMailbox *mailbox)
+{
+    LibBalsaMailboxImap *mimap = LIBBALSA_MAILBOX_IMAP(mailbox);
+    unsigned first_to_fetch = 0;
+    GHashTable *dupes;
+    GArray     *res;
+    unsigned i;
+    /* a+b. */
+    for(i=mimap->msgids->len; i>=1; i--) {
+	ImapMessage *imsg;
+	gchar *msg_id = g_ptr_array_index(mimap->msgids, i-1);
+	if(msg_id) {
+	    if(!first_to_fetch)
+		first_to_fetch = i+1;
+	} else {
+	    imsg = imap_mbox_handle_get_msg(mimap->handle, i);
+	    if(imsg && imsg->envelope)
+		g_ptr_array_index(mimap->msgids, i-1) =
+		    g_strdup(imsg->envelope->message_id);
+	}
+    }
+    /* c. */
+    if(imap_mbox_complete_msgids(mimap->handle, mimap->msgids,
+				 first_to_fetch) != IMR_OK)
+	return NULL;
+    /* d. */
+    dupes = g_hash_table_new(g_str_hash, g_str_equal);
+    res   = g_array_new(FALSE, FALSE, sizeof(unsigned));
+    for(i=1; i<=mimap->msgids->len; i++) {
+	gchar *msg_id = g_ptr_array_index(mimap->msgids, i-1);
+	if(!msg_id) { /* g_warning("msgid not completed %u", i); */continue; }
+	if(!*msg_id || *msg_id == '\r' || *msg_id == '\n') continue;
+	if(!g_hash_table_lookup(dupes, msg_id))
+	    g_hash_table_insert(dupes, msg_id, GINT_TO_POINTER(1));
+	else {
+	    g_array_append_val(res, i);
+	    printf("(IMAP) Found duplicate: %u for '%s'\n", i, msg_id);
+	}
+    }
+    g_hash_table_destroy(dupes);
+    printf("total elements: %d\n", res->len);
+    for(i=0; i<res->len; i++)
+	printf("%u ", GPOINTER_TO_UINT(g_array_index(res, unsigned, i)));
+    puts("");
+    return res;
+}
+
 /* libbalsa_mailbox_imap_add_message: 
    can be called for a closed mailbox.
    Called with mailbox locked.
@@ -2331,7 +2431,7 @@ libbalsa_mailbox_imap_add_message(LibBalsaMailbox * mailbox,
     if ( LIBBALSA_MESSAGE_IS_REPLIED(message) )
 	IMSG_FLAG_SET(imap_flags,IMSGF_ANSWERED);
 
-    stream = libbalsa_mailbox_get_message_stream(message->mailbox, message);
+    stream = libbalsa_message_stream(message);
     tmpstream = g_mime_stream_filter_new_with_stream(stream);
     g_object_unref(stream);
 
@@ -2349,7 +2449,9 @@ libbalsa_mailbox_imap_add_message(LibBalsaMailbox * mailbox,
     }
 
     outstream = g_mime_stream_fs_new(outfd);
+    libbalsa_mime_stream_shared_lock(tmpstream);
     g_mime_stream_write_to_stream(tmpstream, outstream);
+    libbalsa_mime_stream_shared_unlock(tmpstream);
     g_object_unref(tmpstream);
 
     len = g_mime_stream_tell(outstream);
