@@ -82,6 +82,9 @@ static void libbalsa_mailbox_local_fetch_headers(LibBalsaMailbox *mailbox,
 static gboolean libbalsa_mailbox_local_get_msg_part(LibBalsaMessage *msg,
 						    LibBalsaMessageBody *);
 
+static GArray *libbalsa_mailbox_local_duplicate_msgnos(LibBalsaMailbox *
+                                                       mailbox);
+
 GType
 libbalsa_mailbox_local_get_type(void)
 {
@@ -152,6 +155,8 @@ libbalsa_mailbox_local_class_init(LibBalsaMailboxLocalClass * klass)
         libbalsa_mailbox_local_fetch_headers;
     libbalsa_mailbox_class->get_message_part = 
         libbalsa_mailbox_local_get_msg_part;
+    libbalsa_mailbox_class->duplicate_msgnos =
+        libbalsa_mailbox_local_duplicate_msgnos;
     klass->load_message = NULL;
     klass->remove_files = NULL;
 }
@@ -392,19 +397,15 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
 
     switch (cond->type) {
     case CONDITION_STRING:
-        if(CONDITION_CHKMATCH(cond,
-                              CONDITION_MATCH_CC|CONDITION_MATCH_BODY|
-                              CONDITION_MATCH_US_HEAD))
+        if (CONDITION_CHKMATCH
+            (cond, (CONDITION_MATCH_CC | CONDITION_MATCH_BODY))) {
             message = libbalsa_mailbox_get_message(mailbox, msgno);
-
-        if(CONDITION_CHKMATCH(cond,CONDITION_MATCH_CC) ||
-           CONDITION_CHKMATCH(cond,CONDITION_MATCH_BODY)) {
             is_refed = libbalsa_message_body_ref(message, FALSE, FALSE);
             if (!is_refed) {
                 libbalsa_information(LIBBALSA_INFORMATION_ERROR,
                                      _("Unable to load message body to "
                                        "match filter"));
-                return FALSE; /* We don't want to match if an error occured */
+                return FALSE;   /* We don't want to match if an error occured */
             }
         }
 
@@ -414,8 +415,8 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
                 match = libbalsa_utf8_strstr(entry->from,
                                              cond->match.string.string);
             else {
-                g_return_val_if_fail(!message, FALSE);
-                message = libbalsa_mailbox_get_message(mailbox, msgno);
+                if (!message)
+		    message = libbalsa_mailbox_get_message(mailbox, msgno);
                 str =
                     internet_address_list_to_string(message->headers->
                                                     to_list, FALSE);
@@ -424,19 +425,27 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
             }
             if(match) break;
 	}
-	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_FROM)) {
-            if(mailbox->view && mailbox->view->show == LB_MAILBOX_SHOW_FROM)
-                match = libbalsa_utf8_strstr(entry->from,
+        if (CONDITION_CHKMATCH(cond, CONDITION_MATCH_FROM)) {
+            if (mailbox->view
+                && mailbox->view->show == LB_MAILBOX_SHOW_FROM)
+                match =
+                    libbalsa_utf8_strstr(entry->from,
+                                         cond->match.string.string);
+            else {
+                if (!message)
+                    message = libbalsa_mailbox_get_message(mailbox, msgno);
+                if (message->headers->from) {
+                    str =
+                        internet_address_list_to_string(message->headers->
+                                                        from, FALSE);
+                    match =
+                        libbalsa_utf8_strstr(str,
                                              cond->match.string.string);
-            else if(message->headers->from) {
-                g_return_val_if_fail(message, FALSE);
-                str =
-                    internet_address_list_to_string(message->headers->
-                                                    from, FALSE);
-                match = libbalsa_utf8_strstr(str,cond->match.string.string);
-                g_free(str);
+                    g_free(str);
+                }
             }
-	    if (match) break;
+            if (match)
+                break;
         }
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_SUBJECT)) {
 	    if (libbalsa_utf8_strstr(entry->subject,
@@ -446,7 +455,7 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
             }
 	}
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_CC)) {
-            g_return_val_if_fail(is_refed, FALSE);
+            g_assert(is_refed);
             str =
                 internet_address_list_to_string(message->headers->cc_list,
                                                 FALSE);
@@ -456,7 +465,11 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
 	}
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_US_HEAD)) {
 	    if (cond->match.string.user_header) {
-		GList * header =
+		GList *header;
+
+                if (!message)
+		    message = libbalsa_mailbox_get_message(mailbox, msgno);
+		header =
 		    libbalsa_message_find_user_hdr(message,
                                                    cond->match.string
                                                    .user_header);
@@ -472,10 +485,11 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
 	}
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_BODY)) {
             GString *body;
-            g_return_val_if_fail(is_refed, FALSE);
+            g_assert(is_refed);
 	    if (!message->mailbox)
 		return FALSE; /* We don't want to match if an error occured */
-	    body = content2reply(message,NULL,0,FALSE,FALSE);
+            body =
+                content2reply(message, NULL, 0, FALSE, FALSE, NULL, NULL);
 	    if (body) {
 		if (body->str)
                     match = libbalsa_utf8_strstr(body->str,
@@ -669,7 +683,7 @@ lbm_local_update_view_filter(LibBalsaMailbox * mailbox,
     for (msgno = 1;
 	 msgno <= libbalsa_mailbox_total_messages(mailbox);
 	 msgno++)
-	libbalsa_mailbox_msgno_filt_check(mailbox, msgno, iter_view);
+	libbalsa_mailbox_msgno_filt_check(mailbox, msgno, iter_view, FALSE);
     libbalsa_mailbox_search_iter_free(iter_view);
 
     if(mailbox->view_filter)
@@ -1499,3 +1513,51 @@ libbalsa_mailbox_local_queue_sync(LibBalsaMailboxLocal * local)
                                         (GSourceFunc)lbml_sync_idle,
                                         local, NULL);
 }
+
+static GArray *
+libbalsa_mailbox_local_duplicate_msgnos(LibBalsaMailbox * mailbox)
+{
+    LibBalsaMailboxLocal *local = (LibBalsaMailboxLocal *) mailbox;
+    GHashTable *table;
+    guint i;
+    GArray *msgnos;
+
+    if (!local->threading_info)
+        return NULL;
+
+    table = g_hash_table_new(g_str_hash, g_str_equal);
+    msgnos = g_array_new(FALSE, FALSE, sizeof(guint));
+
+    for (i = 0; i < local->threading_info->len; i++) {
+        LibBalsaMailboxLocalInfo *info;
+        gpointer tmp;
+        guint master, msgno = i + 1;
+
+        if (libbalsa_mailbox_msgno_has_flags
+            (mailbox, msgno, LIBBALSA_MESSAGE_FLAG_DELETED, 0))
+            continue;
+
+        info =
+            &g_array_index(local->threading_info, LibBalsaMailboxLocalInfo,
+                           i);
+        if (!info->message_id)
+            continue;
+
+        tmp = g_hash_table_lookup(table, info->message_id);
+        master = tmp ? GPOINTER_TO_UINT(tmp) : 0;
+        if (!master ||
+            libbalsa_mailbox_msgno_has_flags(mailbox, msgno,
+                                             LIBBALSA_MESSAGE_FLAG_REPLIED,
+                                             0)) {
+            g_hash_table_insert(table, info->message_id,
+                                GUINT_TO_POINTER(msgno));
+            msgno = master;
+        }
+
+        if (msgno)
+            g_array_append_val(msgnos, msgno);
+    }
+    g_hash_table_destroy(table);
+
+    return msgnos;
+}
