@@ -36,6 +36,7 @@
 #include "libbalsa-conf.h"
 #include "misc.h"
 #include "libbalsa_private.h"
+#include "mime-stream-shared.h"
 #include "i18n.h"
 
 struct message_info {
@@ -63,8 +64,7 @@ static void libbalsa_mailbox_maildir_load_config(LibBalsaMailbox * mailbox,
 
 static GMimeStream *libbalsa_mailbox_maildir_get_message_stream(LibBalsaMailbox *
 							   mailbox,
-							   LibBalsaMessage *
-							   message);
+							   guint msgno);
 static void libbalsa_mailbox_maildir_remove_files(LibBalsaMailboxLocal *mailbox);
 
 static gboolean libbalsa_mailbox_maildir_open(LibBalsaMailbox * mailbox,
@@ -312,13 +312,13 @@ libbalsa_mailbox_maildir_load_config(LibBalsaMailbox * mailbox,
 
 static GMimeStream *
 libbalsa_mailbox_maildir_get_message_stream(LibBalsaMailbox * mailbox,
-					    LibBalsaMessage * message)
+					    guint msgno)
 {
     struct message_info *msg_info;
 
     g_return_val_if_fail(MAILBOX_OPEN(mailbox), NULL);
 
-    msg_info = message_info_from_msgno(mailbox, message->msgno);
+    msg_info = message_info_from_msgno(mailbox, msgno);
     if (!msg_info)
 	return NULL;
 
@@ -423,9 +423,14 @@ static void parse_mailbox(LibBalsaMailbox * mailbox, const gchar *subdir)
 	msg_info = g_hash_table_lookup(messages_info, key);
 	if (msg_info) {
 	    g_free(key);
+	    g_free(msg_info->filename);
+	    msg_info->filename = g_strdup(filename);
 	    if (FLAGS_REALLY_DIFFER(msg_info->orig_flags, flags)) {
+#undef DEBUG /* #define DEBUG TRUE */
+#ifdef DEBUG
 		g_message("Message flags for \"%s\" changed\n",
                           msg_info->key);
+#endif
 		msg_info->orig_flags = flags;
 	    }
 	} else {
@@ -472,14 +477,8 @@ libbalsa_mailbox_maildir_open(LibBalsaMailbox * mailbox, GError **err)
 				  NULL, (GDestroyNotify)free_message_info);
     mdir->msgno_2_msg_info = g_ptr_array_new();
 
-    mdir->mtime_cur = 0;
-    mdir->mtime_new = 0;
-    if (stat(mdir->curdir, &st) != -1) {
-	mdir->mtime_cur = st.st_mtime;
-    }
-    if (stat(mdir->newdir, &st) != -1) {
-	mdir->mtime_new = st.st_mtime;
-    }
+    if (stat(mdir->tmpdir, &st) != -1)
+	libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
 
     mailbox->readonly = 
 	!(access(mdir->curdir, W_OK) == 0 &&
@@ -526,38 +525,29 @@ lbm_maildir_check(const gchar * subdir)
 static void
 libbalsa_mailbox_maildir_check(LibBalsaMailbox * mailbox)
 {
-    struct stat st_cur, st_new;
-    int modified = 0;
+    struct stat st;
     LibBalsaMailboxMaildir *mdir;
     guint renumber, msgno;
     struct message_info *msg_info;
     const gchar *path;
+    time_t mtime;
 
     g_assert(LIBBALSA_IS_MAILBOX_MAILDIR(mailbox));
 
     mdir = LIBBALSA_MAILBOX_MAILDIR(mailbox);
 
-    if (stat(mdir->curdir, &st_cur) == -1)
+    if (stat(mdir->tmpdir, &st) == -1)
 	return;
-    if (mdir->mtime_cur == 0)
+
+    if ((mtime = libbalsa_mailbox_get_mtime(mailbox)) == 0) {
 	/* First check--just cache the mtime. */
-	mdir->mtime_cur = st_cur.st_mtime;
-    else if (st_cur.st_mtime > mdir->mtime_cur)
-	modified = 1;
-
-    if (stat(mdir->newdir, &st_new) == -1)
-	return;
-    if (mdir->mtime_new == 0)
-	/* First check--just cache the mtime. */
-	mdir->mtime_new = st_new.st_mtime;
-    else if (st_new.st_mtime > mdir->mtime_new)
-	modified = 1;
-
-    if (!modified)
+	libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
+        return;
+    }
+    if (st.st_mtime <= mtime)
 	return;
 
-    mdir->mtime_cur = st_cur.st_mtime;
-    mdir->mtime_new = st_new.st_mtime;
+    libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
 
     if (!MAILBOX_OPEN(mailbox)) {
 	libbalsa_mailbox_set_unread_messages_flag(mailbox,
@@ -634,7 +624,7 @@ libbalsa_mailbox_maildir_close_mailbox(LibBalsaMailbox * mailbox,
     }
     if (mdir->msgno_2_msg_info) {
 	if (mdir->msgno_2_msg_info->len != len)
-	    g_signal_emit_by_name(mailbox, "changed");
+	    libbalsa_mailbox_changed(mailbox);
 	g_ptr_array_free(mdir->msgno_2_msg_info, TRUE);
 	mdir->msgno_2_msg_info = NULL;
     }
@@ -643,16 +633,26 @@ libbalsa_mailbox_maildir_close_mailbox(LibBalsaMailbox * mailbox,
                                                             expunge);
 }
 
-static int libbalsa_mailbox_maildir_open_temp (const gchar *dest_path,
-					  char **name_used)
+static gchar *
+lbm_mdir_get_key(void)
+{
+    static int counter;
+
+    return g_strdup_printf("%s-%d-%d", g_get_user_name(), getpid(),
+                           counter++);
+}
+
+static int
+libbalsa_mailbox_maildir_open_temp(const gchar *dest_path,
+                                   char **name_used)
 {
     int fd;
-    static int counter;
     gchar *filename;
 
     do {
-	filename = g_strdup_printf("%s/tmp/%s-%d-%d", dest_path,
-	   		      g_get_user_name(), getpid (), counter++);
+	gchar *key = lbm_mdir_get_key();
+	filename = g_build_filename(dest_path, "tmp", key, NULL);
+	g_free(key);
 	fd = open (filename, O_WRONLY | O_EXCL | O_CREAT, 0600);
 	if (fd == -1)
 	{
@@ -672,6 +672,7 @@ maildir_sync_add(struct message_info *msg_info, const gchar * path)
     gchar new_flags[10];
     int len;
     const gchar *subdir;
+    gchar *filename;
     gchar *new;
     gchar *orig;
     gboolean retval = TRUE;
@@ -690,22 +691,39 @@ maildir_sync_add(struct message_info *msg_info, const gchar * path)
     new_flags[len] = '\0';
 
     subdir = msg_info->flags & LIBBALSA_MESSAGE_FLAG_RECENT ? "new" : "cur";
-    new = g_strdup_printf("%s/%s/%s%s%s", path, subdir, msg_info->key,
-			  (len ? ":2," : ""), new_flags);
-    orig = g_strdup_printf("%s/%s/%s", path, msg_info->subdir,
-			   msg_info->filename);
+    filename =
+        g_strconcat(msg_info->key, (len ? ":2," : ""), new_flags, NULL);
+    new = g_build_filename(path, subdir, filename, NULL);
+    orig =
+        g_build_filename(path, msg_info->subdir, msg_info->filename, NULL);
     if (strcmp(orig, new)) {
+        while (g_file_test(new, G_FILE_TEST_EXISTS)) {
+#ifdef DEBUG
+	    g_message("File \"%s\" exists, requesting new key.\n", new);
+#endif
+            g_free(msg_info->key);
+            msg_info->key = lbm_mdir_get_key();
+            g_free(filename);
+            filename =
+                g_strconcat(msg_info->key, (len ? ":2," : ""), new_flags,
+                            NULL);
+            g_free(new);
+            new = g_build_filename(path, subdir, filename, NULL);
+        }
 	if (rename(orig, new) >= 0) /* FIXME: change to safe_rename??? */
 	    msg_info->subdir = subdir;
-	else
+	else {
+#ifdef DEBUG
+            g_message("Rename \"%s\" \"%s\": %s\n", orig, new,
+                      g_strerror(errno));
+#endif
 	    retval = FALSE;
+	}
     }
     g_free(orig);
     g_free(new);
-    new = g_strdup_printf("%s%s%s", msg_info->key, (len ? ":2," : ""),
-			  new_flags);
     g_free(msg_info->filename);
-    msg_info->filename = new;
+    msg_info->filename = filename;
     msg_info->orig_flags = REAL_FLAGS(msg_info->flags);
 
     return retval;
@@ -783,13 +801,11 @@ libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	    msg_info->message->msgno = msgno;
     }
 
-    if (changes) {              /* Record mtime of dirs. */
+    if (changes) {              /* Record mtime of dir. */
         struct stat st;
 
-        stat(mdir->curdir, &st);
-        mdir->mtime_cur = st.st_mtime;
-        stat(mdir->newdir, &st);
-        mdir->mtime_new = st.st_mtime;
+        if (stat(mdir->tmpdir, &st) == 0)
+            libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
     }
 
     return TRUE;
@@ -866,6 +882,7 @@ libbalsa_mailbox_maildir_add_message(LibBalsaMailbox * mailbox,
     struct message_info *msg_info;
     gint retval;
     LibBalsaMailboxMaildir *mdir;
+    time_t mtime;
 
     /* open tempfile */
     path = libbalsa_mailbox_local_get_path(mailbox);
@@ -874,8 +891,7 @@ libbalsa_mailbox_maildir_add_message(LibBalsaMailbox * mailbox,
 	return -1;
     out_stream = g_mime_stream_fs_new(fd);
 
-    tmp_stream =
-        libbalsa_mailbox_get_message_stream(message->mailbox, message);
+    tmp_stream = libbalsa_message_stream(message);
     if (!tmp_stream) {
         g_object_unref(out_stream);
         unlink(tmp);
@@ -894,8 +910,10 @@ libbalsa_mailbox_maildir_add_message(LibBalsaMailbox * mailbox,
     g_mime_stream_filter_add(GMIME_STREAM_FILTER(in_stream), crlffilter);
     g_object_unref(crlffilter);
  
+    libbalsa_mime_stream_shared_lock(in_stream);
     if (g_mime_stream_write_to_stream( in_stream, out_stream) == -1)
     {
+        libbalsa_mime_stream_shared_unlock(in_stream);
 	g_object_unref(in_stream);
 	g_object_unref(out_stream);
 	unlink (tmp);
@@ -905,6 +923,7 @@ libbalsa_mailbox_maildir_add_message(LibBalsaMailbox * mailbox,
                     _("Data copy error"));
 	return -1;
     }
+    libbalsa_mime_stream_shared_unlock(in_stream);
     g_object_unref(in_stream);
     g_object_unref(out_stream);
 
@@ -923,11 +942,11 @@ libbalsa_mailbox_maildir_add_message(LibBalsaMailbox * mailbox,
     g_free(tmp);
 
     mdir = (LibBalsaMailboxMaildir *) mailbox;
-    if (mdir->mtime_new)
+    if ((mtime = libbalsa_mailbox_get_mtime(mailbox)) != 0)
 	/* If we checked or synced the mailbox less than 1 second ago,
 	 * the cached modification time could be the same as the new
 	 * modification time, so we'll invalidate the cached time. */
-	--mdir->mtime_new;
+	libbalsa_mailbox_set_mtime(mailbox, --mtime);
 
     return retval;
 }
