@@ -43,6 +43,7 @@
 #include "misc.h"
 /* for mx_lock_file and mx_unlock_file */
 #include "mailbackend.h"
+#include "mime-stream-shared.h"
 #include "i18n.h"
 
 struct message_info {
@@ -66,8 +67,7 @@ static void libbalsa_mailbox_mbox_init(LibBalsaMailboxMbox * mailbox);
 
 static GMimeStream *libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox *
 							     mailbox,
-							     LibBalsaMessage *
-							     message);
+							     guint msgno);
 static void libbalsa_mailbox_mbox_remove_files(LibBalsaMailboxLocal *mailbox);
 
 static gboolean libbalsa_mailbox_mbox_open(LibBalsaMailbox * mailbox,
@@ -100,6 +100,17 @@ libbalsa_mailbox_mbox_msgno_has_flags(LibBalsaMailbox * mailbox,
 static guint
 libbalsa_mailbox_mbox_total_messages(LibBalsaMailbox * mailbox);
 
+struct _LibBalsaMailboxMboxClass {
+    LibBalsaMailboxLocalClass klass;
+};
+
+struct _LibBalsaMailboxMbox {
+    LibBalsaMailboxLocal parent;
+
+    GArray* messages_info;
+    GMimeStream *gmime_stream;
+    gint size;
+};
 
 GType libbalsa_mailbox_mbox_get_type(void)
 {
@@ -170,7 +181,7 @@ libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
 
 
 static void
-libbalsa_mailbox_mbox_init(LibBalsaMailboxMbox * mailbox)
+libbalsa_mailbox_mbox_init(LibBalsaMailboxMbox * mbox)
 {
 }
 
@@ -246,24 +257,29 @@ lbm_mbox_stream_seek_to_message(GMimeStream * stream, off_t offset)
 static gboolean
 lbm_mbox_seek_to_message(LibBalsaMailboxMbox * mbox, off_t offset)
 {
-    return lbm_mbox_stream_seek_to_message(mbox->gmime_stream, offset);
+    gboolean retval;
+    
+    libbalsa_mime_stream_shared_lock(mbox->gmime_stream);
+    retval = lbm_mbox_stream_seek_to_message(mbox->gmime_stream, offset);
+    libbalsa_mime_stream_shared_unlock(mbox->gmime_stream);
+
+    return retval;
 }
 
 static GMimeStream *
 libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox *mailbox,
-                                         LibBalsaMessage *message)
+                                         guint msgno)
 {
 	GMimeStream *stream = NULL;
 	struct message_info *msg_info;
 	LibBalsaMailboxMbox *mbox;
 
 	g_return_val_if_fail (LIBBALSA_IS_MAILBOX_MBOX(mailbox), NULL);
-	g_return_val_if_fail (LIBBALSA_IS_MESSAGE(message), NULL);
 
 	mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
 
 	msg_info = &g_array_index(mbox->messages_info,
-				  struct message_info, message->msgno - 1);
+				  struct message_info, msgno - 1);
 	if (!lbm_mbox_seek_to_message(mbox, msg_info->start))
 	    return NULL;
 	stream = g_mime_stream_substream(mbox->gmime_stream,
@@ -289,22 +305,14 @@ libbalsa_mailbox_mbox_remove_files(LibBalsaMailboxLocal *mailbox)
 static int mbox_lock(LibBalsaMailbox * mailbox, GMimeStream *stream)
 {
     const gchar *path = libbalsa_mailbox_local_get_path(mailbox);
-    int fd;
-    if (stream)
-	fd = GMIME_STREAM_FS(stream)->fd;
-    else
-	fd = GMIME_STREAM_FS(LIBBALSA_MAILBOX_MBOX(mailbox)->gmime_stream)->fd;
+    int fd = GMIME_STREAM_FS(stream)->fd;
     return libbalsa_lock_file(path, fd, FALSE, TRUE, 1);
 }
 
 static void mbox_unlock(LibBalsaMailbox * mailbox, GMimeStream *stream)
 {
     const gchar *path = libbalsa_mailbox_local_get_path(mailbox);
-    int fd;
-    if (stream)
-	fd = GMIME_STREAM_FS(stream)->fd;
-    else
-	fd = GMIME_STREAM_FS(LIBBALSA_MAILBOX_MBOX(mailbox)->gmime_stream)->fd;
+    int fd = GMIME_STREAM_FS(stream)->fd;
     libbalsa_unlock_file(path, fd, 1);
 }
 
@@ -467,10 +475,12 @@ libbalsa_mailbox_mbox_open(LibBalsaMailbox * mailbox, GError **err)
 		    _("Cannot open mailbox."));
 	return FALSE;
     }
-    gmime_stream = g_mime_stream_fs_new(fd);
+    gmime_stream = libbalsa_mime_stream_shared_new(fd);
 
+    libbalsa_mime_stream_shared_lock(gmime_stream);
     if (st.st_size > 0
         && !lbm_mbox_stream_seek_to_message(gmime_stream, 0)) {
+        libbalsa_mime_stream_shared_unlock(gmime_stream);
         g_object_unref(gmime_stream);
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_OPEN_ERROR,
@@ -479,6 +489,7 @@ libbalsa_mailbox_mbox_open(LibBalsaMailbox * mailbox, GError **err)
     }
 
     if (mbox_lock(mailbox, gmime_stream)) {
+        libbalsa_mime_stream_shared_unlock(gmime_stream);
 	g_object_unref(gmime_stream);
 	g_set_error(err, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_OPEN_ERROR,
 		    _("Cannot lock mailbox."));
@@ -486,7 +497,7 @@ libbalsa_mailbox_mbox_open(LibBalsaMailbox * mailbox, GError **err)
     }
 
     mbox->size = st.st_size;
-    mbox->mtime = st.st_mtime;
+    libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
     mbox->gmime_stream = gmime_stream;
 
     mbox->messages_info =
@@ -499,7 +510,8 @@ libbalsa_mailbox_mbox_open(LibBalsaMailbox * mailbox, GError **err)
     LIBBALSA_MAILBOX_LOCAL(mailbox)->sync_time = time(NULL) - t0;
     LIBBALSA_MAILBOX_LOCAL(mailbox)->sync_cnt  = 1;
 
-    mbox_unlock(mailbox, NULL);
+    mbox_unlock(mailbox, gmime_stream);
+    libbalsa_mime_stream_shared_unlock(gmime_stream);
 #ifdef DEBUG
     g_print(_("%s: Opening %s Refcount: %d\n"),
 	    "LibBalsaMailboxMbox", mailbox->name, mailbox->open_ref);
@@ -544,7 +556,7 @@ lbm_mbox_check(LibBalsaMailbox * mailbox, const gchar * path)
 	    line->len = 0;
 	    g_mime_stream_buffer_readln(gmime_stream_buffer, line);
 	} while (!(eos = g_mime_stream_eos(gmime_stream_buffer))
-		 && !libbalsa_str_has_prefix(line->data, "From "));
+		 && !libbalsa_str_has_prefix((gchar *) line->data, "From "));
 	if (eos)
 	    break;
 
@@ -554,17 +566,18 @@ lbm_mbox_check(LibBalsaMailbox * mailbox, const gchar * path)
 	do {
 	    line->len = 0;
 	    g_mime_stream_buffer_readln(gmime_stream_buffer, line);
-	    if (g_ascii_strncasecmp(line->data, "Status: ", 8) == 0) {
-		if (strchr(line->data + 8, 'R'))
+	    if (g_ascii_strncasecmp((gchar *) line->data,
+				    "Status: ", 8) == 0) {
+		if (strchr((gchar *) line->data + 8, 'R'))
 		    new_undeleted = FALSE;
-	    } else if (g_ascii_strncasecmp(line->data, "X-Status: ", 10) ==
-		       0) {
-		if (strchr(line->data + 10, 'D'))
+	    } else if (g_ascii_strncasecmp((gchar *) line->data,
+				           "X-Status: ", 10) == 0) {
+		if (strchr((gchar *) line->data + 10, 'D'))
 		    new_undeleted = FALSE;
 	    } else
-		if (g_ascii_strncasecmp(line->data, "Content-Length: ", 16)
-		    == 0) {
-		content_length = atoi(line->data + 16);
+		if (g_ascii_strncasecmp((gchar *) line->data,
+					"Content-Length: ", 16) == 0) {
+		content_length = atoi((gchar *) line->data + 16);
 	    }
 	    /* Blank line ends headers. */
 	} while (!(eos = g_mime_stream_eos(gmime_stream_buffer))
@@ -598,7 +611,9 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
     struct stat st;
     const gchar *path;
     LibBalsaMailboxMbox *mbox;
+    GMimeStream *mbox_stream;
     guint msgno;
+    time_t mtime;
 
     g_assert(LIBBALSA_IS_MAILBOX_MBOX(mailbox));
 
@@ -611,16 +626,17 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
 	return;
     }
 
-    if (mbox->mtime == 0) {
+    mtime = libbalsa_mailbox_get_mtime(mailbox);
+    if (mtime == 0) {
 	/* First check--just cache the mtime and size. */
-	mbox->mtime = st.st_mtime;
+        libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
 	mbox->size = st.st_size;
 	return;
     }
-    if (st.st_mtime == mbox->mtime && st.st_size == mbox->size)
+    if (st.st_mtime == mtime && st.st_size == mbox->size)
 	return;
 
-    mbox->mtime = st.st_mtime;
+    libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
 
     if (!MAILBOX_OPEN(mailbox)) {
 	libbalsa_mailbox_set_unread_messages_flag(mailbox,
@@ -631,7 +647,8 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
 	return;
     }
 
-    if (mbox_lock(mailbox, NULL) != 0)
+    mbox_stream = mbox->gmime_stream;
+    if (mbox_lock(mailbox, mbox_stream) != 0)
 	/* we couldn't lock the mailbox, but nothing serious happened:
 	 * probably the new mail arrived: no reason to wait till we can
 	 * parse it: we'll get it on the next pass
@@ -644,12 +661,17 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
      * the folder.  If a message was expunged by another MUA, we back up
      * over messages until we find one that is still where we expected
      * to find it, and start parsing at its end.
+     * We must lock the mime-stream for the whole process, as
+     * parse_mailbox assumes that the stream is positioned at the first
+     * message to be parsed.
      */
+    libbalsa_mime_stream_shared_lock(mbox_stream);
     while ((msgno = mbox->messages_info->len) > 0) {
+	off_t offset;
         struct message_info *msg_info =
             &g_array_index(mbox->messages_info, struct message_info,
                            msgno - 1);
-        if (lbm_mbox_seek_to_message(mbox, msg_info->end))
+        if (lbm_mbox_stream_seek_to_message(mbox_stream, msg_info->end))
 	    /* A message begins at the end of this one, so it must(?) be
 	     * in its original position--start parsing here. */
             break;
@@ -658,15 +680,27 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
         if ((msg_info->flags & LIBBALSA_MESSAGE_FLAG_NEW)
             && !(msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED))
             --mailbox->unread_messages;
+
+	/* We must drop the mime-stream lock to call
+	 * libbalsa_mailbox_local_msgno_removed(), as it will grab the
+	 * gdk lock to emit gtk signals; we save and restore the current
+	 * stream position, in case someone changes it while we're not
+	 * holding the lock. */
+        offset = g_mime_stream_tell(mbox_stream);
+        libbalsa_mime_stream_shared_unlock(mbox_stream);
         libbalsa_mailbox_local_msgno_removed(mailbox, msgno);
+        libbalsa_mime_stream_shared_lock(mbox_stream);
+        g_mime_stream_seek(mbox_stream, offset, GMIME_STREAM_SEEK_SET);
+
         free_message_info(msg_info);
         g_array_remove_index(mbox->messages_info, msgno - 1);
     }
     if(msgno == 0)
-        g_mime_stream_seek(mbox->gmime_stream, 0, GMIME_STREAM_SEEK_SET);
+        g_mime_stream_seek(mbox_stream, 0, GMIME_STREAM_SEEK_SET);
     parse_mailbox(mbox);
-    mbox->size = g_mime_stream_tell(mbox->gmime_stream);
-    mbox_unlock(mailbox, NULL);
+    mbox->size = g_mime_stream_tell(mbox_stream);
+    libbalsa_mime_stream_shared_unlock(mbox_stream);
+    mbox_unlock(mailbox, mbox_stream);
     libbalsa_mailbox_local_load_messages(mailbox, msgno);
 }
 
@@ -682,7 +716,7 @@ libbalsa_mailbox_mbox_close_mailbox(LibBalsaMailbox * mailbox,
 	len = mbox->messages_info->len;
         libbalsa_mailbox_mbox_sync(mailbox, expunge);
 	if (mbox->messages_info->len != len)
-	    g_signal_emit_by_name(mailbox, "changed");
+	    libbalsa_mailbox_changed(mailbox);
 	free_messages_info(mbox->messages_info);
 	mbox->messages_info = NULL;
     }
@@ -697,20 +731,22 @@ libbalsa_mailbox_mbox_close_mailbox(LibBalsaMailbox * mailbox,
 
 static GMimeMessage *
 lbm_mbox_get_mime_message(LibBalsaMailbox * mailbox,
-			  LibBalsaMessage * message)
+			  guint msgno)
 {
     GMimeStream *stream;
     GMimeParser *parser;
     GMimeMessage *mime_message;
 
-    stream = libbalsa_mailbox_mbox_get_message_stream(mailbox, message);
+    stream = libbalsa_mailbox_mbox_get_message_stream(mailbox, msgno);
     if (!stream)
 	return NULL;
+    libbalsa_mime_stream_shared_lock(stream);
     parser = g_mime_parser_new_with_stream(stream);
-    g_object_unref(stream);
 
     mime_message = g_mime_parser_construct_message(parser);
     g_object_unref(parser);
+    libbalsa_mime_stream_shared_unlock(stream);
+    g_object_unref(stream);
 
     return mime_message;
 }
@@ -821,7 +857,7 @@ lbm_mbox_rewrite_helper(off_t offset, GMimeStream * stream,
 /* Rewrite message status headers in place, if possible.
  *
  * msg_info:	struct message_info for the message;
- * stream:	mbox stream.
+ * stream:	mbox stream--must be locked by caller.
  *
  * Returns TRUE if it was possible to rewrite in place,
  * 	   FALSE otherwise.
@@ -876,34 +912,39 @@ lbm_mbox_rewrite_in_place(struct message_info *msg_info,
 /* Length of the line beginning at offset, including trailing '\n'.
  * Returns -1 if no '\n' found, or if seek to offset fails. */
 static gint
-lbm_mbox_line_len(GMimeStream * stream, off_t offset)
+lbm_mbox_line_len(LibBalsaMailboxMbox * mbox, off_t offset)
 {
-    gchar buf[80];
-    gint old;
+    GMimeStream *stream = mbox->gmime_stream;
+    gint retval = -1;
 
-    if (g_mime_stream_seek(stream, offset, GMIME_STREAM_SEEK_SET) == -1)
-	return -1;
+    libbalsa_mime_stream_shared_lock(mbox->gmime_stream);
+    if (g_mime_stream_seek(stream, offset, GMIME_STREAM_SEEK_SET) >= 0) {
+        gint old = 0;
+        while (retval < 0) {
+            gint i, len;
+            gchar buf[80];
 
-    old = 0;
-    for (;;) {
-	gint i, len;
+            len = g_mime_stream_read(stream, buf, sizeof buf);
+            if (len <= 0)
+                break;
 
-	len = g_mime_stream_read(stream, buf, sizeof buf);
-	if (len <= 0)
-	    return -1;
-
-	i = 0;
-	do 
-	    if (buf[i++] == '\n')
-		return old + i;
-	while (i < len);
-
-	old += len;
+            i = 0;
+            do
+                if (buf[i++] == '\n') {
+                    retval = old + i;
+                    break;
+                }
+            while (i < len);
+            old += len;
+        }
     }
+    libbalsa_mime_stream_shared_unlock(mbox->gmime_stream);
+
+    return retval;
 }
 
 static gboolean
-lbm_mbox_copy_stream(GMimeStream * src, off_t start, off_t end,
+lbm_mbox_copy_stream(LibBalsaMailboxMbox * mbox, off_t start, off_t end,
 		     GMimeStream * dest)
 {
     GMimeStream *substream;
@@ -912,8 +953,10 @@ lbm_mbox_copy_stream(GMimeStream * src, off_t start, off_t end,
     if (start >= end)
 	return TRUE;
 
-    substream = g_mime_stream_substream(src, start, end);
+    substream = g_mime_stream_substream(mbox->gmime_stream, start, end);
+    libbalsa_mime_stream_shared_lock(substream);
     retval = g_mime_stream_write_to_stream(substream, dest) == end - start;
+    libbalsa_mime_stream_shared_unlock(substream);
     g_object_unref(substream);
 
     return retval;
@@ -1013,10 +1056,15 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	    first = i;
         if ((msg_info->orig_flags ^ msg_info->flags)
             & LIBBALSA_MESSAGE_FLAGS_REAL) {
-	    if (lbm_mbox_rewrite_in_place(msg_info, mbox_stream))
-		++j;
-	    else
+	    gboolean can_rewrite_in_place;
+
+	    libbalsa_mime_stream_shared_lock(mbox_stream);
+	    can_rewrite_in_place =
+		lbm_mbox_rewrite_in_place(msg_info, mbox_stream);
+	    libbalsa_mime_stream_shared_unlock(mbox_stream);
+	    if (!can_rewrite_in_place)
 		break;
+	    ++j;
 	}
     }
     if (i >= messages) {
@@ -1032,7 +1080,7 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	if (fstat(GMIME_STREAM_FS(mbox_stream)->fd, &st))
 	    g_warning("can't stat \"%s\"", path);
 	else
-	    mbox->mtime = st.st_mtime;
+            libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
 	mbox_unlock(mailbox, mbox_stream);
 	return TRUE;
     }
@@ -1064,7 +1112,7 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	    continue;
 
 	if (msg_info->status >= 0) {
-	    status_len = lbm_mbox_line_len(mbox_stream, msg_info->status);
+	    status_len = lbm_mbox_line_len(mbox, msg_info->status);
 	    if (status_len < 0)
 		break;
 	} else {
@@ -1076,8 +1124,7 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	}
 
 	if (msg_info->x_status >= 0) {
-	    x_status_len =
-		lbm_mbox_line_len(mbox_stream, msg_info->x_status);
+	    x_status_len = lbm_mbox_line_len(mbox, msg_info->x_status);
 	    if (x_status_len < 0)
 		break;
 	} else {
@@ -1086,35 +1133,36 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	}
 
 	if (msg_info->status <= msg_info->x_status) {
-	    if (!lbm_mbox_copy_stream(mbox_stream, msg_info->start,
+	    if (!lbm_mbox_copy_stream(mbox, msg_info->start,
 				      msg_info->status, temp_stream)
 		|| !lbm_mbox_write_status_hdr(temp_stream, msg_info->flags)
-		|| !lbm_mbox_copy_stream(mbox_stream,
+		|| !lbm_mbox_copy_stream(mbox,
 					 msg_info->status + status_len,
 					 msg_info->x_status, temp_stream)
 		|| !lbm_mbox_write_x_status_hdr(temp_stream,
 						msg_info->flags)
-		|| !lbm_mbox_copy_stream(mbox_stream,
+		|| !lbm_mbox_copy_stream(mbox,
 					 msg_info->x_status +
 					 x_status_len, msg_info->end,
 					 temp_stream))
 		break;
 	} else {
-	    if (!lbm_mbox_copy_stream(mbox_stream, msg_info->start,
+	    if (!lbm_mbox_copy_stream(mbox, msg_info->start,
 				      msg_info->x_status, temp_stream)
 		|| !lbm_mbox_write_x_status_hdr(temp_stream,
 						msg_info->flags)
-		|| !lbm_mbox_copy_stream(mbox_stream,
+		|| !lbm_mbox_copy_stream(mbox,
 					 msg_info->x_status +
 					 x_status_len,
 					 msg_info->status, temp_stream)
 		|| !lbm_mbox_write_status_hdr(temp_stream, msg_info->flags)
-		|| !lbm_mbox_copy_stream(mbox_stream,
+		|| !lbm_mbox_copy_stream(mbox,
 					 msg_info->status + status_len,
 					 msg_info->end, temp_stream))
 		break;
 	}
     }
+
     if (i < messages) {
 	/* We broke on an error. */
 	g_warning("error making temporary copy\n");
@@ -1137,9 +1185,10 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     }
 
     save_failed = TRUE;
+    libbalsa_mime_stream_shared_lock(mbox_stream);
     if (g_mime_stream_reset(temp_stream) == -1) {
         g_warning("mbox_sync: can't rewind temporary copy.\n");
-    } else if (!lbm_mbox_seek_to_message(mbox, offset))
+    } else if (!lbm_mbox_stream_seek_to_message(mbox_stream, offset))
         g_warning("mbox_sync: message not in expected position.\n");
     else if (g_mime_stream_write_to_stream(temp_stream, mbox_stream) != -1) {
         save_failed = FALSE;
@@ -1148,8 +1197,10 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     }
     g_object_unref(temp_stream);
     mbox_unlock(mailbox, mbox_stream);
-    if (g_mime_stream_flush(mbox_stream) == -1 || save_failed)
-    {
+    if (g_mime_stream_flush(mbox_stream) == -1)
+        save_failed = TRUE;
+    libbalsa_mime_stream_shared_unlock(mbox_stream);
+    if (save_failed) {
 	/*
 	 * error occured while writing the mailbox back,
 	 * so keep the temp copy around
@@ -1198,9 +1249,11 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     }
 
     /* update the rewritten messages */
+    libbalsa_mime_stream_shared_lock(mbox_stream);
     if (g_mime_stream_seek(mbox_stream, offset, GMIME_STREAM_SEEK_SET)
 	== -1) {
 	g_warning("Can't update message info");
+	libbalsa_mime_stream_shared_unlock(mbox_stream);
 	return FALSE;
     }
     gmime_parser = g_mime_parser_new_with_stream(mbox_stream);
@@ -1216,7 +1269,16 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	msg_info =
 	    &g_array_index(mbox->messages_info, struct message_info, j);
 	if (expunge && (msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
+	    /* We must drop the mime-stream lock to call
+	     * libbalsa_mailbox_local_msgno_removed(), as it will grab
+	     * the gdk lock to emit gtk signals; we save and restore the
+	     * current file position, in case someone changes it while
+	     * we're not holding the lock. */
+	    offset = g_mime_stream_tell(mbox_stream);
+	    libbalsa_mime_stream_shared_unlock(mbox_stream);
 	    libbalsa_mailbox_local_msgno_removed(mailbox, j + 1);
+	    libbalsa_mime_stream_shared_lock(mbox_stream);
+	    g_mime_stream_seek(mbox_stream, offset, GMIME_STREAM_SEEK_SET);
 	    free_message_info(msg_info);
 	    g_array_remove_index(mbox->messages_info, j);
 	    continue;
@@ -1255,6 +1317,7 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 						mime_msg->mime_part);
 	}
     }
+    libbalsa_mime_stream_shared_unlock(mbox_stream);
     mbox->messages_info->len = j;
     g_object_unref(G_OBJECT(gmime_parser));
 
@@ -1264,27 +1327,18 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 static LibBalsaMessage*
 libbalsa_mailbox_mbox_get_message(LibBalsaMailbox * mailbox, guint msgno)
 {
-    GMimeMessage *mime_message;
-    GMimeParser *gmime_parser;
     LibBalsaMailboxMbox *mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
     struct message_info *msg_info;
 
     msg_info = &g_array_index(mbox->messages_info,
 			      struct message_info, msgno-1);
     if(!msg_info->message) {
-	g_mime_stream_seek(mbox->gmime_stream, msg_info->start,
-			   GMIME_STREAM_SEEK_SET);
-	gmime_parser = g_mime_parser_new_with_stream(mbox->gmime_stream);
-	g_mime_parser_set_scan_from(gmime_parser, TRUE);
-	g_mime_parser_set_respect_content_length(gmime_parser, TRUE);
-	mime_message = g_mime_parser_construct_message(gmime_parser);
-	msg_info->message = lbm_mbox_message_new(mime_message, msg_info);
+	msg_info->message = libbalsa_message_new();
 	msg_info->message->mailbox = mailbox;
 	msg_info->message->msgno   = msgno;
 	msg_info->message->flags   =
 	    msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
-	g_object_unref(mime_message);
-	g_object_unref(gmime_parser);
+        libbalsa_message_load_envelope(msg_info->message);
 	g_object_weak_ref(G_OBJECT(msg_info->message), mbox_msg_unref,
 			  mbox);
     } else g_object_ref(msg_info->message);
@@ -1297,7 +1351,7 @@ libbalsa_mailbox_mbox_fetch_message_structure(LibBalsaMailbox * mailbox,
 					      LibBalsaFetchFlag flags)
 {
     if (!message->mime_msg)
-	message->mime_msg = lbm_mbox_get_mime_message(mailbox, message);
+	message->mime_msg = lbm_mbox_get_mime_message(mailbox, message->msgno);
 
     return LIBBALSA_MAILBOX_CLASS(parent_class)->
         fetch_message_structure(mailbox, message, flags);
@@ -1523,7 +1577,7 @@ libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
     }
     mbox_lock ( mailbox, dest );
 
-    orig = libbalsa_mailbox_get_message_stream(message->mailbox, message);
+    orig = libbalsa_message_stream(message);
     if (!orig) {
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_APPEND_ERROR,
@@ -1536,8 +1590,8 @@ libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
     }
 
     /* From_ armor */
+    libbalsa_mime_stream_shared_lock(orig);
     armored_object = lbm_mbox_armored_object(orig);
-    g_object_unref(orig);
     /* Make sure we have "Status" and "X-Status" headers, so we can
      * update them in place later, if necessary. */
     update_message_status_headers(GMIME_MESSAGE(armored_object),
@@ -1554,6 +1608,8 @@ libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
     }
     g_free(from);
     g_object_unref(armored_object);
+    libbalsa_mime_stream_shared_unlock(orig);
+    g_object_unref(orig);
     g_object_unref(armored_dest);
 
     if (retval > 0)
