@@ -38,6 +38,7 @@
 #include "libbalsa.h"
 #include "misc.h"
 #include "libbalsa_private.h"
+#include "mime-stream-shared.h"
 #include "i18n.h"
 
 struct message_info {
@@ -55,8 +56,7 @@ static void libbalsa_mailbox_mh_finalize(GObject * object);
 
 static GMimeStream *libbalsa_mailbox_mh_get_message_stream(LibBalsaMailbox *
 							   mailbox,
-							   LibBalsaMessage *
-							   message);
+							   guint msgno);
 static void libbalsa_mailbox_mh_remove_files(LibBalsaMailboxLocal *mailbox);
 
 static gboolean libbalsa_mailbox_mh_open(LibBalsaMailbox * mailbox,
@@ -248,7 +248,7 @@ libbalsa_mailbox_mh_finalize(GObject * object)
 
 static GMimeStream *
 libbalsa_mailbox_mh_get_message_stream(LibBalsaMailbox * mailbox,
-				       LibBalsaMessage * message)
+				       guint msgno)
 {
     GMimeStream *stream;
     struct message_info *msg_info;
@@ -257,7 +257,7 @@ libbalsa_mailbox_mh_get_message_stream(LibBalsaMailbox * mailbox,
     g_return_val_if_fail(MAILBOX_OPEN(mailbox), NULL);
 
     msg_info = lbm_mh_message_info_from_msgno(LIBBALSA_MAILBOX_MH(mailbox),
-					      message->msgno);
+					      msgno);
     tmp = MH_BASENAME(msg_info);
     stream = libbalsa_mailbox_local_get_message_stream(mailbox, tmp, NULL);
     g_free(tmp);
@@ -448,10 +448,12 @@ lbm_mh_parse_sequences(LibBalsaMailboxMh * mailbox)
     g_object_unref(gmime_stream);
     line = g_byte_array_new();
     do {
-	g_mime_stream_buffer_readln(gmime_stream_buffer, line);
-	g_byte_array_append(line, "", 1);
+	guint8 zero = 0;
 
-	lbm_mh_handle_seq_line(mailbox, line->data);
+	g_mime_stream_buffer_readln(gmime_stream_buffer, line);
+	g_byte_array_append(line, &zero, 1);
+
+	lbm_mh_handle_seq_line(mailbox, (gchar *) line->data);
 	line->len = 0;
     } while (!g_mime_stream_eos(gmime_stream_buffer));
     g_object_unref(gmime_stream_buffer);
@@ -495,7 +497,7 @@ libbalsa_mailbox_mh_open(LibBalsaMailbox * mailbox, GError **err)
 	return FALSE;
     }
 
-    mh->mtime = st.st_mtime;
+    libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
 
     mh->messages_info =
 	g_hash_table_new_full(NULL, NULL, NULL,
@@ -538,9 +540,10 @@ lbm_mh_check(LibBalsaMailboxMh * mh, const gchar * path)
 	line->len = 0;
 	g_mime_stream_buffer_readln(gmime_stream_buffer, line);
 
-	if (libbalsa_str_has_prefix(line->data, LibBalsaMailboxMhUnseen)) {
+	if (libbalsa_str_has_prefix((gchar *) line->data,
+				    LibBalsaMailboxMhUnseen)) {
 	    /* Found the "unseen: " line... */
-	    gchar *p = line->data + strlen(LibBalsaMailboxMhUnseen);
+	    gchar *p = (gchar *) line->data + strlen(LibBalsaMailboxMhUnseen);
 	    gchar **sequences, **seq;
 	    
 	    sequences = g_strsplit(p, " ", 0);
@@ -588,6 +591,7 @@ libbalsa_mailbox_mh_check(LibBalsaMailbox * mailbox)
     int modified = 0;
     guint renumber, msgno;
     struct message_info *msg_info;
+    time_t mtime;
 
     if (stat(path, &st) == -1)
 	return;
@@ -610,10 +614,11 @@ libbalsa_mailbox_mh_check(LibBalsaMailbox * mailbox)
 	    modified = 1;
     }
 
-    if (mh->mtime == 0)
+    mtime = libbalsa_mailbox_get_mtime(mailbox);
+    if (mtime == 0)
 	/* First check--just cache the mtime. */
-	mh->mtime = st.st_mtime;
-    else if (st.st_mtime > mh->mtime)
+        libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
+    else if (st.st_mtime > mtime)
 	modified = 1;
 
     if (mh->mtime_sequences == 0)
@@ -625,7 +630,7 @@ libbalsa_mailbox_mh_check(LibBalsaMailbox * mailbox)
     if (!modified)
 	return;
 
-    mh->mtime = st.st_mtime;
+    libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
     mh->mtime_sequences = st_sequences.st_mtime;
 
     if (!MAILBOX_OPEN(mailbox)) {
@@ -684,7 +689,7 @@ libbalsa_mailbox_mh_close_mailbox(LibBalsaMailbox * mailbox,
     }
     if (mh->msgno_2_msg_info) {
 	if (mh->msgno_2_msg_info->len != len)
-	    g_signal_emit_by_name(mailbox, "changed");
+	    libbalsa_mailbox_changed(mailbox);
 	g_ptr_array_free(mh->msgno_2_msg_info, TRUE);
 	mh->msgno_2_msg_info = NULL;
     }
@@ -899,16 +904,19 @@ libbalsa_mailbox_mh_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     g_object_unref(gmime_stream);
     line = g_byte_array_new();
     do {
+	gchar *tmp;
+
 	line->len = 0;
 	g_mime_stream_buffer_readln(gmime_stream_buffer, line);
-	if (line->data &&
-	    !libbalsa_str_has_prefix(line->data, LibBalsaMailboxMhUnseen) &&
-	    !libbalsa_str_has_prefix(line->data, LibBalsaMailboxMhFlagged) &&
-	    !libbalsa_str_has_prefix(line->data, LibBalsaMailboxMhReplied) &&
-	    !libbalsa_str_has_prefix(line->data, LibBalsaMailboxMhRecent))
+	tmp = (gchar *) line->data;
+	if (tmp &&
+	    !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhUnseen) &&
+	    !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhFlagged) &&
+	    !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhReplied) &&
+	    !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhRecent))
 	{
 	    /* unknown sequence */
-	    g_mime_stream_write(temp_stream, line->data, line->len);
+	    g_mime_stream_write(temp_stream, tmp, line->len);
 	}
     } while (!g_mime_stream_eos(gmime_stream_buffer));
     g_object_unref(gmime_stream_buffer);
@@ -950,7 +958,7 @@ libbalsa_mailbox_mh_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     /* Record the mtimes; we'll just use the current time--someone else
      * might have changed something since we did, despite the file
      * locking, but we'll find out eventually. */
-    mh->mtime = mh->mtime_sequences = time(NULL);
+    libbalsa_mailbox_set_mtime(mailbox, mh->mtime_sequences = time(NULL));
 
     libbalsa_unlock_file(sequences_filename, sequences_fd, 1);
     return retval;
@@ -1079,7 +1087,7 @@ libbalsa_mailbox_mh_add_message(LibBalsaMailbox * mailbox,
     }
     out_stream = g_mime_stream_fs_new(fd);
     {
-	GMimeStream *tmp = libbalsa_mailbox_get_message_stream( message->mailbox, message );
+	GMimeStream *tmp = libbalsa_message_stream(message);
 	GMimeFilter *crlffilter = 
 	    g_mime_filter_crlf_new (  GMIME_FILTER_CRLF_DECODE,
 				      GMIME_FILTER_CRLF_MODE_CRLF_ONLY );
@@ -1088,8 +1096,11 @@ libbalsa_mailbox_mh_add_message(LibBalsaMailbox * mailbox,
 	g_object_unref (crlffilter);
 	g_object_unref (tmp);
     }    
+    libbalsa_mime_stream_shared_lock(in_stream);
     if (g_mime_stream_write_to_stream( in_stream, out_stream) == -1)
     {
+        libbalsa_mime_stream_shared_unlock(in_stream);
+        g_object_unref(in_stream);
 	g_object_unref(out_stream);
 	unlink (tmp);
 	g_free(tmp);
@@ -1099,6 +1110,7 @@ libbalsa_mailbox_mh_add_message(LibBalsaMailbox * mailbox,
 	return -1;
     }
     g_object_unref(out_stream);
+    libbalsa_mime_stream_shared_unlock(in_stream);
     g_object_unref(in_stream);
 
     fileno = mh->last_fileno; 
