@@ -43,6 +43,7 @@
 
 /* needed for truncate_string */
 #include "misc.h"
+#include "mime-stream-shared.h"
 #include "i18n.h"
 
 #include <gmime/gmime.h>
@@ -181,19 +182,6 @@ libbalsa_message_finalize(GObject * object)
     /* printf("%p message finalized.\n", message); */  message_cnt--;
 }
 
-static void
-libbalsa_message_find_charset(GMimeObject * mime_part,
-                              const gchar ** charset)
-{
-    const GMimeContentType *type;
-
-    if (*charset)
-        return;
-
-    type = g_mime_object_get_content_type(mime_part);
-    *charset = g_mime_content_type_get_parameter(type, "charset");
-}
-
 static void
 lb_message_headers_extra_destroy(LibBalsaMessageHeaders * headers)
 {
@@ -256,57 +244,29 @@ libbalsa_message_headers_destroy(LibBalsaMessageHeaders * headers)
 const gchar *
 libbalsa_message_body_charset(LibBalsaMessageBody * body)
 {
-    const gchar *charset = NULL;
+    const gchar *charset;
+
+    if (!body)
+	return NULL;
 
     if (body->charset) /* This overrides all! Important for non
                         * us-ascii messages over IMAP. */
         return body->charset;
-    if (body->mime_part) {
-	if (GMIME_IS_MULTIPART(body->mime_part))
-	    g_mime_multipart_foreach(GMIME_MULTIPART(body->mime_part),
-				     (GMimePartFunc)
-				     libbalsa_message_find_charset, &charset);
-	else
-	    libbalsa_message_find_charset(body->mime_part, &charset);
-    } else {
-	do {
-	    if (body->charset) {
-		charset = body->charset;
-		break;
-	    }
-	    if (body->parts)
-		charset = libbalsa_message_body_charset(body->parts);
-	} while (!charset && (body = body->next));
+
+    if (GMIME_IS_PART(body->mime_part)) {
+        const GMimeContentType *type;
+
+        type = g_mime_object_get_content_type(body->mime_part);
+        return g_mime_content_type_get_parameter(type, "charset");
     }
 
-    return charset;
+    charset = libbalsa_message_body_charset(body->parts);
+    if (charset)
+        return charset;
+
+    return libbalsa_message_body_charset(body->next);
 }
 
-/* Note: libbalsa_message_charset returns a pointer to a newly allocated
- * string containing the canonical form of the charset field, or NULL.
- * When the pointer is nonNULL, the string must be deallocated with
- * g_free. */
-gchar *
-libbalsa_message_charset(LibBalsaMessage * message)
-{
-    LibBalsaMessageBody *body;
-    const gchar *charset;
-    const char *tmp;
-    g_return_val_if_fail(message != NULL, NULL);
-    body = message->body_list;
-    g_return_val_if_fail(body != NULL, NULL);
-
-    charset = body->charset;
-    if (!charset) {
-        charset = libbalsa_message_body_charset(body);
-        if (!charset)
-            return NULL;
-    }
-    tmp = g_mime_charset_canon_name(charset);
-    return g_strdup(tmp);
-}
-
-
 static void
 canonize_header_value(gchar *value)
 {
@@ -386,11 +346,12 @@ prepend_header_misc(const char *name, const char *value,
     /* Standard Headers*/
     for(i=0; name[i] && i<sizeof(lcname)-1; i++)
         lcname[i] = tolower(name[i]);
-    if(name[i]) /* too long to be on the ignored-headers list */
-        return;
-    lcname[i] = '\0';
-    if(strstr(ignored_headers, lcname))
-        return;
+    if (i < sizeof(lcname)) {
+	/* short enough to be on the ignored-headers list */
+        lcname[i] = '\0';
+        if(strstr(ignored_headers, lcname))
+            return;
+    }
 
     res = g_list_prepend(res, libbalsa_create_hdr_pair(name, g_strdup(value)));
     *(GList **)user_data = res;
@@ -444,8 +405,12 @@ libbalsa_message_user_hdrs_from_gmime(GMimeMessage * message)
 
     value = g_mime_message_get_header(message, "In-Reply-To");
     if (value) {
-	res = g_list_prepend(res, libbalsa_create_hdr_pair("In-Reply-To",
-				g_mime_utils_header_decode_text(value)));
+        res =
+            g_list_prepend(res,
+                           libbalsa_create_hdr_pair
+                           ("In-Reply-To",
+                            g_mime_utils_header_decode_text
+                            ((unsigned char *) value)));
     }
 
     g_mime_header_foreach(GMIME_OBJECT(message)->headers,
@@ -483,11 +448,13 @@ libbalsa_message_save(LibBalsaMessage * message, const gchar *filename)
     if( (outfile = fopen(filename, "w")) == NULL) return FALSE;
     g_return_val_if_fail(outfile, FALSE);
 
-    msg_stream = libbalsa_mailbox_get_message_stream(message->mailbox, message);
+    msg_stream = libbalsa_message_stream(message);
     if (msg_stream == NULL)
 	return FALSE;
     out_stream = g_mime_stream_file_new(outfile);
+    libbalsa_mime_stream_shared_lock(msg_stream);
     res = g_mime_stream_write_to_stream(msg_stream, out_stream);
+    libbalsa_mime_stream_shared_unlock(msg_stream);
 
     g_object_unref(msg_stream);
     g_object_unref(out_stream);
@@ -975,7 +942,8 @@ libbalsa_message_init_from_gmime(LibBalsaMessage * message,
 #ifdef MESSAGE_COPY_CONTENT
     header = g_mime_message_get_subject(mime_msg);
     if (header) {
-	message->subj = g_mime_utils_header_decode_text(header);
+        message->subj =
+            g_mime_utils_header_decode_text((unsigned char *) header);
         canonize_header_value(message->subj);
 	libbalsa_utf8_sanitize(&message->subj, TRUE, NULL);
     }
@@ -1100,7 +1068,8 @@ lbmsg_set_header(LibBalsaMessage *message, const gchar *name,
 	    return FALSE;
 #if MESSAGE_COPY_CONTENT
 	g_free(message->subj);
-        message->subj = g_mime_utils_header_decode_text(value);
+        message->subj =
+	    g_mime_utils_header_decode_text((unsigned char *) value);
 	libbalsa_utf8_sanitize(&message->subj, TRUE, NULL);
 #endif
     } else
@@ -1180,23 +1149,22 @@ libbalsa_message_load_envelope(LibBalsaMessage *message)
     GMimeStream *gmime_stream;
     GMimeStream *gmime_stream_buffer;
     GByteArray *line;
-    char lookahead;
+    guchar lookahead;
     gboolean ret = FALSE;
 
-    gmime_stream =
-	libbalsa_mailbox_get_message_stream(message->mailbox, message);
+    gmime_stream = libbalsa_message_stream(message);
     if (!gmime_stream)
 	return;
+    libbalsa_mime_stream_shared_lock(gmime_stream);
     gmime_stream_buffer = g_mime_stream_buffer_new(gmime_stream,
 					GMIME_STREAM_BUFFER_BLOCK_READ);
-    g_object_unref(gmime_stream);
 
     line = g_byte_array_new();
     do {
 	g_mime_stream_buffer_readln(gmime_stream_buffer, line);
 	while (!g_mime_stream_eos(gmime_stream_buffer)
-	       && g_mime_stream_read(gmime_stream_buffer, &lookahead, 1) == 1)
-	{
+	       && g_mime_stream_read(gmime_stream_buffer,
+		                     (char *) &lookahead, 1) == 1) {
 		if (lookahead == ' ' || lookahead == '\t') {
 		    g_byte_array_append(line, &lookahead, 1);
 		    g_mime_stream_buffer_readln(gmime_stream_buffer, line);
@@ -1209,7 +1177,9 @@ libbalsa_message_load_envelope(LibBalsaMessage *message)
 	    break;
 	}
 	line->data[line->len-1]='\0'; /* terminate line by overwriting '\n' */
-	if (!lb_message_set_headers_from_string(message, line->data, FALSE)) {
+	if (!lb_message_set_headers_from_string(message,
+				                (gchar *) line->data,
+						FALSE)) {
 	    /* Ignore error return caused by malformed header. */
 	}
 	if (lookahead == '\n') {/* end of headers */
@@ -1223,5 +1193,30 @@ libbalsa_message_load_envelope(LibBalsaMessage *message)
 	/* calculate size */
     }
     g_object_unref(gmime_stream_buffer);
+    libbalsa_mime_stream_shared_unlock(gmime_stream);
+    g_object_unref(gmime_stream);
     g_byte_array_free(line, TRUE);
 }
+
+GMimeStream *
+libbalsa_message_stream(LibBalsaMessage * message)
+{
+    LibBalsaMailbox *mailbox;
+    GMimeStream *mime_stream;
+
+    g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), NULL);
+    mailbox = message->mailbox;
+    g_return_val_if_fail(mailbox != NULL || message->mime_msg != NULL,
+                         NULL);
+
+    if (mailbox)
+        return libbalsa_mailbox_get_message_stream(mailbox,
+                                                   message->msgno);
+
+    mime_stream = g_mime_stream_mem_new();
+    g_mime_object_write_to_stream(GMIME_OBJECT(message->mime_msg),
+                                  mime_stream);
+    g_mime_stream_reset(mime_stream);
+
+    return mime_stream;
+}
