@@ -300,8 +300,7 @@ void libbalsa_message_body_unref(LibBalsaMessage * message);
 gboolean libbalsa_message_postpone(LibBalsaMessage * message,
 				   LibBalsaMailbox * draftbox,
 				   LibBalsaMessage * reply_message,
-				   gchar * fcc, gint encoding, 
-				   gboolean flow);
+				   gchar * fcc, gboolean flow);
 
 /*
  * misc message releated functions
@@ -309,7 +308,6 @@ gboolean libbalsa_message_postpone(LibBalsaMessage * message,
 gchar **libbalsa_create_hdr_pair(const gchar * name, gchar * value);
 
 const gchar *libbalsa_message_pathname(LibBalsaMessage * message);
-gchar *libbalsa_message_charset(LibBalsaMessage * message);
 const gchar *libbalsa_message_body_charset(LibBalsaMessageBody * body);
 gboolean libbalsa_message_is_multipart(LibBalsaMessage * message);
 gboolean libbalsa_message_is_partial(LibBalsaMessage * message,
@@ -351,4 +349,5 @@ void libbalsa_message_set_references_from_string(LibBalsaMessage * message,
 						 const gchar *str);
 void libbalsa_message_set_in_reply_to_from_string(LibBalsaMessage * message,
 						  const gchar *str);
+GMimeStream *libbalsa_message_stream(LibBalsaMessage * message);
 #endif				/* __LIBBALSA_MESSAGE_H__ */
