@@ -35,7 +35,9 @@ static GString *process_mime_multipart(LibBalsaMessage * message,
                                        LibBalsaMessageBody * body,
 				       gchar * reply_prefix_str,
 				       gint llen, gboolean ignore_html,
-                                       gboolean flow);
+                                       gboolean flow,
+				       LibBalsaCharsetFunc charset_cb,
+				       gpointer charset_cb_data);
 
 /* process_mime_part:
    returns string representation of given message part.
@@ -45,7 +47,8 @@ static GString *process_mime_multipart(LibBalsaMessage * message,
 GString *
 process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
 		  gchar * reply_prefix_str, gint llen, gboolean ignore_html,
-                  gboolean flow)
+                  gboolean flow, LibBalsaCharsetFunc charset_cb,
+		  gpointer charset_cb_data)
 {
     gchar *res = NULL;
     size_t allocated;
@@ -64,7 +67,8 @@ process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
 	break;
     case LIBBALSA_MESSAGE_BODY_TYPE_MULTIPART:
         reply = process_mime_multipart(message, body, reply_prefix_str,
-                                       llen, ignore_html, flow);
+                                       llen, ignore_html, flow,
+                                       charset_cb, charset_cb_data);
 	break;
     case LIBBALSA_MESSAGE_BODY_TYPE_TEXT:
 	/* don't return text/html stuff... */
@@ -86,6 +90,10 @@ process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
 	}
 #endif /* HAVE_GTKHTML */
 
+        if (charset_cb)
+            charset_cb(libbalsa_message_body_charset(body),
+                       charset_cb_data);
+
 	if (llen > 0) {
             if (flow && libbalsa_message_body_is_flowed(body)) {
                 /* we're making a `format=flowed' message, and the
@@ -144,14 +152,17 @@ static GString *
 process_mime_multipart(LibBalsaMessage * message,
                        LibBalsaMessageBody * body,
 		       gchar * reply_prefix_str, gint llen,
-		       gboolean ignore_html, gboolean flow)
+		       gboolean ignore_html, gboolean flow,
+		       LibBalsaCharsetFunc charset_cb,
+		       gpointer charset_cb_data)
 {
     LibBalsaMessageBody *part;
     GString *res = NULL, *s;
 
     for (part = body->parts; part; part = part->next) {
-	s = process_mime_part(message, part, reply_prefix_str, llen,
-                          ignore_html, flow);
+        s = process_mime_part(message, part, reply_prefix_str, llen,
+                              ignore_html, flow, charset_cb,
+                              charset_cb_data);
 	if (!s)
 	    continue;
 	if (res) {
@@ -168,7 +179,8 @@ process_mime_multipart(LibBalsaMessage * message,
 
 GString *
 content2reply(LibBalsaMessage * message, gchar * reply_prefix_str,
-	      gint llen, gboolean ignore_html, gboolean flow)
+	      gint llen, gboolean ignore_html, gboolean flow,
+	      LibBalsaCharsetFunc charset_cb, gpointer charset_cb_data)
 {
     LibBalsaMessageBody *body;
     GString *reply = NULL, *res;
@@ -177,7 +189,8 @@ content2reply(LibBalsaMessage * message, gchar * reply_prefix_str,
     body = message->body_list;
     for (body = message->body_list; body; body = body->next) {
 	res = process_mime_part(message, body, reply_prefix_str, llen,
-                                ignore_html, flow);
+                                ignore_html, flow, charset_cb, 
+				charset_cb_data);
 	if (!res)
 	    continue;
 	if (reply) {
