@@ -1337,14 +1337,13 @@ static void
 lb_text_attr(const gchar * text, gboolean * has_esc, gboolean * has_hi_bit,
              gboolean * has_hi_ctrl)
 {
-    const guchar *p;
-
-    for (p = text; *p; p++) {
-        if (*p == 0x1b)
+    for (; *text; text++) {
+	guchar c = *text;
+        if (c == 0x1b)
             *has_esc = TRUE;
-        if (*p >= 0x80) {
+        if (c >= 0x80) {
             *has_hi_bit = TRUE;
-            if (*p <= 0x9f)
+            if (c <= 0x9f)
                 *has_hi_ctrl = TRUE;
         }
     }
@@ -1612,7 +1611,7 @@ libbalsa_insert_with_url(GtkTextBuffer * buffer,
 		    /* check if we hit a multi-line URL... (see RFC 1738) */
 		    if (all_p && (p[url_match.rm_so - 1] == '<' ||
 				  (url_match.rm_so > 4 &&
-				   g_ascii_strcasecmp(p + url_match.rm_so - 5, "<URL:"))) &&
+				   g_ascii_strcasecmp(p + url_match.rm_so - 5, "<URL:") == 0)) &&
 			!strchr(p + url_match.rm_eo, '>')) {
 			regex_t *ml_url_reg = get_ml_url_reg();
 			regmatch_t ml_url_match;
@@ -1851,7 +1850,7 @@ libbalsa_lock_file (const char *path, int fd, int excl, int dot, int timeout)
 #if defined (USE_FCNTL) || defined (USE_FLOCK)
     int count;
     int attempt;
-    struct stat prev_sb;
+    struct stat prev_sb = { 0 };
 #endif
     int r = 0;
 
@@ -1982,7 +1981,7 @@ libbalsa_unlock_file (const char *path, int fd, int dot)
     return 0;
 }
 
-#ifndef HAVE_GLIB22
+#if !GLIB_CHECK_VERSION(2, 2, 0)
 gboolean
 libbalsa_str_has_prefix(const gchar * str, const gchar * prefix)
 {
@@ -1996,7 +1995,7 @@ libbalsa_str_has_prefix(const gchar * str, const gchar * prefix)
 
     return *prefix == '\0';
 }
-#endif				/* HAVE_GLIB22 */
+#endif				/* !GLIB_CHECK_VERSION(2, 2, 0) */
 
 
 /* libbalsa_ia_rfc2821_equal
