@@ -43,13 +43,17 @@
 #include "information.h"
 
 #if ENABLE_ESMTP
+#include "smtp-server.h"
 #include <stdarg.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#endif
+#endif /* ESMTP */
+
 #include <sys/utsname.h>
 
+#include "mime-stream-shared.h"
+
 #include "i18n.h"
 
 typedef struct _MessageQueueItem MessageQueueItem;
@@ -190,17 +194,19 @@ send_message_info_destroy(SendMessageInfo *smi)
 
 #if HAVE_GPGME
 static gint
-libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part);
+libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part,
+			       GtkWindow * parent);
 static LibBalsaMsgCreateResult
-do_multipart_crypto(LibBalsaMessage * message, GMimeObject ** mime_root);
+do_multipart_crypto(LibBalsaMessage * message, GMimeObject ** mime_root,
+		    GtkWindow * parent);
 #endif
 
 static guint balsa_send_message_real(SendMessageInfo* info);
 static LibBalsaMsgCreateResult
-libbalsa_message_create_mime_message(LibBalsaMessage* message, gint encoding,
+libbalsa_message_create_mime_message(LibBalsaMessage* message,
 				     gboolean flow, gboolean postponing);
 static LibBalsaMsgCreateResult libbalsa_create_msg(LibBalsaMessage * message,
-				    gint encoding, gboolean flow);
+				    gboolean flow);
 static LibBalsaMsgCreateResult
 libbalsa_fill_msg_queue_item_from_queu(LibBalsaMessage * message,
                                        MessageQueueItem *mqi);
@@ -282,8 +288,9 @@ lbs_set_content(GMimePart * mime_part, gchar * content)
     stream = g_mime_stream_mem_new();
     g_mime_stream_write(stream, content, strlen(content));
 
-    wrapper = g_mime_data_wrapper_new();
-    g_mime_data_wrapper_set_stream(wrapper, stream);
+    wrapper =
+        g_mime_data_wrapper_new_with_stream(stream,
+                                            GMIME_PART_ENCODING_DEFAULT);
     g_object_unref(stream);
 
     g_mime_part_set_content_object(mime_part, wrapper);
@@ -291,8 +298,7 @@ lbs_set_content(GMimePart * mime_part, gchar * content)
 }
 
 static GMimeObject *
-add_mime_body_plain(LibBalsaMessageBody *body, gint encoding_style, 
-		    gboolean flow)
+add_mime_body_plain(LibBalsaMessageBody *body, gboolean flow)
 {
     GMimePart *mime_part;
     const gchar * charset;
@@ -301,8 +307,6 @@ add_mime_body_plain(LibBalsaMessageBody *body, gint encoding_style,
     
     charset=body->charset;
 
-    g_return_val_if_fail(charset, NULL);
-
     if (body->content_type) {
         /* Use the suplied mime type */
         gchar *type, *subtype;
@@ -322,9 +326,11 @@ add_mime_body_plain(LibBalsaMessageBody *body, gint encoding_style,
     }
 
     g_mime_part_set_content_disposition(mime_part, GMIME_DISPOSITION_INLINE);
-    g_mime_part_set_encoding(mime_part, encoding_style);
+    g_mime_part_set_encoding(mime_part, GMIME_PART_ENCODING_QUOTEDPRINTABLE);
     g_mime_object_set_content_type_parameter(GMIME_OBJECT(mime_part),
-					     "charset", charset);
+                                             "charset",
+                                             charset ? charset :
+                                             "us-ascii");
     if (flow) {
 	g_mime_object_set_content_type_parameter(GMIME_OBJECT(mime_part),
 						 "DelSp", "Yes");
@@ -332,9 +338,9 @@ add_mime_body_plain(LibBalsaMessageBody *body, gint encoding_style,
 						 "Format", "Flowed");
     }
 
-    if (g_ascii_strcasecmp(charset, "UTF-8")!=0 &&
-	g_ascii_strcasecmp(charset, "UTF8")!=0 &&
-	g_ascii_strcasecmp(charset, "US-ASCII")!=0)
+    if (charset &&
+	g_ascii_strcasecmp(charset, "UTF-8")!=0 &&
+	g_ascii_strcasecmp(charset, "UTF8")!=0)
     {
 	GMimeStream *stream, *filter_stream;
 	GMimeFilter *filter;
@@ -344,18 +350,18 @@ add_mime_body_plain(LibBalsaMessageBody *body, gint encoding_style,
 	filter_stream = g_mime_stream_filter_new_with_stream(stream);
 	filter = g_mime_filter_charset_new("UTF-8", charset);
 	g_mime_stream_filter_add(GMIME_STREAM_FILTER(filter_stream), filter);
+	g_object_unref(G_OBJECT(filter));
 
 	g_mime_stream_write(filter_stream, body->buffer, strlen(body->buffer));
-
-	wrapper = g_mime_data_wrapper_new();
-	g_mime_data_wrapper_set_stream(wrapper, stream);
-	g_mime_data_wrapper_set_encoding (wrapper, GMIME_PART_ENCODING_DEFAULT);
-	g_mime_part_set_content_object(mime_part, wrapper);
-
-	g_object_unref(G_OBJECT(wrapper));
-	g_object_unref(G_OBJECT(filter));
 	g_object_unref(filter_stream);
+
+        wrapper =
+            g_mime_data_wrapper_new_with_stream(stream,
+                                                GMIME_PART_ENCODING_DEFAULT);
 	g_object_unref(stream);
+
+	g_mime_part_set_content_object(mime_part, wrapper);
+	g_object_unref(G_OBJECT(wrapper));
     } else
 	lbs_set_content(mime_part, body->buffer);
 
@@ -380,68 +386,32 @@ static void dump_queue(const char*msg)
 */
 LibBalsaMsgCreateResult
 libbalsa_message_queue(LibBalsaMessage * message, LibBalsaMailbox * outbox,
-		       LibBalsaMailbox * fccbox, gint encoding,
+		       LibBalsaMailbox * fccbox,
+#if ENABLE_ESMTP
+                       LibBalsaSmtpServer * smtp_server,
+#endif /* ESMTP */
 		       gboolean flow)
 {
     LibBalsaMsgCreateResult result;
 
     g_return_val_if_fail(message, LIBBALSA_MESSAGE_CREATE_ERROR);
 
-    if ((result = libbalsa_create_msg(message, encoding, flow)) !=
+    if ((result = libbalsa_create_msg(message, flow)) !=
 	LIBBALSA_MESSAGE_CREATE_OK)
         return result;
 
     if (fccbox)
         g_mime_message_set_header(message->mime_msg, "X-Balsa-Fcc",
                                   fccbox->url);
+#if ENABLE_ESMTP
+    g_mime_message_set_header(message->mime_msg, "X-Balsa-SmtpServer",
+	                      libbalsa_smtp_server_get_name(smtp_server));
+#endif /* ESMTP */
     if(libbalsa_mailbox_copy_message( message, outbox, NULL) < 0)
         return LIBBALSA_MESSAGE_QUEUE_ERROR;
     return LIBBALSA_MESSAGE_CREATE_OK;
 }
 
-/* libbalsa_message_send:
-   send the given messsage (if any, it can be NULL) and all the messages
-   in given outbox.
-*/
-#if ENABLE_ESMTP
-LibBalsaMsgCreateResult
-libbalsa_message_send(LibBalsaMessage * message, LibBalsaMailbox * outbox,
-                      LibBalsaMailbox *fccbox, gint encoding,
-                      LibBalsaFccboxFinder finder,
-                      gchar * smtp_server, auth_context_t smtp_authctx,
-                      gint tls_mode, gboolean flow, gboolean debug)
-{
-    LibBalsaMsgCreateResult result = LIBBALSA_MESSAGE_CREATE_OK;
-
-    if (message != NULL)
-        result = libbalsa_message_queue(message, outbox, fccbox, encoding,
-					flow);
-     if (result == LIBBALSA_MESSAGE_CREATE_OK)
-	 if (!libbalsa_process_queue(outbox, finder, smtp_server,
-				     smtp_authctx, tls_mode, debug))
-	     return LIBBALSA_MESSAGE_SEND_ERROR;
- 
-     return result;
-}
-#else
-LibBalsaMsgCreateResult
-libbalsa_message_send(LibBalsaMessage* message, LibBalsaMailbox* outbox,
-		      LibBalsaMailbox* fccbox, LibBalsaFccboxFinder finder,
-                      gint encoding, gboolean flow, gboolean debug)
-{
-    LibBalsaMsgCreateResult result = LIBBALSA_MESSAGE_CREATE_OK;
-
-    if (message != NULL)
- 	result = libbalsa_message_queue(message, outbox, fccbox,
-                                        encoding, flow);
-    if (result == LIBBALSA_MESSAGE_CREATE_OK)
- 	if (!libbalsa_process_queue(outbox, finder, debug))
- 	    return LIBBALSA_MESSAGE_SEND_ERROR;
- 
-    return result;
-}
-#endif
-
 static void
 lbs_change_flags(LibBalsaMessage * msg, LibBalsaMessageFlag set,
                  LibBalsaMessageFlag clear)
@@ -454,6 +424,56 @@ lbs_change_flags(LibBalsaMessage * msg, LibBalsaMessageFlag set,
     g_array_free(array, TRUE);
 }
 
+/* libbalsa_message_send:
+   send the given messsage (if any, it can be NULL) and all the messages
+   in given outbox.
+*/
+#if ENABLE_ESMTP
+static gboolean lbs_process_queue(LibBalsaMailbox * outbox,
+                                  LibBalsaFccboxFinder finder,
+                                  LibBalsaSmtpServer * smtp_server,
+                                  gboolean debug);
+
+LibBalsaMsgCreateResult
+libbalsa_message_send(LibBalsaMessage * message, LibBalsaMailbox * outbox,
+                      LibBalsaMailbox * fccbox,
+                      LibBalsaFccboxFinder finder,
+                      LibBalsaSmtpServer * smtp_server,
+                      gboolean flow, gboolean debug)
+{
+    LibBalsaMsgCreateResult result = LIBBALSA_MESSAGE_CREATE_OK;
+
+    g_return_val_if_fail(smtp_server != NULL,
+                         LIBBALSA_MESSAGE_SERVER_ERROR);
+
+    if (message != NULL)
+        result = libbalsa_message_queue(message, outbox, fccbox,
+                                        smtp_server, flow);
+
+    if (result == LIBBALSA_MESSAGE_CREATE_OK
+        && !lbs_process_queue(outbox, finder, smtp_server, debug))
+            return LIBBALSA_MESSAGE_SEND_ERROR;
+
+    return result;
+}
+#else /* ENABLE_ESMTP */
+LibBalsaMsgCreateResult
+libbalsa_message_send(LibBalsaMessage* message, LibBalsaMailbox* outbox,
+		      LibBalsaMailbox* fccbox, LibBalsaFccboxFinder finder,
+                      gboolean flow, gboolean debug)
+{
+    LibBalsaMsgCreateResult result = LIBBALSA_MESSAGE_CREATE_OK;
+
+    if (message != NULL)
+ 	result = libbalsa_message_queue(message, outbox, fccbox, flow);
+    if (result == LIBBALSA_MESSAGE_CREATE_OK)
+ 	if (!libbalsa_process_queue(outbox, finder, debug))
+ 	    return LIBBALSA_MESSAGE_SEND_ERROR;
+ 
+    return result;
+}
+#endif /* ENABLE_ESMTP */
+
 #if ENABLE_ESMTP
 /* [BCS] - libESMTP uses a callback function to read the message from the
    application to the SMTP server.
@@ -474,7 +494,7 @@ libbalsa_message_cb (void **buf, int *len, void *arg)
 
     *len = g_mime_stream_length(current_message->stream)
 	    - g_mime_stream_tell(current_message->stream);
-    ptr = mem_stream->buffer->data
+    ptr = (char *) mem_stream->buffer->data
 	    + g_mime_stream_tell(current_message->stream);
     g_mime_stream_seek(current_message->stream, *len, GMIME_STREAM_SEEK_CUR);
 
@@ -508,12 +528,11 @@ add_recipients(smtp_message_t message, smtp_message_t bcc_message,
 /* This version uses libESMTP. It has slightly different semantics than
    sendmail version so don't get fooled by similar variable names.
  */
-gboolean
-libbalsa_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
-                       gchar * smtp_server,
-                       auth_context_t smtp_authctx, gint tls_mode,
-                       gboolean debug)
+static gboolean
+lbs_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
+		  LibBalsaSmtpServer * smtp_server, gboolean debug)
 {
+    LibBalsaServer *server = LIBBALSA_SERVER(smtp_server);
     MessageQueueItem *new_message;
     SendMessageInfo *send_message_info;
     LibBalsaMessage *msg;
@@ -541,17 +560,18 @@ libbalsa_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
        messages to the session. */
 
     /* FIXME - check for failure returns in the smtp_xxx() calls */
-    host_with_port = strchr(smtp_server, ':') 
-        ? g_strdup(smtp_server) : g_strconcat(smtp_server, ":smtp", NULL);
+    host_with_port = strchr(server->host, ':') ?
+        g_strdup(server->host) : g_strconcat(server->host, ":smtp", NULL);
     session = smtp_create_session ();
     smtp_set_server (session, host_with_port);
     g_free(host_with_port);
 
     /* Tell libESMTP how to use the SMTP STARTTLS extension.  */
-    smtp_starttls_enable (session, tls_mode);
+    smtp_starttls_enable (session, server->tls_mode);
 
     /* Now tell libESMTP it can use the SMTP AUTH extension.  */
-    smtp_auth_set_context (session, smtp_authctx);
+    smtp_auth_set_context(session,
+                          libbalsa_smtp_server_get_authctx(smtp_server));
  
     /* At present Balsa can't handle one recipient only out of many
        failing.  Make libESMTP require all specified recipients to
@@ -560,6 +580,8 @@ libbalsa_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
 
     for (msgno = libbalsa_mailbox_total_messages(outbox);
 	 msgno > 0; msgno--) {
+	GList *smtp_server_list;
+	const gchar *smtp_server_name;
         LibBalsaMsgCreateResult created;
 
 	msg = libbalsa_mailbox_get_message(outbox, msgno);
@@ -570,8 +592,19 @@ libbalsa_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
             continue;
 	}
 
-        libbalsa_message_body_ref(msg, TRUE, TRUE); /* FIXME: do we need 
-                                                     * all headers? */
+        libbalsa_message_body_ref(msg, TRUE, TRUE);
+        smtp_server_list =
+            libbalsa_message_find_user_hdr(msg, "X-Balsa-SmtpServer");
+        smtp_server_name = smtp_server_list ?
+            ((gchar **) smtp_server_list->data)[1] :
+            libbalsa_smtp_server_get_name(NULL);
+        if (strcmp(smtp_server_name,
+                   libbalsa_smtp_server_get_name(smtp_server)) != 0) {
+            libbalsa_message_body_unref(msg);
+            g_object_unref(msg);
+            continue;
+        }
+
 	new_message = msg_queue_item_new(finder);
         created = libbalsa_fill_msg_queue_item_from_queu(msg, new_message);
         libbalsa_message_body_unref(msg);
@@ -721,6 +754,8 @@ libbalsa_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
 	    smtp_set_header_option(message, "X-Status", Hdr_PROHIBIT, 1);
             /* ... and FCC header. */
             smtp_set_header_option(message, "X-Balsa-Fcc", Hdr_PROHIBIT, 1);
+            smtp_set_header_option(message, "X-Balsa-SmtpServer",
+		                   Hdr_PROHIBIT, 1);
 
 
 	    /* Estimate the size of the message.  This need not be exact
@@ -775,6 +810,22 @@ libbalsa_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
     return TRUE;
 }
 
+gboolean
+libbalsa_process_queue(LibBalsaMailbox * outbox,
+                       LibBalsaFccboxFinder finder,
+                       GSList * smtp_servers,
+		       gboolean debug)
+{
+    for (; smtp_servers; smtp_servers = smtp_servers->next) {
+        LibBalsaSmtpServer *smtp_server =
+		LIBBALSA_SMTP_SERVER(smtp_servers->data);
+        if (!lbs_process_queue(outbox, finder, smtp_server, debug))
+            return FALSE;
+    }
+
+    return TRUE;
+}
+
 static void
 disp_recipient_status(smtp_recipient_t recipient,
                       const char *mailbox, void *arg)
@@ -988,6 +1039,49 @@ libbalsa_smtp_event_cb (smtp_session_t session, int event_no, void *arg, ...)
     }
     va_end (ap);
 }
+#else /* BALSA: USE_THREADS */
+static void
+libbalsa_smtp_event_cb_serial(smtp_session_t session, int event_no,
+                              void *arg, ...)
+{
+    va_list ap;
+
+    va_start (ap, arg);
+    switch (event_no) {
+#ifdef USE_TLS
+        /* SMTP_TLS related things. Observe that we need to have SSL
+	 * enabled in balsa to properly interpret libesmtp
+	 * messages. */
+    case SMTP_EV_INVALID_PEER_CERTIFICATE: {
+        long vfy_result;
+	SSL  *ssl;
+	X509 *cert;
+        int *ok;
+        vfy_result = va_arg(ap, long); ok = va_arg(ap, int*);
+	ssl = va_arg(ap, SSL*);
+	cert = SSL_get_peer_certificate(ssl);
+	if(cert) {
+	    *ok = libbalsa_is_cert_known(cert, vfy_result);
+	    X509_free(cert);
+	}
+        break;
+    }
+    case SMTP_EV_NO_PEER_CERTIFICATE:
+    case SMTP_EV_WRONG_PEER_CERTIFICATE:
+#if LIBESMTP_1_0_3_AVAILABLE
+    case SMTP_EV_NO_CLIENT_CERTIFICATE:
+#endif
+    {
+	int *ok;
+	printf("SMTP-TLS event_no=%d\n", event_no);
+	ok = va_arg(ap, int*);
+	*ok = 1;
+	break;
+    }
+#endif /* USE_TLS */
+    }
+    va_end (ap);
+}
 #endif /* BALSA_USE_THREADS */
 
 #else /* ESMTP */
@@ -1023,8 +1117,10 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, LibBalsaFccboxFinder finder,
 
     ensure_send_progress_dialog();
     if (!libbalsa_mailbox_open(outbox, NULL)) {
+#ifdef BALSA_USE_THREADS
 	sending_threads--;
 	send_unlock();
+#endif
 	return FALSE;
     }
     for (msgno = libbalsa_mailbox_total_messages(outbox);
@@ -1161,6 +1257,8 @@ balsa_send_message_real(SendMessageInfo* info)
        feedback in non-MT version.
     */
     smtp_set_eventcb (info->session, libbalsa_smtp_event_cb, NULL);
+#else
+    smtp_set_eventcb (info->session, libbalsa_smtp_event_cb_serial, NULL);
 #endif
 
     /* Add a protocol monitor when debugging is enabled. */
@@ -1391,8 +1489,8 @@ parse_content_type(const char* content_type)
 }
 
 static LibBalsaMsgCreateResult
-libbalsa_message_create_mime_message(LibBalsaMessage* message, gint encoding,
-				     gboolean flow, gboolean postponing)
+libbalsa_message_create_mime_message(LibBalsaMessage* message, gboolean flow,
+				     gboolean postponing)
 {
     gchar **mime_type;
     GMimeObject *mime_root = NULL;
@@ -1400,11 +1498,13 @@ libbalsa_message_create_mime_message(LibBalsaMessage* message, gint encoding,
     LibBalsaMessageBody *body;
     gchar *tmp;
     GList *list;
+    GtkWindow * parent;
 
     body = message->body_list;
     if (body && body->next)
 	mime_root=GMIME_OBJECT(g_mime_multipart_new_with_subtype(message->subtype));
 
+    parent = g_object_get_data(G_OBJECT(message), "parent-window");
     while (body) {
 	GMimeObject *mime_part;
 	mime_part=NULL;
@@ -1515,28 +1615,25 @@ libbalsa_message_create_mime_message(LibBalsaMessage* message, gint encoding,
 	    g_strfreev(mime_type);
 	} else if (body->buffer) {
 #ifdef HAVE_GPGME
-	    /* force quoted printable encoding if only signing is requested */
-	    mime_part =
-		add_mime_body_plain(body,
-				    (message->gpg_mode & LIBBALSA_PROTECT_MODE) == LIBBALSA_PROTECT_SIGN ?
-				    GMIME_PART_ENCODING_QUOTEDPRINTABLE :
-				    encoding, flow);
+	    mime_part = add_mime_body_plain(body, flow);
 	    /* in '2440 mode, touch *only* the first body! */
 	    if (!postponing && body == body->message->body_list &&
 		message->gpg_mode > 0 &&
 		(message->gpg_mode & LIBBALSA_PROTECT_OPENPGP) != 0) {
 		gint result = 
 		    libbalsa_create_rfc2440_buffer(body,
-			                           GMIME_PART(mime_part));
+			                           GMIME_PART(mime_part),
+						   parent);
 
 		if (result != LIBBALSA_MESSAGE_CREATE_OK) {
 		    g_object_unref(G_OBJECT(mime_part));
-		    g_object_unref(G_OBJECT(mime_root));
+		    if (mime_root)
+			g_object_unref(G_OBJECT(mime_root));
 		    return LIBBALSA_MESSAGE_CREATE_ERROR;
 		}
 	    }
 #else
-	    mime_part = add_mime_body_plain(body, encoding, flow);
+	    mime_part = add_mime_body_plain(body, flow);
 #endif /* HAVE_GPGME */
 	}
 
@@ -1553,7 +1650,8 @@ libbalsa_message_create_mime_message(LibBalsaMessage* message, gint encoding,
 
 #ifdef HAVE_GPGME
     if (message->body_list != NULL && !postponing) {
-	LibBalsaMsgCreateResult crypt_res = do_multipart_crypto(message, &mime_root);
+	LibBalsaMsgCreateResult crypt_res =
+	    do_multipart_crypto(message, &mime_root, parent);
 	if (crypt_res != LIBBALSA_MESSAGE_CREATE_OK)
 	    return crypt_res;
     }
@@ -1651,13 +1749,12 @@ gboolean
 libbalsa_message_postpone(LibBalsaMessage * message,
 			  LibBalsaMailbox * draftbox,
 			  LibBalsaMessage * reply_message,
-			  gchar * fcc, gint encoding,
-			  gboolean flow)
+			  gchar * fcc, gboolean flow)
 {
     int thereturn; 
 
     if (!message->mime_msg
-	&& libbalsa_message_create_mime_message(message, encoding, flow,
+	&& libbalsa_message_create_mime_message(message, flow,
 						TRUE) !=
 	LIBBALSA_MESSAGE_CREATE_OK)
 	return FALSE;
@@ -1708,12 +1805,11 @@ libbalsa_set_message_id(GMimeMessage * mime_message)
    copies message to msg.
 */ 
 static LibBalsaMsgCreateResult
-libbalsa_create_msg(LibBalsaMessage * message,
-		    gint encoding, gboolean flow)
+libbalsa_create_msg(LibBalsaMessage * message, gboolean flow)
 {
     if (!message->mime_msg) {
 	LibBalsaMsgCreateResult res =
-	    libbalsa_message_create_mime_message(message, encoding, flow,
+	    libbalsa_message_create_mime_message(message, flow,
 						 FALSE);
 	if (res != LIBBALSA_MESSAGE_CREATE_OK)
 	    return res;
@@ -1731,19 +1827,26 @@ libbalsa_fill_msg_queue_item_from_queu(LibBalsaMessage * message,
     mqi->orig = message;
     g_object_ref(mqi->orig);
     if (message->mime_msg) {
+        GMimeStream *tmp;
+
         g_mime_object_remove_header(GMIME_OBJECT(message->mime_msg),
                                     "Status");
         g_mime_object_remove_header(GMIME_OBJECT(message->mime_msg),
                                     "X-Status");
         g_mime_object_remove_header(GMIME_OBJECT(message->mime_msg),
                                     "X-Balsa-Fcc");
+        g_mime_object_remove_header(GMIME_OBJECT(message->mime_msg),
+                                    "X-Balsa-SmtpServer");
 	mqi->stream = g_mime_stream_mem_new();
+        tmp = libbalsa_message_stream(message);
+        libbalsa_mime_stream_shared_lock(tmp);
 	g_mime_object_write_to_stream(GMIME_OBJECT(message->mime_msg),
                                       mqi->stream);
+        libbalsa_mime_stream_shared_unlock(tmp);
+        g_object_unref(tmp);
 	g_mime_stream_reset(mqi->stream);
     } else
-	mqi->stream = libbalsa_mailbox_get_message_stream(message->mailbox,
-                                                          message);
+	mqi->stream = libbalsa_message_stream(message);
     if (mqi->stream == NULL)
 	return LIBBALSA_MESSAGE_CREATE_ERROR;
   
@@ -1770,17 +1873,20 @@ lb_send_from(LibBalsaMessage *message)
 }
 
 static gint
-libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part)
+libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part,
+			       GtkWindow * parent)
 {
     LibBalsaMessage *message = body->message;
     gint mode = message->gpg_mode;
+    gboolean always_trust = (mode & LIBBALSA_PROTECT_ALWAYS_TRUST) != 0;
 
     switch (mode & LIBBALSA_PROTECT_MODE)
 	{
 	case LIBBALSA_PROTECT_SIGN:   /* sign only */
 	    if (!libbalsa_rfc2440_sign_encrypt(mime_part, 
 					       lb_send_from(message),
-					       NULL, NULL))
+					       NULL, FALSE,
+					       parent))
 		return LIBBALSA_MESSAGE_SIGN_ERROR;
 	    break;
 	case LIBBALSA_PROTECT_ENCRYPT:
@@ -1807,13 +1913,15 @@ libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part)
 			libbalsa_rfc2440_sign_encrypt(mime_part, 
 					              lb_send_from(message),
 						      encrypt_for,
-						      NULL);
+						      always_trust,
+						      parent);
 		else
 		    result = 
 			libbalsa_rfc2440_sign_encrypt(mime_part, 
 						      NULL,
 						      encrypt_for,
-						      NULL);
+						      always_trust,
+						      parent);
 		g_list_foreach(encrypt_for, (GFunc) g_free, NULL);
 		g_list_free(encrypt_for);
 		if (!result)
@@ -1831,9 +1939,11 @@ libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part)
   
 /* handle rfc2633 and rfc3156 signing and/or encryption of a message */
 static LibBalsaMsgCreateResult
-do_multipart_crypto(LibBalsaMessage * message, GMimeObject ** mime_root)
+do_multipart_crypto(LibBalsaMessage * message, GMimeObject ** mime_root,
+		    GtkWindow * parent)
 {
     gpgme_protocol_t protocol;
+    gboolean always_trust;
 
     /* check if we shall do any protection */
     if (!(message->gpg_mode & LIBBALSA_PROTECT_MODE))
@@ -1851,13 +1961,14 @@ do_multipart_crypto(LibBalsaMessage * message, GMimeObject ** mime_root)
     else
 	return LIBBALSA_MESSAGE_ENCRYPT_ERROR;  /* hmmm.... */
 
+    always_trust = (message->gpg_mode & LIBBALSA_PROTECT_ALWAYS_TRUST) != 0;
     /* sign and/or encrypt */
     switch (message->gpg_mode & LIBBALSA_PROTECT_MODE)
 	{
 	case LIBBALSA_PROTECT_SIGN:   /* sign message */
 	    if (!libbalsa_sign_mime_object(mime_root,
 					   lb_send_from(message),
-					   protocol, NULL))
+					   protocol, parent))
 		return LIBBALSA_MESSAGE_SIGN_ERROR;
 	    break;
 	case LIBBALSA_PROTECT_ENCRYPT:
@@ -1884,11 +1995,12 @@ do_multipart_crypto(LibBalsaMessage * message, GMimeObject ** mime_root)
 			libbalsa_sign_encrypt_mime_object(mime_root,
 							  lb_send_from(message),
 							  encrypt_for, protocol,
-							  NULL);
+							  always_trust, parent);
 		else
 		    success = 
 			libbalsa_encrypt_mime_object(mime_root, encrypt_for,
-						     protocol, NULL);
+						     protocol, always_trust,
+						     parent);
 		g_list_free(encrypt_for);
 		
 		if (!success)
