@@ -30,6 +30,7 @@
 #include "libbalsa.h"
 #include "libbalsa_private.h"
 #include "misc.h"
+#include "mime-stream-shared.h"
 #include "i18n.h"
 
 static void close_cb(GtkAction * action, gpointer data);
@@ -154,21 +155,23 @@ lsv_escape_cb(GtkAction * action, gpointer data)
 			     _("Mailbox closed"));
 	return;
     }
-    msg_stream = libbalsa_mailbox_get_message_stream(msg->mailbox, msg);
+    msg_stream = libbalsa_mailbox_get_message_stream(msg->mailbox, msg->msgno);
     if (msg_stream == NULL)
 	return;
 
     mem_stream = g_mime_stream_mem_new();
+    libbalsa_mime_stream_shared_lock(msg_stream);
     g_mime_stream_write_to_stream(msg_stream, mem_stream);
+    libbalsa_mime_stream_shared_unlock(msg_stream);
     g_mime_stream_write(mem_stream, "", 1); /* close string */
-    raw_message = GMIME_STREAM_MEM(mem_stream)->buffer->data;
+    raw_message = (char *) GMIME_STREAM_MEM(mem_stream)->buffer->data;
 
     *(lsvi->escape_specials) =
 	gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
     lsv_show_message(raw_message, lsvi, *(lsvi->escape_specials));
 
-    g_mime_stream_unref(msg_stream);
-    g_mime_stream_unref(mem_stream);
+    g_object_unref(msg_stream);
+    g_object_unref(mem_stream);
 }
 
 static void
