@@ -39,12 +39,15 @@
 #endif
 #include "address-book-vcard.h"
 #include "address-book-gpe.h"
+#include "libbalsa-conf.c"
+#include "i18n.h"
 
 struct ABMainWindow {
     GtkWidget *entry_list; /* GtkListView widget */
     GtkWidget *edit_box, *apply_button, *remove_button, *cancel_button;
     GtkWidget *edit_widget;
     GtkWidget *entries[NUM_FIELDS];
+    GtkRadioAction *first_radio_action;
 
     GList *address_book_list;
     LibBalsaAddressBook* address_book;
@@ -62,33 +65,19 @@ static gint bab_kill_session(GnomeClient * client, gpointer client_data);
 
 static void ab_set_edit_widget(GtkWidget *w, gboolean can_remove);
 
-#define BALSA_CONFIG_PREFIX "balsa/"
 #define ADDRESS_BOOK_SECTION_PREFIX "address-book-"
-static void
-bab_config_init(void)
+static gboolean
+bab_config_init(const gchar * group, const gchar * value, gpointer data)
 {
     LibBalsaAddressBook *address_book;
-    void *iterator;
-    gchar *key, *val, *tmp;
-    int pref_len = strlen(ADDRESS_BOOK_SECTION_PREFIX);
+    GList **address_book_list = data;
 
-    iterator = gnome_config_init_iterator_sections(BALSA_CONFIG_PREFIX);
-    while ((iterator = gnome_config_iterator_next(iterator, &key, &val))) {
-
-	if (strncmp(key, ADDRESS_BOOK_SECTION_PREFIX, pref_len) == 0) {
-	    tmp = g_strconcat(BALSA_CONFIG_PREFIX, key, "/", NULL);
-
-	    address_book = libbalsa_address_book_new_from_config(tmp);
-	    if (address_book)
-		contacts_app.address_book_list =
-		    g_list_append(contacts_app.address_book_list,
-				  address_book);
-	    g_free(tmp);
-	}
-	g_free(key);
-	g_free(val);
-    }
+    address_book = libbalsa_address_book_new_from_config(group);
+    if (address_book)
+        *address_book_list =
+            g_list_append(*address_book_list, address_book);
 
+    return FALSE;
 }
 
 static void ab_warning(const char *fmt, ...);
@@ -171,19 +160,19 @@ bab_set_address_book(LibBalsaAddressBook *ab, GtkWidget* list,
 
 static void
 select_address_book_cb(GtkRadioAction * action, GtkRadioAction * current,
-                       gpointer callback_data)
+                       GtkWindow * window)
 {
     GList *l;
 
-    if (action != current)
-        return;
     l = g_list_nth(contacts_app.address_book_list,
-                   GPOINTER_TO_INT(callback_data));
+                   gtk_radio_action_get_current_value(action));
     if (!l)
         return;
+
     ab_set_edit_widget(NULL, FALSE);
     bab_set_address_book(LIBBALSA_ADDRESS_BOOK(l->data),
                          contacts_app.entry_list, NULL);
+    gtk_window_set_title(window, LIBBALSA_ADDRESS_BOOK(l->data)->name);
 }
 
 static void
@@ -297,15 +286,17 @@ get_main_menu(GtkWidget * window, GtkWidget ** menubar,
         radio_action = gtk_radio_action_new(address_book->name, label,
                                             NULL, NULL, cnt - 1);
 	g_free(label);
+
+
         if (group)
             gtk_radio_action_set_group(radio_action, group);
         else {
-            group = gtk_radio_action_get_group(radio_action);
-	    gtk_action_activate(GTK_ACTION(radio_action));
-	}
-        g_signal_connect(G_OBJECT(radio_action), "changed",
-                         G_CALLBACK(select_address_book_cb),
-                         GINT_TO_POINTER(cnt - 1));
+            g_signal_connect(G_OBJECT(radio_action), "changed",
+                             G_CALLBACK(select_address_book_cb),
+                             window);
+            contacts_app.first_radio_action = radio_action;
+        }
+        group = gtk_radio_action_get_group(radio_action);
 
         accelerator =
             cnt <= 9 ? g_strdup_printf("<control>%d", cnt) : NULL;
@@ -467,7 +458,6 @@ cancel_button_cb(GtkWidget *w, gpointer data)
     } else ab_set_edit_widget(NULL, FALSE);
 }
 
-#define ELEMENTS(x) (sizeof(x)/sizeof((x)[0])) 
 static GtkWidget*
 bab_get_edit_button_box(struct ABMainWindow *abmw)
 {
@@ -542,12 +532,12 @@ bab_get_filter_box(void)
 static GtkWidget*
 bab_window_new()
 {
-    GtkWidget* menubar, *main_vbox, *cont_box, *vbox, *scroll;
+    GtkWidget* menubar = NULL, *main_vbox, *cont_box, *vbox, *scroll;
     GtkWidget *wnd = gnome_app_new("Contacts", "Contacts");
-    GList *first_ab;
 
     get_main_menu(GTK_WIDGET(wnd), &menubar, contacts_app.address_book_list);
-    gnome_app_set_menus(GNOME_APP(wnd), GTK_MENU_BAR(menubar));
+    if (menubar)
+	gnome_app_set_menus(GNOME_APP(wnd), GTK_MENU_BAR(menubar));
     /* main vbox */
     main_vbox = gtk_vbox_new(FALSE, 1);
 
@@ -582,10 +572,9 @@ bab_window_new()
     gtk_window_set_default_size(GTK_WINDOW(wnd), 500, 400);
     gnome_app_set_contents(GNOME_APP(wnd), main_vbox);
 
-    first_ab = g_list_first(contacts_app.address_book_list);
-    if(first_ab)
-        bab_set_address_book(LIBBALSA_ADDRESS_BOOK(first_ab->data),
-                             contacts_app.entry_list, NULL);
+    if (contacts_app.first_radio_action)
+        gtk_action_activate(GTK_ACTION(contacts_app.first_radio_action));
+
     return wnd;
 }
 
@@ -640,10 +629,7 @@ main(int argc, char *argv[])
     bindtextdomain(PACKAGE, GNOMELOCALEDIR);
     bind_textdomain_codeset(PACKAGE, "UTF-8");
     textdomain(PACKAGE);
-    /* FIXME: gnome_i18n_get_language seems to have gone away; 
-     * is this a reasonable replacement? */
-    setlocale(LC_CTYPE,
-              (const char *) gnome_i18n_get_language_list("LC_CTYPE")->data);
+    setlocale(LC_ALL, "");
 #endif
 
     /* FIXME: do we need to allow a non-GUI mode? */
@@ -663,7 +649,9 @@ main(int argc, char *argv[])
     bab_init();
 
     /* load address book data */
-    bab_config_init();
+    libbalsa_conf_foreach_group(ADDRESS_BOOK_SECTION_PREFIX,
+                                bab_config_init,
+                                &contacts_app.address_book_list);
 
     ab_window = bab_window_new();
     g_signal_connect(G_OBJECT(ab_window), "destroy",
