@@ -168,13 +168,9 @@ extern struct BalsaApplication {
     GSList* filters;
 
     gchar *local_mail_directory;
+
 #if ENABLE_ESMTP
-    gchar *smtp_server;
-    gchar *smtp_user;
-    gchar *smtp_passphrase;
-    auth_context_t smtp_authctx;
-    gint smtp_tls_mode;
-    gchar *smtp_certificate_passphrase;
+    GSList *smtp_servers;
 #endif
 
     /* folder scanning */
@@ -332,7 +328,6 @@ extern struct BalsaApplication {
     gchar *subject_font;
 
     /* encoding stuff */
-    guint encoding_style;
     gchar *date_string;
 
     /* printing */
@@ -397,8 +392,11 @@ extern struct BalsaApplication {
     gboolean has_smime;
 #endif 
 
+    /* Most recently used lists */
     GList *folder_mru;
     GList *fcc_mru;
+    GList *pipe_cmds;
+
     gboolean expunge_on_close;
 
     /* use as default email client for GNOME */
