@@ -1342,15 +1342,15 @@ balsa_index_set_column_widths(BalsaIndex * index)
                                          (tree_view, LB_MBOX_SIZE_COL),
                                          40); /* get a better guess */ 
 #endif
-    /* I have no idea why we must add 3 pixels to the icon width - otherwise,
+    /* I have no idea why we must add 5 pixels to the icon width - otherwise,
        the icon will be clipped... */
     gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &icon_w, &icon_h);
     gtk_tree_view_column_set_fixed_width(gtk_tree_view_get_column
                                          (tree_view, LB_MBOX_MARKED_COL),
-                                         icon_w + 3); 
+                                         icon_w + 5); 
     gtk_tree_view_column_set_fixed_width(gtk_tree_view_get_column
                                          (tree_view, LB_MBOX_ATTACH_COL),
-                                         icon_w + 3);
+                                         icon_w + 5);
     gtk_tree_view_column_set_fixed_width(gtk_tree_view_get_column
                                          (tree_view, LB_MBOX_FROM_COL),
                                          balsa_app.index_from_width);
@@ -1763,6 +1763,12 @@ mru_menu_cb(gchar * url, BalsaIndex * index)
 /*
  * bndx_popup_menu_create: create the popup menu at init time
  */
+static void
+bndx_pipe_message(GtkWidget *widget, gpointer user_data)
+{
+    balsa_index_pipe(BALSA_INDEX(user_data));
+}
+
 static GtkWidget *
 bndx_popup_menu_create(BalsaIndex * index)
 {
@@ -1781,6 +1787,8 @@ bndx_popup_menu_create(BalsaIndex * index)
                 GTK_SIGNAL_FUNC(balsa_message_forward_attached)}, {
         BALSA_PIXMAP_FORWARD, N_("Forward _Inline..."),
                 GTK_SIGNAL_FUNC(balsa_message_forward_inline)}, {
+        NULL,                 N_("_Pipe through..."),
+                GTK_SIGNAL_FUNC(bndx_pipe_message)}, {
         BALSA_PIXMAP_BOOK_RED, N_("_Store Address..."),
                 GTK_SIGNAL_FUNC(bndx_store_address)}};
     GtkWidget *menu, *menuitem, *submenu;
@@ -2289,3 +2297,160 @@ balsa_index_previous_msgno(BalsaIndex * index, guint current_msgno)
     return bndx_next_msgno(index, current_msgno, index->search_iter,
                            BNDX_SEARCH_DIRECTION_PREV);
 }
+
+/* Pipe commands. */
+
+static void
+bndx_pipe(LibBalsaMailbox * mailbox, guint msgno, const gchar * pipe_cmd)
+{
+    FILE *fprog;
+
+    if ((fprog = popen(pipe_cmd, "w")) == 0) {
+        fprintf(stderr, "popen failed for message %d\n", msgno);
+    } else {
+        GMimeStream *stream =
+            libbalsa_mailbox_get_message_stream(mailbox, msgno);
+        GMimeStream *pipe;
+
+        g_return_if_fail(stream);
+
+        pipe = g_mime_stream_file_new(fprog);
+        g_mime_stream_file_set_owner(GMIME_STREAM_FILE(pipe), FALSE);
+        g_mime_stream_write_to_stream(stream, pipe);
+        g_object_unref(pipe);
+        g_object_unref(stream);
+        if (pclose(fprog) == -1)
+            fprintf(stderr, "pclose failed for message %d\n", msgno);
+    }
+}
+
+struct bndx_mailbox_info {
+    GtkWidget *dialog;
+    GtkWidget *entry;
+    LibBalsaMailbox *mailbox;
+    GArray *msgnos;
+};
+
+static void
+bndx_mailbox_notify(gpointer data)
+{
+    struct bndx_mailbox_info *info = data;
+
+    gtk_widget_destroy(info->dialog);
+    g_array_free(info->msgnos, TRUE);
+    g_free(info);
+}
+
+#define BALSA_INDEX_PIPE_INFO "balsa-index-pipe-info"
+
+static void
+bndx_pipe_response(GtkWidget * dialog, gint response,
+                   struct bndx_mailbox_info *info)
+{
+    LibBalsaMailbox *mailbox = info->mailbox;
+
+    g_object_add_weak_pointer(G_OBJECT(mailbox), (gpointer) & mailbox);
+
+    if (response == GTK_RESPONSE_OK) {
+        gchar *pipe_cmd;
+        GList *active_cmd;
+        guint i;
+
+#if GTK_CHECK_VERSION(2, 6, 0)
+        pipe_cmd =
+            gtk_combo_box_get_active_text(GTK_COMBO_BOX(info->entry));
+#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
+        pipe_cmd =
+            gtk_editable_get_chars(GTK_EDITABLE
+                                   (GTK_BIN(info->entry)->child), 0, -1);
+#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
+        active_cmd =
+            g_list_find_custom(balsa_app.pipe_cmds, pipe_cmd,
+                               (GCompareFunc) strcmp);
+        if (!active_cmd)
+            balsa_app.pipe_cmds =
+                g_list_prepend(balsa_app.pipe_cmds, g_strdup(pipe_cmd));
+        else if (active_cmd != balsa_app.pipe_cmds) {
+            balsa_app.pipe_cmds =
+                g_list_remove_link(balsa_app.pipe_cmds, active_cmd);
+            balsa_app.pipe_cmds =
+                g_list_concat(active_cmd, balsa_app.pipe_cmds);
+        }
+
+        for (i = 0; i < info->msgnos->len && mailbox; i++)
+            bndx_pipe(mailbox, g_array_index(info->msgnos, guint, i),
+                      pipe_cmd);
+        g_free(pipe_cmd);
+    }
+
+    if (!mailbox)
+        return;
+    g_object_remove_weak_pointer(G_OBJECT(mailbox), (gpointer) & mailbox);
+
+    libbalsa_mailbox_unregister_msgnos(mailbox, info->msgnos);
+    libbalsa_mailbox_close(mailbox, balsa_app.expunge_on_close);
+    g_object_set_data(G_OBJECT(mailbox), BALSA_INDEX_PIPE_INFO, NULL);
+}
+
+#define HIG_PADDING 6
+
+void
+balsa_index_pipe(BalsaIndex * index)
+{
+    struct bndx_mailbox_info *info;
+    GtkWidget *label, *entry;
+    GtkWidget *dialog;
+    GtkWidget *vbox;
+    GList *list;
+
+    g_return_if_fail(BALSA_IS_INDEX(index));
+    g_return_if_fail(BALSA_IS_MAILBOX_NODE(index->mailbox_node));
+    g_return_if_fail(LIBBALSA_IS_MAILBOX(index->mailbox_node->mailbox));
+
+    info =
+        g_object_get_data(G_OBJECT(index->mailbox_node->mailbox),
+                          BALSA_INDEX_PIPE_INFO);
+    if (info) {
+        gdk_window_raise(info->dialog->window);
+        return;
+    }
+
+    info = g_new(struct bndx_mailbox_info, 1);
+    info->mailbox = index->mailbox_node->mailbox;
+    g_object_set_data_full(G_OBJECT(info->mailbox), BALSA_INDEX_PIPE_INFO,
+                           info, bndx_mailbox_notify);
+    libbalsa_mailbox_open(info->mailbox, NULL);
+
+    info->msgnos = g_array_sized_new(FALSE, FALSE, sizeof(guint),
+                                     index->selected->len);
+    g_array_append_vals(info->msgnos, index->selected->data,
+                        index->selected->len);
+    libbalsa_mailbox_register_msgnos(info->mailbox, info->msgnos);
+
+    info->dialog = dialog =
+        gtk_dialog_new_with_buttons(_("Pipe message through a program"),
+                                    GTK_WINDOW(balsa_app.main_window),
+                                    GTK_DIALOG_DESTROY_WITH_PARENT,
+                                    _("_Run"), GTK_RESPONSE_OK,
+                                    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                    NULL);
+
+    vbox = GTK_DIALOG(dialog)->vbox;
+    gtk_box_set_spacing(GTK_BOX(vbox), HIG_PADDING);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), HIG_PADDING);
+    gtk_container_add(GTK_CONTAINER(vbox), label =
+                      gtk_label_new(_("Specify the program to run:")));
+
+    info->entry = entry = gtk_combo_box_entry_new_text();
+    for (list = balsa_app.pipe_cmds; list; list = list->next)
+        gtk_combo_box_append_text(GTK_COMBO_BOX(entry), list->data);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(entry), 0);
+    gtk_container_add(GTK_CONTAINER(vbox), entry);
+
+    gtk_widget_show(label);
+    gtk_widget_show(entry);
+    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
+    g_signal_connect(dialog, "response", G_CALLBACK(bndx_pipe_response),
+                     info);
+    gtk_widget_show(dialog);
+}
