@@ -55,7 +55,7 @@ struct _FolderDialogData {
     GtkWidget *folder_name, *port, *username, *remember,
         *password, *subscribed, *list_inbox, *prefix;
     GtkWidget *use_ssl, *tls_mode;
-    GtkWidget *connection_limit, *enable_persistent, *has_bugs;
+    GtkWidget *connection_limit, *enable_persistent, *has_bugs, *use_status;
 };
 
 /* FIXME: identity_name will leak on cancelled folder edition */
@@ -190,6 +190,9 @@ folder_conf_clicked_ok(FolderDialogData * fcw)
     libbalsa_imap_server_set_bug
         (LIBBALSA_IMAP_SERVER(s), ISBUG_FETCH,
          gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->has_bugs)));
+    libbalsa_imap_server_set_use_status
+        (LIBBALSA_IMAP_SERVER(s),
+         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->use_status)));
     libbalsa_server_set_username(s, username);
     s->remember_passwd =
         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->remember));
@@ -319,6 +322,13 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
        libbalsa_imap_server_has_bug(LIBBALSA_IMAP_SERVER(s), ISBUG_FETCH))
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fcw->has_bugs),
                                      TRUE);
+    fcw->use_status = 
+        balsa_server_conf_add_checkbox(&fcw->bsc,
+                                       _("Use STATUS for mailbox checking"));
+    if(s &&
+       libbalsa_imap_server_get_use_status(LIBBALSA_IMAP_SERVER(s)))
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fcw->use_status),
+                                     TRUE);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), advanced,
                              gtk_label_new_with_mnemonic(_("_Advanced")));
 
@@ -391,8 +401,7 @@ validate_sub_folder(GtkWidget * w, SubfolderDialogData * sdd)
      * Allow typing in the parent_folder entry box only if we already
      * have the server information in mn:
      */
-    gboolean have_server = (mn && mn->server
-			    && mn->server->type == LIBBALSA_SERVER_IMAP);
+    gboolean have_server = (mn && LIBBALSA_IS_IMAP_SERVER(mn->server));
     gtk_editable_set_editable(GTK_EDITABLE(sdd->parent_folder),
 			      have_server);
     /*
@@ -472,8 +481,7 @@ folder_selection_func(GtkTreeSelection * selection, GtkTreeModel * model,
 
     gtk_tree_model_get_iter(model, &iter, path);
     gtk_tree_model_get(model, &iter, 0, &mbnode, -1);
-    retval = (mbnode->server
-	      && mbnode->server->type == LIBBALSA_SERVER_IMAP
+    retval = (LIBBALSA_IS_IMAP_SERVER(mbnode->server)
 	      && (sdd->mbnode == NULL
 		  || sdd->mbnode->server == mbnode->server));
     g_object_unref(mbnode);
