@@ -29,9 +29,13 @@
 #include <gnome.h>
 
 #include "libbalsa.h"
+#include "i18n.h"
 #include "balsa-app.h"
 #include "information-dialog.h"
 
+static void balsa_information_bar(GtkWindow *parent,
+                                  LibBalsaInformationType type,
+                                  const char *msg);
 static void balsa_information_list(GtkWindow *parent,
                                    LibBalsaInformationType type,
 				   const char *msg);
@@ -61,6 +65,9 @@ balsa_information_real(GtkWindow *parent, LibBalsaInformationType type,
                        const char *msg)
 {
     BalsaInformationShow show;
+    gchar * show_msg = g_strdup(msg);
+    
+    libbalsa_utf8_sanitize(&show_msg, balsa_app.convert_unknown_8bit, NULL);
     switch (type) {
     case LIBBALSA_INFORMATION_MESSAGE:
 	show = balsa_app.information_message;
@@ -84,15 +91,19 @@ balsa_information_real(GtkWindow *parent, LibBalsaInformationType type,
     case BALSA_INFORMATION_SHOW_NONE:
 	break;
     case BALSA_INFORMATION_SHOW_DIALOG:
-	balsa_information_dialog(parent, type, msg);
+	balsa_information_dialog(parent, type, show_msg);
 	break;
     case BALSA_INFORMATION_SHOW_LIST:
-	balsa_information_list(parent, type, msg);
+	balsa_information_list(parent, type, show_msg);
+	break;
+    case BALSA_INFORMATION_SHOW_BAR:
+	balsa_information_bar(parent, type, show_msg);
 	break;
     case BALSA_INFORMATION_SHOW_STDERR:
-	balsa_information_stderr(type, msg);
+	balsa_information_stderr(type, show_msg);
 	break;
     }
+    g_free(show_msg);
 
     if (type == LIBBALSA_INFORMATION_FATAL)
 	gtk_main_quit();
@@ -136,10 +147,6 @@ balsa_information_dialog(GtkWindow *parent, LibBalsaInformationType type,
 {
     GtkMessageType message_type;
     GtkWidget *messagebox;
-    gchar * real_msg = g_strdup(msg);
-
-    libbalsa_utf8_sanitize(&real_msg, balsa_app.convert_unknown_8bit,
-			   balsa_app.convert_unknown_8bit_codeset, NULL);
 
     switch (type) {
     case LIBBALSA_INFORMATION_MESSAGE:
@@ -162,11 +169,13 @@ balsa_information_dialog(GtkWindow *parent, LibBalsaInformationType type,
         break;
     }
 
+    /* Use explicit format so that message_dialog does not try to interpret
+     * the message string. */
     messagebox =
         gtk_message_dialog_new(GTK_WINDOW(parent),
                                GTK_DIALOG_DESTROY_WITH_PARENT,
-                               message_type, GTK_BUTTONS_CLOSE, real_msg);
-    g_free(real_msg);
+                               message_type, GTK_BUTTONS_CLOSE,
+                               "%s", msg);
 
     gtk_dialog_run(GTK_DIALOG(messagebox));
     gtk_widget_destroy(messagebox);
@@ -203,10 +212,6 @@ balsa_information_list(GtkWindow *parent, LibBalsaInformationType type,
     static GtkWidget *information_list = NULL;
     GtkTextBuffer *buffer;
     GtkTextIter iter;
-    gchar * real_msg = g_strdup(msg);
-
-    libbalsa_utf8_sanitize(&real_msg, balsa_app.convert_unknown_8bit,
-			   balsa_app.convert_unknown_8bit_codeset, NULL);
 
     if (information_list == NULL) {
 	GtkWidget *information_dialog;
@@ -259,8 +264,7 @@ balsa_information_list(GtkWindow *parent, LibBalsaInformationType type,
     gtk_text_buffer_place_cursor(buffer, &iter);
     if (gtk_text_buffer_get_char_count(buffer))
         gtk_text_buffer_insert_at_cursor(buffer, "\n", 1);
-    gtk_text_buffer_insert_at_cursor(buffer, real_msg, -1);
-    g_free(real_msg);
+    gtk_text_buffer_insert_at_cursor(buffer, msg, -1);
     gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(information_list),
                                  gtk_text_buffer_get_insert(buffer),
                                  0, FALSE, 0, 0);
@@ -269,6 +273,35 @@ balsa_information_list(GtkWindow *parent, LibBalsaInformationType type,
 	gnome_appbar_set_status(balsa_app.appbar, msg);
 }
 
+static guint bar_timeout_id = 0;
+static unsigned push_cnt = 0;
+static gboolean
+status_bar_refresh(gpointer data)
+{
+    gdk_threads_enter();
+    if (balsa_app.appbar) {
+        gnome_appbar_pop(balsa_app.appbar);
+        push_cnt--;
+        if(push_cnt == 0)
+            bar_timeout_id = 0; /* FIXME: thread locking here! */
+    }
+    gdk_threads_leave();
+    return push_cnt != 0;
+}
+
+static void
+balsa_information_bar(GtkWindow *parent, LibBalsaInformationType type,
+                      const char *msg)
+{
+    if (!balsa_app.appbar)
+        return;
+    /* we used to have _set_status() here but it got over-written by
+       set_default(). _push() seems to be more persistent. */
+    gnome_appbar_push(balsa_app.appbar, msg); push_cnt++;
+    if(!bar_timeout_id)
+        bar_timeout_id = g_timeout_add(4000, status_bar_refresh, NULL);
+}
+
 static void 
 balsa_information_stderr(LibBalsaInformationType type, const char *msg)
 {
