@@ -1320,12 +1320,9 @@ mailbox_conf_view_new(LibBalsaMailbox * mailbox,
     for (list = balsa_app.identities, active = 0; list;
          list = list->next, ++active) {
         LibBalsaIdentity *ident = list->data;
-        gchar *from;
         gchar *name;
 
-        from = internet_address_to_string(ident->ia, FALSE);
-        name = g_strconcat(ident->identity_name, " (", from, ")", NULL);
-        g_free(from);
+        name = internet_address_to_string(ident->ia, FALSE);
         gtk_combo_box_append_text(GTK_COMBO_BOX
                                   (view_info->identity_combo_box), name);
         g_free(name);
@@ -1448,10 +1445,18 @@ mailbox_conf_view_check(BalsaMailboxConfView * view_info,
     if (!changed || !libbalsa_mailbox_get_open(mailbox))
 	return;
 
-    /* Redraw the mailbox; we temporarily increase its open_ref to keep
-     * the backend open. */
-    libbalsa_mailbox_open(mailbox, NULL);
-    balsa_mblist_close_mailbox(mailbox);
-    balsa_mblist_open_mailbox(mailbox);
-    libbalsa_mailbox_close(mailbox, FALSE);
+    /* Redraw the mailbox if it is open already - we MUST NOT attempt
+     * opening closed mailboxes for both performance and security
+     * reasons. Performance is obvious. Security is relevant here too:
+     * the user might have realized that the password must be sent
+     * encrypted and clicked on "Use SSL". we should not attempt to
+     * open the connection with old settings requesting unencrypted
+     * connection. We temporarily increase its open_ref to keep the
+     * backend open. */
+    if(MAILBOX_OPEN(mailbox)) {
+        libbalsa_mailbox_open(mailbox, NULL);
+        balsa_mblist_close_mailbox(mailbox);
+        balsa_mblist_open_mailbox(mailbox);
+        libbalsa_mailbox_close(mailbox, FALSE);
+    }
 }
