@@ -23,8 +23,6 @@
 
 #if defined(BALSA_USE_THREADS) && defined(THREADED_IMAP_SCAN_FIXED)
 #include <pthread.h>
-/* for sched_yield() prototype */
-#include <sched.h>
 #endif
 
 #include <unistd.h>
@@ -62,13 +60,13 @@ static void balsa_mailbox_node_dispose(GObject * object);
 static void balsa_mailbox_node_finalize(GObject * object);
 
 static void balsa_mailbox_node_real_save_config(BalsaMailboxNode* mn,
-						const gchar * prefix);
+						const gchar * group);
 static void balsa_mailbox_node_real_load_config(BalsaMailboxNode* mn,
-						const gchar * prefix);
+						const gchar * group);
 
 static BalsaMailboxNode *imap_scan_create_mbnode(BalsaMailboxNode * root,
 						 imap_scan_item * isi,
-						 char delim);
+						 int delim);
 static gboolean imap_scan_attach_mailbox(BalsaMailboxNode * mbnode,
                                          imap_scan_item * isi);
 static gboolean bmbn_scan_children_idle(BalsaMailboxNode ** mn);
@@ -85,8 +83,7 @@ static void handle_imap_path(const char *fn, char delim, int noselect,
 			     int noscan, int marked, void *data);
 static gint check_imap_path(const char *fn, LibBalsaServer * server,
 			    guint depth);
-static void mark_imap_path(const gchar * fn, gint noselect, gint noscan,
-			   gpointer data);
+static void mark_imap_path(const gchar * fn, gpointer data);
 
 enum {
     SAVE_CONFIG,
@@ -230,10 +227,10 @@ balsa_mailbox_node_finalize(GObject * object)
 }
 
 static void
-balsa_mailbox_node_real_save_config(BalsaMailboxNode* mn, const gchar * prefix)
+balsa_mailbox_node_real_save_config(BalsaMailboxNode* mn, const gchar * group)
 {
     if(mn->name)
-	printf("Saving mailbox node %s with prefix %s\n", mn->name, prefix);
+	printf("Saving mailbox node %s with group %s\n", mn->name, group);
     libbalsa_imap_server_save_config(LIBBALSA_IMAP_SERVER(mn->server));
     libbalsa_conf_set_string("Name",      mn->name);
     libbalsa_conf_set_string("Directory", mn->dir);
@@ -241,14 +238,14 @@ balsa_mailbox_node_real_save_config(BalsaMailboxNode* mn, const gchar * prefix)
     libbalsa_conf_set_bool("ListInbox",   mn->list_inbox);
     
     g_free(mn->config_prefix);
-    mn->config_prefix = g_strdup(prefix);
+    mn->config_prefix = g_strdup(group);
 }
 
 static void
-balsa_mailbox_node_real_load_config(BalsaMailboxNode* mn, const gchar * prefix)
+balsa_mailbox_node_real_load_config(BalsaMailboxNode* mn, const gchar * group)
 {
     g_free(mn->config_prefix);
-    mn->config_prefix = g_strdup(prefix);
+    mn->config_prefix = g_strdup(group);
     g_free(mn->name);
     mn->name = libbalsa_conf_get_string("Name");
 }
@@ -420,9 +417,8 @@ imap_dir_cb_real(void* r)
     imap_scan_tree imap_tree = { NULL, '.' };
 
     g_return_val_if_fail(mb->server, NULL);
-
-    libbalsa_scanner_imap_dir(mb, mb->server, mb->dir, mb->subscribed,
-                              mb->list_inbox, 
+    libbalsa_scanner_imap_dir(mb, mb->server, mb->dir, mb->delim,
+			      mb->subscribed, mb->list_inbox, 
                               check_imap_path,
                               mark_imap_path,
                               handle_imap_path,
@@ -537,10 +533,10 @@ balsa_mailbox_node_new_from_dir(const gchar* dir)
    similiar to mailbox creation from configuration data.
 */
 BalsaMailboxNode*
-balsa_mailbox_node_new_from_config(const gchar* prefix)
+balsa_mailbox_node_new_from_config(const gchar* group)
 {
     BalsaMailboxNode * folder = balsa_mailbox_node_new();
-    libbalsa_conf_push_prefix(prefix);
+    libbalsa_conf_push_group(group);
 
     folder->server = LIBBALSA_SERVER(libbalsa_imap_server_new_from_config());
 
@@ -553,14 +549,14 @@ balsa_mailbox_node_new_from_config(const gchar* prefix)
 		     G_CALLBACK(imap_dir_cb), NULL);
     libbalsa_server_connect_signals(folder->server,
                                     G_CALLBACK(ask_password), NULL);
-    balsa_mailbox_node_load_config(folder, prefix);
+    balsa_mailbox_node_load_config(folder, group);
 
     folder->dir = libbalsa_conf_get_string("Directory");
     folder->subscribed =
 	libbalsa_conf_get_bool("Subscribed"); 
     folder->list_inbox =
 	libbalsa_conf_get_bool("ListInbox=true"); 
-    libbalsa_conf_pop_prefix();
+    libbalsa_conf_pop_group();
 
     return folder;
 }
@@ -632,17 +628,17 @@ balsa_mailbox_node_show_prop_dialog_cb(GtkWidget * widget, gpointer data)
    with some sane defaults.
 */
 void
-balsa_mailbox_node_load_config(BalsaMailboxNode* mn, const gchar* prefix)
+balsa_mailbox_node_load_config(BalsaMailboxNode* mn, const gchar* group)
 {
     g_signal_emit(G_OBJECT(mn),
-		  balsa_mailbox_node_signals[LOAD_CONFIG], 0, prefix);
+		  balsa_mailbox_node_signals[LOAD_CONFIG], 0, group);
 }
 
 void
-balsa_mailbox_node_save_config(BalsaMailboxNode* mn, const gchar* prefix)
+balsa_mailbox_node_save_config(BalsaMailboxNode* mn, const gchar* group)
 {
     g_signal_emit(G_OBJECT(mn),
-		  balsa_mailbox_node_signals[SAVE_CONFIG], 0, prefix);
+		  balsa_mailbox_node_signals[SAVE_CONFIG], 0, group);
 }
 
 /* ---------------------------------------------------------------------
@@ -771,19 +767,25 @@ bmbn_scan_children_idle(BalsaMailboxNode ** mbnode)
     list = g_object_get_data(G_OBJECT(*mbnode), BALSA_MAILBOX_NODE_LIST_KEY);
     for (l = list; l; l = g_slist_next(l)) {
         BalsaMailboxNode *mn;
-        gboolean has_unread_messages = FALSE;
         
         if (!l->data)
             continue;
         mn = l->data;
-        if (mn->mailbox)
-            has_unread_messages = mn->mailbox->has_unread_messages;
-        balsa_mailbox_node_rescan(mn);
-        if (!l->data)
-            continue;
-        if (mn->mailbox)
-            mn->mailbox->has_unread_messages = has_unread_messages;
-        mn->scanned = TRUE;
+        if (!mn->scanned) {
+            gboolean has_unread_messages = FALSE;
+
+            if (mn->mailbox)
+                has_unread_messages = mn->mailbox->has_unread_messages;
+            balsa_mailbox_node_rescan(mn);
+            if (!l->data)
+                continue;
+            if (mn->mailbox)
+                mn->mailbox->has_unread_messages = has_unread_messages;
+            mn->scanned = TRUE;
+        } else if (balsa_app.debug)
+            g_print("%s: %s \"%s\" was already scanned\n", __func__,
+                    mn->mailbox ? "mailbox" : "folder",
+                    mn->mailbox ? mn->mailbox->name : mn->name);
         g_object_remove_weak_pointer(G_OBJECT(mn), & l->data);
     }
     g_slist_free(list);
@@ -911,12 +913,20 @@ mb_filter_cb(GtkWidget * widget, BalsaMailboxNode * mbnode)
 	g_print("You can apply filters only on mailbox\n");
 }
 
+static void
+mb_empty_trash_cb(GtkWidget * widget, BalsaMailboxNode * mbnode)
+{
+    empty_trash(balsa_app.main_window);
+}
+
 GtkWidget *
 balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
 {
     GtkWidget *menu;
     GtkWidget *submenu;
     GtkWidget *menuitem;
+    LibBalsaMailbox *mailbox;
+    gboolean special;
 
     /*  g_return_val_if_fail(mailbox != NULL, NULL); */
 
@@ -953,57 +963,66 @@ balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
 	return menu;
     }
     /* If we didn't click on a mailbox node then there is only one option. */
-    add_menu_entry(menu, NULL, NULL, mbnode);
+    add_menu_entry(menu, NULL, NULL, NULL);
 
-    if (mbnode->mailbox) {
-	if (!MAILBOX_OPEN(mbnode->mailbox))
-	    add_menu_entry(menu, _("_Open"),
-                           G_CALLBACK(mb_open_cb), mbnode);
-	else
-	    add_menu_entry(menu, _("_Close"),
-                           G_CALLBACK(mb_close_cb), mbnode);
-    }
     if (g_signal_has_handler_pending(G_OBJECT(mbnode),
                                      balsa_mailbox_node_signals
                                      [SHOW_PROP_DIALOG], 0, FALSE))
         add_menu_entry(menu, _("_Properties..."),
                        G_CALLBACK(mb_conf_cb), mbnode);
-    if (mbnode->mailbox || mbnode->config_prefix)
-	add_menu_entry(menu, _("_Delete"), 
-                       G_CALLBACK(mb_del_cb),  mbnode);
+
     if (g_signal_has_handler_pending(G_OBJECT(mbnode),
 		                     balsa_mailbox_node_signals
 				     [APPEND_SUBTREE], 0, FALSE))
 	add_menu_entry(menu, _("_Rescan"),
 		       GTK_SIGNAL_FUNC(mb_rescan_cb), mbnode);
 
-    
-    if (mbnode->mailbox) {
-	add_menu_entry(menu, NULL, NULL, mbnode);
-	if(LIBBALSA_IS_MAILBOX_IMAP(mbnode->mailbox)) {
-	    add_menu_entry(menu, _("_Subscribe"),   
-                           GTK_SIGNAL_FUNC(mb_subscribe_cb),   mbnode);
-	    add_menu_entry(menu, _("_Unsubscribe"), 
-                           GTK_SIGNAL_FUNC(mb_unsubscribe_cb), mbnode);
-	}
-	
-	if(mbnode->mailbox != balsa_app.inbox)
-	    add_menu_entry(menu, _("Mark as _Inbox"),    
-                           GTK_SIGNAL_FUNC(mb_inbox_cb),    mbnode);
-	if(mbnode->mailbox != balsa_app.sentbox)
-	    add_menu_entry(menu, _("_Mark as Sentbox"), 
-                           GTK_SIGNAL_FUNC(mb_sentbox_cb),  mbnode);
-	if(mbnode->mailbox != balsa_app.trash)
-	    add_menu_entry(menu, _("Mark as _Trash"),    
-                           GTK_SIGNAL_FUNC(mb_trash_cb),    mbnode);
-	if(mbnode->mailbox != balsa_app.draftbox)
-	    add_menu_entry(menu, _("Mark as D_raftbox"),
-                           GTK_SIGNAL_FUNC(mb_draftbox_cb), mbnode);
-	/* FIXME : No test on mailbox type is made yet, should we ? */
-	add_menu_entry(menu, _("_Edit/Apply filters"), 
-                       GTK_SIGNAL_FUNC(mb_filter_cb), mbnode);
+    if (mbnode->config_prefix)
+	add_menu_entry(menu, _("_Delete"), G_CALLBACK(mb_del_cb),  mbnode);
+
+    if (!(mailbox = mbnode->mailbox))
+        return menu;
+
+    if (!MAILBOX_OPEN(mailbox))
+        add_menu_entry(menu, _("_Open"),  G_CALLBACK(mb_open_cb),  mbnode);
+    else
+        add_menu_entry(menu, _("_Close"), G_CALLBACK(mb_close_cb), mbnode);
+
+    special = (   mailbox == balsa_app.inbox
+               || mailbox == balsa_app.sentbox
+               || mailbox == balsa_app.draftbox
+               || mailbox == balsa_app.outbox
+               || mailbox == balsa_app.trash);
+    if (!special && !mbnode->config_prefix)
+	add_menu_entry(menu, _("_Delete"), G_CALLBACK(mb_del_cb),  mbnode);
+
+    if(LIBBALSA_IS_MAILBOX_IMAP(mailbox)) {
+        add_menu_entry(menu, NULL, NULL, NULL);
+        add_menu_entry(menu, _("_Subscribe"),   
+                       GTK_SIGNAL_FUNC(mb_subscribe_cb),   mbnode);
+        add_menu_entry(menu, _("_Unsubscribe"), 
+                       GTK_SIGNAL_FUNC(mb_unsubscribe_cb), mbnode);
     }
 
+    if (!special) {
+        add_menu_entry(menu, NULL, NULL, NULL);
+        add_menu_entry(menu, _("Mark as _Inbox"),    
+                       GTK_SIGNAL_FUNC(mb_inbox_cb),    mbnode);
+        add_menu_entry(menu, _("_Mark as Sentbox"), 
+                       GTK_SIGNAL_FUNC(mb_sentbox_cb),  mbnode);
+        add_menu_entry(menu, _("Mark as _Trash"),    
+                       GTK_SIGNAL_FUNC(mb_trash_cb),    mbnode);
+        add_menu_entry(menu, _("Mark as D_raftbox"),
+                       GTK_SIGNAL_FUNC(mb_draftbox_cb), mbnode);
+    } else if (mailbox == balsa_app.trash)
+        add_menu_entry(menu, _("_Empty trash"),    
+                       GTK_SIGNAL_FUNC(mb_empty_trash_cb), mbnode);
+
+    /* FIXME : No test on mailbox type is made yet, should we ? */
+    add_menu_entry(menu, NULL, NULL, NULL);
+    add_menu_entry(menu, _("_Edit/Apply filters"), 
+                   GTK_SIGNAL_FUNC(mb_filter_cb), mbnode);
+
     return menu;
 }
 
@@ -1147,7 +1166,7 @@ get_parent_folder_name(const gchar* path, char delim)
  */
 static BalsaMailboxNode *
 imap_scan_create_mbnode(BalsaMailboxNode * root, imap_scan_item * isi,
-			char delim)
+			int delim)
 {
     gchar *parent_name;
     BalsaMailboxNode *mbnode;
@@ -1189,6 +1208,7 @@ imap_scan_create_mbnode(BalsaMailboxNode * root, imap_scan_item * isi,
     else
         basename++;
     mbnode->name = g_strdup(basename);
+    mbnode->delim = delim;
     mbnode->parent = parent;
     mbnode->subscribed = parent->subscribed;
     mbnode->scanned = isi->scanned;
@@ -1286,15 +1306,10 @@ check_imap_path(const gchar *fn, LibBalsaServer * server, guint depth)
 
 /* mark_imap_path:
  *
- * find the imap_scan_item for fn and set flags.
- *
- * noselect may be -1, 0, or 1; -1 means no change, 0 or 1
- * mean set accordingly.
- *
- * noscan is always set
+ * find the imap_scan_item for fn and set scanned.
  */
 static void
-mark_imap_path(const gchar * fn, gint noselect, gint noscan, gpointer data)
+mark_imap_path(const gchar * fn, gpointer data)
 {
     imap_scan_tree *tree = data;
     GSList *list;
@@ -1304,11 +1319,7 @@ mark_imap_path(const gchar * fn, gint noselect, gint noscan, gpointer data)
     for (list = tree->list; list; list = list->next) {
         imap_scan_item *item = list->data;
         if (!strcmp(item->fn, fn)) {
-            if (noselect >= 0)
-        	item->selectable = !noselect;
-            item->scanned = noscan;
-            if (balsa_app.debug)
-        	printf(" noselect %d, noscan %d.\n", noselect, noscan);
+            item->scanned = TRUE;
             break;
         }
     }
